/* Example program to show information about .TAF files
 *
 * This program is in the Public Domain (PD) 2004, Rik Snel
 */
#include <config.h>
#include <stdio.h>
#include <stdlib.h>
#include <getopt.h>
#include <string.h>
#include "../src/exception.h"
#include "../src/verbose.h"
#include "../src/taf/generic.h"

extern char *optarg;
extern int optind, opterr, optopt;

int main(int argc, char **argv) {	
	la_taf_t taf;
	int opt, nowarn = 0;

	la_verbose_init_fp_quiet(stdout);

	if (atexit(la_verbose_free)) {
		fprintf(stderr, "fatal error\n");
		exit(1);
	}
	
	while ((opt = getopt(argc, argv, "vdDihq")) != -1) {
		if (opt == '?' || opt == ':') exit(1);
		switch (opt) {
			case 'q':
				nowarn = 1;
				break;
			case 'v':
				la_verbose_channel_add(Taf_Parser, Verbose, 
						&la_verbose_fatal_channel);
				break;
			case 'd':
				la_verbose_channel_add(Taf_Parser, Debug, 
						&la_verbose_fatal_channel);
				break;
			case 'D':
				la_verbose_channel_add(Taf_Parser, Dbg2, 
						&la_verbose_fatal_channel);
				break;
			case 'i':
				la_verbose_channel_add(Line_In, Dbg2, 
						&la_verbose_fatal_channel);
				break;
		}
	}

	if (!nowarn) 
		la_verbose_channel_add_alltypes(Warning, 
				&la_verbose_fatal_channel);

	if (argc - optind < 1) {
		fprintf(stderr, 
"This program accepts an (un)obfuscated .TAF file on the commandline\n"
"and it will show some info about it\n"
"\n"
"Usage: %s [-v] [-d] [-D] [-i] [-q] INFILE1 [INFILE2] ...\n", argv[0]);
		exit(1);
	}


	do {
		la_taf_init(&taf);
		try {
			la_exception_cleanup_adda(taf, &la_taf_free);
			la_taf_parse_file(&taf, argv[optind]);
		}
		endtry;
	} while (argc - ++optind > 0);

	exit(0);
}
