/* Example program to show off some of the highlevel capabilities of 
 * libadrift, it recompresses taf 4.00 files to compressionlevel 9
 * using zlib.
 *
 * It works through parsing the complete file before outputting,
 * this version of deflataf only works for 3.80, 3.90 and 4.00 files.
 * The lowlevel version may work for other versions. This highlevel
 * version is also a lot slower.
 *
 * This program is in the Public Domain (PD) 2004, Rik Snel
 */
#include <config.h>
#include <stdio.h>
#include <stdlib.h>
#include "../src/exception.h"
#include "../src/verbose.h"
#include "../src/taf/generic.h"

int main(int argc, char **argv) {	
	la_taf_t taf;

	if (argc != 3) {
		fprintf(stderr, "Usage: %s INFILE OUTFILE\n", argv[0]);
		exit(1);
	}

	la_verbose_init_fp(stderr);
	la_verbose_channel_add(Taf_Parser, Dbg2, &la_verbose_fatal_channel);
	la_verbose_channel_add(Char_IO, Dbg2, &la_verbose_fatal_channel);
	if(atexit(la_verbose_free)) {
		fprintf(stderr, "fatal error\n");
		exit(1);
	}

	la_taf_init(&taf);

	try {
		la_exception_cleanup_adda(taf, &la_taf_free);
		la_taf_parse_file(&taf, argv[1]);

		if (taf.version != 400) {
			fprintf(stderr, "unable to recompress %s, version should be 4.00 but is %.02f.\n", argv[1], taf.version/100.);
			throw();
		}

		la_taf_write_file(&taf, argv[2], LA_TAF_WZLIBHIGH);
	}
	endtry;

	exit(0);
}
