/* Example program to show off some of the capabilities of libadrift,
 * it recomresses TAF 4.00 files with compression level 9
 * 	http://www.phys.uu.nl/~snel/adrift
 *
 * This program is in the Public Domain (PD) 2004, Rik Snel
 */
#include <config.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "../src/adrift.h"
#include "../src/exception.h"
#include "../src/array.h"
#include "../src/verbose.h"
#include "../src/char_io.h"
#include "../src/taf/generic.h"

#define LA_VERBOSE_TYPE Char_IO

int cat4(la_char_io_t *chrin, int version) {
	FILE *fp;
	char buf[14];
	int media_offset, c;
	char *bytes = NULL;
	int no_bytes = 0, i = 0, j, k = 0;
	int xors[14];
	la_array_t a;
	la_char_io_t cout;

	la_array_ninit(&a, bytes, NULL); 
	la_char_io_init(&cout, "temporary buffer");
	la_char_filter_push(&cout, "bufout", &a);

	la_char_filter_command(chrin, 0, "passive", NULL);

	la_char_io_read(buf, 8, chrin); 
	buf[8] = '\0';
	if (sscanf(buf, "%d", &media_offset) != 1) {
		fprintf(stderr, "error reading media offset\n");
		throw();
	}

	la_char_filter_push(chrin, "inflate", NULL);
	c = 9;
	la_char_filter_push(&cout, "deflate", &c);

	try while (1) la_char_io_putc(la_char_io_getc_nowhine(chrin), &cout);
	endtry;

	if (chrin->err != 2) {
		fprintf(stderr, "error decompressing data, probably\n");
		exit(1);
	}
	la_char_filter_pop(chrin);
		
	la_char_io_flush(&cout);
	la_char_io_free(&cout);

	la_char_io_init(&cout, "test.taf");
	
	fp = fopen("test.taf", "w");

	la_char_filter_push(&cout, "fpout", fp);
	la_char_filter_push(&cout, "crypt", NULL);
	la_char_io_printf(&cout, "Version %.02f\r\n", version/100.);
	la_char_io_flush(&cout);
	la_char_filter_command(&cout, 0, "passive", NULL);
	la_char_io_printf(&cout, "%08d", no_bytes + 23);
	la_char_io_write(bytes, no_bytes, &cout);
	while (1) {
		c = la_char_io_getc(chrin);
		if (i >= 14) la_char_io_putc(buf[i%14], &cout);
		buf[i%14] = c;
		la_char_filter_command(chrin, 0, "getXOR", &xors[i%14]);
		i++;
	}

	la_char_io_flush(&cout);
	la_char_filter_command(&cout, 0, "active", NULL);
	
	for (j = i%14; j < 14; j++) 
		 if (k++ < 12) la_char_io_putc(buf[j%14]^xors[j%14], &cout);
	for (j = 0; j < i%14; j++) 
		 if (k++ < 12) la_char_io_putc(buf[j%14]^xors[j%14], &cout);

	la_char_io_flush(&cout);
	la_char_filter_pop(&cout);
	la_char_io_printf(&cout, "\r\n");
	la_char_io_flush(&cout);
	la_char_io_free(&cout);
	fclose(fp);
	return LA_OK;
}

int main(int argc, char **argv) {
	la_taf_t taf;
	int exitval = 0, obf = 1;
	la_char_io_t chrin;

	la_verbose_init_fp(stderr);
	la_char_io_init(&chrin, "standard input");

	try {
		la_char_filter_push(&chrin, "fpin", stdin);
		la_char_filter_push(&chrin, "crypt", NULL);

		la_taf_parse_version(&taf, &chrin, &obf);

		if (!obf) {
			fprintf(stderr, "unsupported format (unobfuscted)");
			throw();
		}
		if (taf.version < 400) {
			fprintf(stderr, "unable to compress files < version 4.00\n");
			throw();
		}

		if (taf.version > 400) 
			fprintf(stderr, "warning: version > 4.00, recompression may fail\n");

		cat4(&chrin, taf.version);
	}
	catch exitval = 1;
	endtry;

	la_char_io_free(&chrin);
	la_verbose_free();
	exit(exitval);
}
