/* Example ADRIFT Runner
 */
#define LA_USE_NCURSES 1
#include <config.h>
#include <stdio.h>
#include <stdlib.h>
#include <getopt.h>
#include <string.h>
#include "../src/exception.h"
#include "../src/verbose.h"
#include "../src/taf/generic.h"
#include "../src/output.h"
#include "../src/game.h"
#include "../src/object.h"
#include "../src/resolve.h"

#ifdef LA_USE_NCURSES
#include <ncurses.h>

typedef struct {
	int (*putchar)(void*, int);
	int (*flush)(void*);
	void *ptr;
} fout_priv_t;

int ncurput(void *ptr, int c) {
	int i = *(int*)ptr;
	addch(c | i);
	return c;
}

int ncurflush(void *ptr) {
	refresh();
	return 0;
}
#endif

void resolvetest(la_tas_t *tas) {
	int i;//, j;
	int *objects = NULL;
	int no_objects = 0;
	la_array_t a;
	la_array_ninit(&a, objects, NULL);
	la_resolve_all_worn_objects(tas, &a);
	for (i = 0; i < no_objects; i++) {
		la_output_red_print(tas, "%s %s ", OF(objects[i])->prefix, OF(objects[i])->name);
	}

#if 0
	for (i = 0; i < NO_OBJECTS; i++) {
		la_resolve_object_roomlocation(tas, &a, i);
		la_output_red_print(tas, "%s %s ", OF(i)->prefix, OF(i)->name);
		if (!no_rooms) la_output_red_print(tas, "hidden");
		for (j = 0; j < no_rooms; j++) {
			la_output_red_print(tas, "%d ", rooms[j]);
		}
		la_output_red_print(tas, "\n");
		la_array_empty(&a);
	}
#endif
}

int main(int argc, char **argv) {	
	la_taf_t taf;
	la_tas_t tas;
	la_char_io_t red;
	la_char_io_t green;
#ifdef LA_USE_NCURSES
	fout_priv_t fprv;
	int red_attr;
	int green_attr;
#endif

	la_verbose_init_fp_quiet(stdout);

	if (atexit(la_verbose_free)) {
		fprintf(stderr, "fatal error\n");
		exit(1);
	}
	
	if (argc < 2) {
		fprintf(stderr, 
"This program accepts an (un)obfuscated .TAF file on the commandline,\n"
"it loads and runs it.\n");
		exit(1);
	}

#ifdef LA_USE_NCURSES
	initscr();
	if (has_colors() == FALSE) {
		endwin();
		printf("Your terminal does not support color\n");
		exit(1);
	}
	curs_set(0);
	start_color();
	scrollok(stdscr, 1);
	init_pair(1, COLOR_RED, COLOR_BLACK);
	init_pair(2, COLOR_GREEN, COLOR_BLACK);
	red_attr = COLOR_PAIR(1);
	//green_attr = COLOR_PAIR(2) | A_BOLD;
	green_attr = COLOR_PAIR(2);
#endif
	la_char_io_init(&red, "classic ADRIFT red text");
	la_char_io_init(&green, "classic ADRIFT green text");
	la_taf_init(&taf);
	la_tas_init(&tas, &taf, &green, &red);

	try {
		la_exception_cleanup_adda(tas, &la_tas_free);
		la_exception_cleanup_adda(taf, &la_taf_free);
		la_exception_cleanup_adda(red, &la_char_io_free);
		la_exception_cleanup_adda(green, &la_char_io_free);
#ifdef LA_USE_NCURSES
		fprv.putchar = &ncurput;
		fprv.flush = &ncurflush;
		fprv.ptr = &red_attr;
		la_char_filter_push(&red, "fout", &fprv);
		fprv.ptr = &green_attr;
		la_char_filter_push(&green, "fout", &fprv);
#else
		la_char_filter_push(&red, "fpout", stdout);
		la_char_filter_push(&green, "fpout", stdout);
#endif
		la_char_filter_push(&green, "linebreak", (void*)70);
		la_char_filter_push(&green, "tagkill", NULL);
		la_output_red_print(&tas, "Loading...\n");
		la_taf_parse_file(&taf, argv[optind]);
#ifdef LA_USE_NCURSES
		erase();
		refresh();
#endif
		la_game_restart(&tas);
		la_output_intro(&tas);
		la_output_red_print(&tas, "> inventory\n");
		la_output_inventory(&tas);
	}
	endtry;

#ifdef LA_USE_NCURSES
	endwin();
#endif

	exit(0);
}
