/* Example program to show off some of the highlevel capabilities of 
 * libadrift, it mimics the behaviour of the original salvage.c on
 * 	http://www.phys.uu.nl/~snel/adrift
 *
 * It works through parsing the complete file before outputting,
 * this version of salvage only works for 3.80, 3.90 and 4.00 files.
 * The lowlevel version may work for other versions. The highlevel
 * version is also a lot slower.
 *
 * This program is in the Public Domain (PD) 2004, Rik Snel
 */
#include <config.h>
#include <stdio.h>
#include <stdlib.h>
#include "../src/exception.h"
#include "../src/verbose.h"
#include "../src/taf/generic.h"

int main(int argc, char **argv) {	
	la_taf_t taf;

	if (argc != 1) {
		fprintf(stderr, 
"This program accepts an (un)obfuscated .TAF file in standard input\n"
"and it will send the unobfuscated version to stdout\n"
"\n"
"Usage: %s < INFILE > OUTFILE\n", argv[0]);
		exit(1);
	}

	la_verbose_init_fp(stderr);
	if (atexit(la_verbose_free)) {
		fprintf(stderr, "fatal error\n");
		exit(1);
	}

	la_taf_init(&taf);

	try {
		la_exception_cleanup_adda(taf, &la_taf_free);
		la_taf_parse_fp(&taf, stdin, "standard input");
		la_taf_write_fp(&taf, stdout, "standard output",
				LA_TAF_WUNOBFUSC|LA_TAF_WNOPASSWD);

	}
	endtry;

	exit(0);
}
