/* Example program to show off some of the lowlevel capabilities of 
 * libadrift, it mimics the behaviour of the original salvage.c on
 * 	http://www.phys.uu.nl/~snel/adrift
 *
 * This program is in the Public Domain (PD) 2004, Rik Snel
 */
#include <config.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "../src/adrift.h"
#include "../src/exception.h"
#include "../src/verbose.h"
#include "../src/char_io.h"
#include "../src/taf/generic.h"

#define LA_VERBOSE_TYPE Char_IO

void cat3(la_char_io_t *chrin) {
	int c, i = 0;
	char buf[14];
	while (1) {
		c = la_char_io_getc_nowhine(chrin);
		if (i >= 14) printf("%c", buf[i%14]);
		buf[i%14] = c;
		i++;
	}
}

void cat4(la_char_io_t *chrin) {
	char buf[14];
	int media_offset, offset;

	la_char_filter_pop(chrin);

	la_char_io_read(buf, 8, chrin); 
	buf[8] = '\0';
	if (sscanf(buf, "%d", &media_offset) != 1) {
		fprintf(stderr, "error reading media offset\n");
		throw();
	}

	la_char_filter_push(chrin, "inflate", NULL);

	try while(1) printf("%c", la_char_io_getc_nowhine(chrin)); 
	endtry;
	
	if (chrin->err != 2) throw();

	la_char_filter_pop(chrin);
	
	try if (la_char_io_getc_nowhine(chrin) != 0) fprintf(stderr, 
				"ASCII NUL byte expected, but not found\n");
	endtry;

	la_char_filter_bytecount(&offset, chrin, 0);
	
	if (media_offset < offset) 
		fprintf(stderr, "media offset in file is wrong, strange...\n");
}

int main(int argc, char **argv) {
	la_taf_t taf;
	int exitval = 0, obf = 1;
	la_char_io_t chrin;

	la_verbose_init_fp(stderr);
	la_char_io_init(&chrin, "standard input");

	try {
		la_char_filter_push(&chrin, "fpin", stdin);
		la_char_filter_push(&chrin, "crypt", NULL);

		la_taf_parse_version(&taf, &chrin, &obf);
		printf("Version %.02f\r\n", taf.version/100.);
		if (!obf) {
			/* input is already unobfuscated */
			try while (1) printf("%c", la_char_io_getc_nowhine(&chrin)); 
			endtry;
		} else {

		switch (taf.version) {
			case 380:
			case 390:
				cat3(&chrin);
				break;
			case 400:
				cat4(&chrin);
				break;
			default:
				fprintf(stderr, "version %.02f may not display correcty\n", taf.version/100.);
				if (taf.version < 400) cat3(&chrin);
				else cat4(&chrin);
		}	

		}
	}
	catch exitval = 1;
	endtry;

	la_char_io_free(&chrin);
	la_verbose_free();
	exit(exitval);
}
