#include <stdlib.h>
#include <string.h>
#include <gtk/gtk.h>
#include "../src/taf/generic.h"
#include "../src/taf/gtkwidgets.h"

GtkWidget *window;
GtkWidget *scrolled_window;
GtkWidget *scrolled_vbox;
int taf_open = 0;
int verbose_parsing = 0;
int preferences_semaphore = 0;
la_taf_data_widget_t w;
la_taf_t taf;
la_tas_t tas;

static void show_stuff() {
	tas.taf = &taf;
	gtk_widget_destroy(scrolled_vbox);
	scrolled_vbox = gtk_vbox_new(FALSE, 1);
	gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(scrolled_window), scrolled_vbox);
	la_taf_data_widget(&taf.data, &w, GTK_BOX(scrolled_vbox), "", &tas);
	gtk_widget_show_all(scrolled_vbox);
}

static void validate_password(GtkWidget *entry, GtkWidget *password_dialog) {
	if (la_taf_validate_password(&taf, (char*)gtk_entry_get_text(GTK_ENTRY(entry)))) {
		gtk_widget_destroy(password_dialog);
		show_stuff();
	} else {
		GtkWidget *dialog;
		gtk_widget_destroy(password_dialog);
		dialog = gtk_message_dialog_new(GTK_WINDOW(window), GTK_DIALOG_DESTROY_WITH_PARENT,
				GTK_MESSAGE_ERROR, GTK_BUTTONS_CLOSE, "I pity da fool who don't enter the right password!");
		g_signal_connect_swapped(dialog, "response", G_CALLBACK(gtk_widget_destroy), dialog);
		gtk_widget_show_all(dialog);
	}

}

static void ok_open(GtkWidget *w, GtkFileSelection *fs) {
	GtkWidget *password_dialog, *ok, *cancel, *label, *text, *hbox;
	GtkWidget *dialog;
	la_taf_free(&taf);
	la_taf_init(&taf);

	try {
		la_exception_cleanup_add(fs, gtk_widget_destroy);
		la_taf_parse_file(&taf, (char*)gtk_file_selection_get_filename(fs));
	}
	catch {
		dialog = gtk_message_dialog_new(GTK_WINDOW(window), GTK_DIALOG_DESTROY_WITH_PARENT,
				GTK_MESSAGE_ERROR, GTK_BUTTONS_CLOSE, "Some error occured");
		g_signal_connect_swapped(dialog, "response", G_CALLBACK(gtk_widget_destroy), dialog);
		gtk_widget_show_all(dialog);
		return;
	}
	endtry;

	if (la_taf_has_password(&taf)) {
		password_dialog = gtk_dialog_new();
		ok = gtk_button_new_from_stock(GTK_STOCK_OK);
		cancel = gtk_button_new_from_stock(GTK_STOCK_CANCEL);
		hbox = gtk_hbox_new(FALSE, 0);
		text = gtk_entry_new();
		gtk_entry_set_visibility(GTK_ENTRY(text), 0);
		label = gtk_label_new("Password");
		gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, TRUE, 0);
		gtk_box_pack_start(GTK_BOX(hbox), text, TRUE, TRUE, 0);
		gtk_box_pack_start(GTK_BOX(GTK_DIALOG(password_dialog)->action_area), ok, TRUE, TRUE, 0);
		gtk_box_pack_start(GTK_BOX(GTK_DIALOG(password_dialog)->action_area), cancel, TRUE, TRUE, 0);
		gtk_box_pack_start(GTK_BOX(GTK_DIALOG(password_dialog)->vbox), hbox, TRUE, TRUE, 0);
		gtk_window_set_title(GTK_WINDOW(password_dialog), "Password dialog");
		g_signal_connect_swapped(ok, "clicked", G_CALLBACK(gtk_widget_activate), text);
		g_signal_connect_swapped(cancel, "clicked", G_CALLBACK(gtk_widget_destroy), password_dialog);
		g_signal_connect(text, "activate", G_CALLBACK(validate_password), password_dialog);
		gtk_widget_show_all(password_dialog);
	} else {
		show_stuff();
	}
}

static void open() {
	GtkWidget *filesel;
	filesel = gtk_file_selection_new(
			"Select a .TAF file to open");
	g_signal_connect(G_OBJECT(GTK_FILE_SELECTION(filesel)->ok_button),
			"clicked", G_CALLBACK(ok_open), (gpointer)filesel);
	g_signal_connect_swapped(G_OBJECT(GTK_FILE_SELECTION(filesel)->
				cancel_button), "clicked", 
			G_CALLBACK(gtk_widget_destroy), G_OBJECT(filesel));
	gtk_widget_show(filesel);
}

static void change_ignore(GtkWidget *widget) {
	if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget))) {
		if (verbose_parsing == 0) {
			verbose_parsing = 1;
			la_verbose_channel_add(Taf_Parser, Verbose, &la_verbose_fatal_channel);
			la_verbose_channel_add(Taf_Parser, Dbg2, &la_verbose_fatal_channel);
		}
	} else {
		if (verbose_parsing == 1) {
			verbose_parsing = 0;
			la_verbose_channel_remove(Taf_Parser, Verbose, &la_verbose_fatal_channel);
			la_verbose_channel_remove(Taf_Parser, Dbg2, &la_verbose_fatal_channel);
		}
	}
}

static void preferences_destroy(GtkWidget *pref) {
	gtk_widget_destroy(pref);
	preferences_semaphore = 0;
}

static void preferences() {
	GtkWidget *dialog, *checkbox;
	if (preferences_semaphore)  return;
	preferences_semaphore = 1;
	dialog = gtk_dialog_new_with_buttons("Preferences", GTK_WINDOW(window),
			GTK_DIALOG_DESTROY_WITH_PARENT, GTK_STOCK_CLOSE, NULL);
	checkbox = gtk_check_button_new_with_label("verbose .TAF parsing");
	gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(checkbox), verbose_parsing);
	g_signal_connect_swapped(dialog, "response", G_CALLBACK(preferences_destroy), dialog);
	g_signal_connect_swapped(checkbox, "toggled", G_CALLBACK(change_ignore), checkbox);
	gtk_container_add(GTK_CONTAINER(GTK_DIALOG(dialog)->vbox), checkbox);
	gtk_widget_show_all(dialog);
}

static void about() {
	GtkWidget *dialog;

	dialog = gtk_message_dialog_new(GTK_WINDOW(window), GTK_DIALOG_DESTROY_WITH_PARENT,
			GTK_MESSAGE_INFO, GTK_BUTTONS_CLOSE, "Just a small GUI test.\nNothing to see, move along\n\nLicense GNU GPL v2");
	g_signal_connect_swapped(dialog, "response", G_CALLBACK(gtk_widget_destroy), dialog);
	gtk_window_set_title(GTK_WINDOW(dialog), "About");
	gtk_widget_show_all(dialog);
}

static GtkItemFactoryEntry menu_items[] = {
	{ "/_File",      NULL, NULL, 0, "<Branch>" }, 
//	{ "/File/_New",  NULL, hello, 0, "<StockItem>", GTK_STOCK_NEW },
	{ "/File/_Open", NULL, open, 0, "<StockItem>", GTK_STOCK_OPEN },
//	{ "/File/_Save", NULL, hello, 0, "<StockItem>", GTK_STOCK_SAVE },
//	{ "/File/Save _As", NULL, NULL, 0, "<Item>" },
	{ "/File/sep1",     NULL, NULL, 0, "<Separator>" },
	{ "/File/_Quit", NULL, gtk_main_quit, 0, 
		"<StockItem>", GTK_STOCK_QUIT },
	{ "/_Edit",     NULL, NULL, 0, "<Branch>" },
//	{ "/Edit/_Add", NULL, NULL, 0, "<Branch>", GTK_STOCK_ADD },
//	{ "/Edit/Add/_Room", NULL, NULL, 0, "<Item>" },
//	{ "/Edit/Add/_Object", NULL, NULL, 0, "<Item>" },
//	{ "/Edit/Add/_Action", NULL, NULL, 0, "<Item>" },
//	{ "/Edit/Add/_Event", NULL, NULL, 0, "<Item>" },
//	{ "/Edit/Add/_Character", NULL, NULL, 0, "<Item>" },
//	{ "/Edit/sep1", NULL, NULL, 0, "<Separator>" },
	{ "/Edit/Preferences", NULL, preferences, 0, 
		"<StockItem>", GTK_STOCK_PREFERENCES },
	{ "/_Help", NULL, NULL, 0, "<LastBranch>" },
	{ "/_Help/About", NULL, about, 0, "<Item>" }
};

static GtkWidget *get_menubar_menu(GtkWidget *window) {
	GtkItemFactory *item_factory;

	item_factory = gtk_item_factory_new(GTK_TYPE_MENU_BAR, "<main>", NULL);

	gtk_item_factory_create_items(item_factory, 
			sizeof(menu_items)/sizeof(menu_items[0]),
			menu_items, NULL);

	return gtk_item_factory_get_widget (item_factory, "<main>");
}

int main(int argc, char **argv) {
	//la_tas_t tas;
	GtkWidget *main_vbox;
	GtkWidget *menubar;
 
	la_verbose_init_fp(stderr);
	//la_verbose_channel_add_alltypes(Verbose, &la_verbose_fatal_channel);
	if (atexit(la_verbose_free)) {
		fprintf(stderr, "fatal error\n");
		exit(1);
	}
	la_taf_init(&taf);
	gtk_init (&argc, &argv);

	window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
	g_signal_connect (G_OBJECT(window), "destroy", 
			G_CALLBACK(gtk_main_quit), NULL);

	gtk_window_set_title(GTK_WINDOW(window), "Free ADRIFT 'Generator'");
	gtk_widget_set_size_request(GTK_WIDGET(window), 300, 400);
 
	main_vbox = gtk_vbox_new(FALSE, 1);
	scrolled_vbox = gtk_vbox_new(FALSE, 1);
	gtk_container_add (GTK_CONTAINER(window), main_vbox);
	scrolled_window = gtk_scrolled_window_new(NULL, NULL);
 
	menubar = get_menubar_menu (window);
	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrolled_window), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);

	gtk_box_pack_start(GTK_BOX(main_vbox), menubar, FALSE, TRUE, 0);
	gtk_box_pack_start(GTK_BOX(main_vbox), scrolled_window, TRUE, TRUE, 1);
	gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(scrolled_window), scrolled_vbox);


#if 0
	if (argc != 2) {
		fprintf(stderr, "Usage: ./%s FILE.taf\n", argv[0]);
		exit(1);
	}
#endif

#if 0
	try {
		la_taf_parse_file(&taf, argv[1]);
		tas.taf = &taf;
		la_taf_data_widget(&taf.data, &w, GTK_BOX(scrolled_vbox), "", &tas);
#endif
		gtk_widget_show_all (window);
  
		gtk_main ();
#if 0
	} endtry;
#endif

	exit(0);
}


