/* action.c - functions for actions on the gamestate
 *
 * Copyright (C) 2004 Rik Snel <snel@phys.uu.nl>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; only version 2 of the License, not
 * any other version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. 
 *
 * $Id: game.h 454 2004-04-01 19:45:12Z rsnel $
 */
#include "taf/generic.h"
#include "object.h"
#include "resolve.h"
#include "output.h"

#define LA_VERBOSE_TYPE Action

void la_action_player_room(la_tas_t *tas, la_taf_room_spec_t *room) {
	switch (tas->data.player.location.posture.realtype) {
		case la_taf_person_posture_standing_on_e:
			if (tas->data.player.location.posture.spec.standing_on.realtype != la_taf_object_spec_the_floor_e)
				LA_THROW("unimplemented (standing on something)");
			break;
		default:
			LA_THROW("unimplemented (sitting or lying)");
	}
	if (room->realtype != la_taf_room_spec_room_e) LA_THROW("unimplemented (destination room not resolvable)");
	la_taf_room_spec_copy(&tas->data.player.location.room, room, "", tas);
	//la_output_red_print(tas, "ACTION: moving Player to %s\n", ((la_taf_description_t*)RF(tas->data.player.location.room.spec.room)->descriptions.ptr)->sdesc);
}

void la_action_player_posture(la_tas_t *tas, la_taf_person_posture_t *posture) {
	la_array_t a;
	int *rooms = NULL;
	int no_rooms = 0;
	//int i;
	la_taf_person_posture_t *spst = &tas->data.player.location.posture;
	la_taf_person_posture_copy(spst, posture, "", tas);
	la_array_ninit(&a, rooms, NULL);
	//la_output_red_print(tas, "ACTION: changing Player posture to ");
	switch (spst->realtype) {
		case la_taf_person_posture_standing_on_e:
			//la_output_red_print(tas, "standing on ");
			la_resolve_object_f(tas->taf, &spst->spec.standing_on);
			if (spst->spec.standing_on.realtype == la_taf_object_spec_the_floor_e) {
				//la_output_red_print(tas, "the floor\n");
			} else if (spst->spec.standing_on.realtype == la_taf_object_spec_direct_e) {
				la_output_red_print(tas, "%s %s\n", OF(spst->spec.standing_on.spec.direct)->prefix, OF(spst->spec.standing_on.spec.direct)->name);
				//la_resolve_object_roomlocation(tas, &a, spst->spec.standing_on.spec.direct);
				//for (i = 0; i < no_rooms; i++) la_output_red_print(tas, "%d ", rooms[i]);
			} else LA_THROW("unimplemented (posture object not resolvable)");
			break;
		case la_taf_person_posture_sitting_on_e:
			//la_output_red_print(tas, "sitting on ");
			la_resolve_object_f(tas->taf, &spst->spec.sitting_on);
			if (spst->spec.sitting_on.realtype == la_taf_object_spec_the_floor_e) {
				//la_output_red_print(tas, "the floor\n");
			} else if (spst->spec.sitting_on.realtype == la_taf_object_spec_direct_e) {
				//la_output_red_print(tas, "%s %s\n", OF(spst->spec.sitting_on.spec.direct)->prefix, OF(spst->spec.sitting_on.spec.direct)->name);
			} else LA_THROW("unimplemented (posture object not resolvable)");
			break;
		case la_taf_person_posture_lying_on_e:
			//la_output_red_print(tas, "lying on ");
			la_resolve_object_f(tas->taf, &spst->spec.lying_on);
			if (spst->spec.lying_on.realtype == la_taf_object_spec_the_floor_e) {
				//la_output_red_print(tas, "the floor\n");
			} else if (spst->spec.lying_on.realtype == la_taf_object_spec_direct_e) {
				//la_output_red_print(tas, "%s %s\n", OF(spst->spec.lying_on.spec.direct)->prefix, OF(spst->spec.lying_on.spec.direct)->name);
			} else LA_THROW("unimplemented (posture object not resolvable)");
			break;
	}
	la_array_free(&a);
}

void la_action_character_room(la_tas_t *tas, int character, la_taf_room_spec_t *room) {
	la_tas_character_t *chr = CS(character);
	la_taf_room_spec_copy(&chr->location.room, room, "", tas);
	//la_output_red_print(tas, "ACTION: moving %s to ", CF(character)->name);
	if (room->realtype == la_taf_room_spec_none_e) {
		//la_output_red_print(tas, "Hidden\n");
	} else if (room->realtype == la_taf_room_spec_room_e) {
		//la_output_red_print(tas, "%s\n", ((la_taf_description_t*)RF(room->spec.room)->descriptions.ptr)->sdesc);
	} else LA_THROW("runtime error");
}

void la_action_dynamic_object_location(la_tas_t *tas, int object, la_taf_definite_dynamic_location_t *loc) {
	la_tas_object_t *obj = (((la_tas_object_t*)tas->data.objects.ptr) + object);
	la_taf_definite_dynamic_location_copy(&obj->location, loc, "", tas);
	//la_output_red_print(tas, "ACTION: moving %s %s ", (((la_taf_object_t*)tas->taf->data.objects.ptr) + object)->prefix, (((la_taf_object_t*)tas->taf->data.objects.ptr) + object)->name);
	switch (obj->location.posture.realtype) {
		case la_taf_dynamic_posture_held_by_e:
			switch (obj->location.posture.spec.held_by.realtype) {
				case la_taf_person_spec_player_e:
					//la_output_red_print(tas, "to held by Player\n");
					obj->location.posture.pwtype = 1;
					obj->location.posture.spec.held_by.exp0 = 0;
					break;
				case la_taf_person_spec_character_e:
					//la_output_red_print(tas, "to held by %s\n", CF(obj->location.posture.spec.held_by.spec.character)->name);
					obj->location.posture.pwtype = 5;
					break;
				default:
					LA_THROW("unimplemented");
			}
			break;
		case la_taf_dynamic_posture_worn_by_e:
			switch (obj->location.posture.spec.worn_by.realtype) {
				case la_taf_person_spec_player_e:
					//la_output_red_print(tas, "to worn by Player\n");
					obj->location.posture.pwtype = 4;
					obj->location.posture.spec.worn_by.exp0 = 0;
					break;
				case la_taf_person_spec_character_e:
					//la_output_red_print(tas, "to worn by %s\n", CF(obj->location.posture.spec.worn_by.spec.character)->name);
					obj->location.posture.pwtype = 6;
					break;
				default:
					LA_THROW("unimplemented");
			}
			break;
		case la_taf_dynamic_posture_in_e:
			la_resolve_object_f(tas->taf, &obj->location.posture.spec.in);
			//la_output_red_print(tas, "inside %s %s\n", OF(obj->location.posture.spec.in.spec.direct)->prefix, OF(obj->location.posture.spec.in.spec.direct)->name);
			obj->location.posture.pwtype = 2;
			break;
		case la_taf_dynamic_posture_on_e:
			la_resolve_object_f(tas->taf, &obj->location.posture.spec.on);
			//la_output_red_print(tas, "on top of %s %s\n", OF(obj->location.posture.spec.on.spec.direct)->prefix, OF(obj->location.posture.spec.on.spec.direct)->name);
			obj->location.posture.pwtype = 3;
			break;
		case la_taf_dynamic_posture_around_e:
			obj->location.posture.spec.exp0 = -1;
			obj->location.posture.pwtype = 0;
			if (obj->location.room.realtype == la_taf_room_spec_none_e) {
				//la_output_red_print(tas, "to Hidden\n");
			} else if (obj->location.room.realtype == la_taf_room_spec_room_e) {
				//la_output_red_print(tas, "to %s\n", ((la_taf_description_t*)RF(obj->location.room.spec.room)->descriptions.ptr)->sdesc);
			} else LA_THROW("runtime error");
			break;
	}

}	

void la_action_object_openstate(la_tas_t *tas, int object, la_taf_open_state_types_t type) {
	la_tas_object_t *obj = (((la_tas_object_t*)tas->data.objects.ptr) + object);
	//la_output_red_print(tas, "ACTION: changing openstate of %s %s to ", (((la_taf_object_t*)tas->taf->data.objects.ptr) + object)->prefix, (((la_taf_object_t*)tas->taf->data.objects.ptr) + object)->name);
	obj->open_state.pwtype = type;
	obj->open_state.realtype = type;
	switch (type) {
		case la_taf_open_state_open_e:
			//la_output_red_print(tas, "open\n");
			break;
		case la_taf_open_state_closed_e:
			//la_output_red_print(tas, "closed\n");
			break;
		case la_taf_open_state_locked_e:
			//la_output_red_print(tas, "locked\n");
			break;
	}
}

#if 0
void la_action_object(la_tas_t *tas, la_laf_object_action_t *action) {
	int i;
	la_taf_object_spec_t *lhs;
	for (i = 0; i < la_slot_total(action->lhss.slots); i++) {
	}
	switch (action->realtype) {
		case la_taf_object_action_change_location_to_e:
#endif
			
void la_action(la_tas_t *tas, la_taf_action_t *action) {
	if (!action->valid) return;
	switch (action->realtype) {
#if 0
		case la_taf_action_object_e:
			la_action_object(tas, &action->spec.object);
			break;
#endif
		default:
			//LA_THROW("action not implemented");
			break;
	}
}
