/* cex.c - function for calculating simple C expressions
 *
 * Copyright (C) 2004 Rik Snel <snel@phys.uu.nl>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; only version 2 of the License, not
 * any other version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. 
 *
 * $Id: cex.c 366 2004-03-14 17:27:24Z rsnel $
 */
#include <stdio.h>
#include <stdlib.h>
#include "verbose.h"
#include "exception.h"
#include "cex_parse.h"
#include "cex_lex.h"
#include "cex.h"

#define LA_VERBOSE_TYPE Cex

extern int cex_parse(void*);

int la_cex(const char *cex, int in) {
	la_cex_t io;
	yyscan_t scanner;
	YY_BUFFER_STATE buf;
	int err;
	io.in = in;
	if ((err = yylex_init(&scanner))) {
		if (err == ENOMEM) LA_THROW("out of memory initing lexer");
		LA_THROW("internal error initing lexer");
	}
	yyset_extra(&io, scanner);
	buf = yy_scan_string(cex, scanner);
	if (!buf) {
		yylex_destroy(scanner);
		LA_THROW("impossible error, you should not see this");
	}
	err = cex_parse(scanner);
	yy_delete_buffer(buf, scanner);
	yylex_destroy(scanner);
	if (err) LA_THROW("error parsing %s", cex);
	return io.out;
}

#if 0
/* shut up compiler warnings about this function being
 * nonexistant and unused */
static int yy_init_globals(yyscan_t p) {
	void *bla;
	bla = yy_init_globals;
	return 0;
}
#endif
