/* cex_parse.y - simple parser for calculating simple C expressions
 *
 * Copyright (C) 2004 Rik Snel <snel@phys.uu.nl>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; only version 2 of the License, not
 * any other version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. 
 *
 * $Id: cex_parse.y 366 2004-03-14 17:27:24Z rsnel $
 */
%{
#define YYPARSE_PARAM scanner
#define YYLEX_PARAM scanner
#define YYERROR_VERBOSE 1
#include <stdio.h>
#include "verbose.h"
#include "cex.h"

#define LA_VERBOSE_TYPE Cex
int cex_lex();
void yyerror(char*);
la_cex_t *yyget_extra();
%}
%locations
%token_table
//%output="cex_parse.c"
%name-prefix="cex_"
%pure_parser
%defines
%union {
	int num;
}
%token <num> NUMBER
%type <num> exp
%right ':' '?'
%left AND OR
%left '|'
%left '^'
%left '&'
%left EQ NEQ
%left SM_EQ '<' BG_EQ '>'
%left SHL SHR
%left '+' '-'
%left '*' '/' '%'
%right NEG POS
%%
input:	exp 	{ yyget_extra(scanner)->out = $1; }
;

exp:	  NUMBER			{ $$ = $1; }
	| 'i'				{ $$ = yyget_extra(scanner)->in; }
	| exp '?' exp ':' exp 		{ $$ = $1 ? $3 : $5; }
	| exp '+' exp			{ $$ = $1 + $3; }
	| '+' exp %prec POS		{ $$ = $2; }
	| exp '-' exp			{ $$ = $1 - $3; }
	| '-' exp %prec NEG		{ $$ = -$2; }
	| exp '%' exp			{ $$ = $1%$3; }
	| exp '*' exp			{ $$ = $1*$3; }
	| exp '/' exp			{ $$ = $1/$3; }
	| exp SHL exp			{ $$ = $1<<$3; }
	| exp SHR exp			{ $$ = $1>>$3; }
	| exp NEQ exp			{ $$ = $1 != $3 ? 1 : 0; }
	| exp EQ exp			{ $$ = $1 == $3 ? 1 : 0; }
	| exp SM_EQ exp			{ $$ = $1 <= $3 ? 1 : 0; }
	| exp BG_EQ exp			{ $$ = $1 >= $3 ? 1 : 0; }
	| exp '<' exp			{ $$ = $1 < $3 ? 1 : 0; }
	| exp '>' exp			{ $$ = $1 > $3 ? 1 : 0; }
	| exp '&' exp			{ $$ = $1&$3; }
	| exp '^' exp			{ $$ = $1^$3; }
	| exp '|' exp			{ $$ = $1|$3; }
	| exp AND exp			{ $$ = $1 && $3 ? 1 : 0; }
	| exp OR exp			{ $$ = $1 || $3 ? 1 : 0; }
	| '(' exp ')'			{ $$ = $2; }
;
%%
void yyerror(char *s) {
	LA_ERROR("%s", s);
}
