/* bufout.c - output to buffer
 *
 * Copyright (C) 2004 Rik Snel <snel@phys.uu.nl>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; only version 2 of the License, not
 * any other version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. 
 *
 * $Id: bufout.c 365 2004-03-14 17:19:16Z rsnel $
 */
#include <stdio.h>
#include <errno.h>
#include <string.h>
#include <stdlib.h>
#include <assert.h>
#include "../verbose.h"
#include "../char_io.h"
#include "../array.h"

typedef struct {
	int ptr;
	la_array_t *a;
} priv_bufout_t;

#define LA_VERBOSE_TYPE Char_IO

static int init(void **priv, void *argument) {
	priv_bufout_t *p;

	if (((la_array_t*)argument)->size != 1) {
		LA_ERROR("wrong 'size' in argument");
		return LA_CHAR_IO_ERROR;
	}

	p = malloc(sizeof(*p));
	if (!p) {
		LA_ERROR("%s", strerror(errno));
		return LA_CHAR_IO_ERROR;
	}

	p->ptr = 0;
	p->a = argument;
	
	*priv = p;
	return LA_CHAR_IO_OK;
}

static int filter(void *priv, char *next_in, int *avail_in,
		char *next_out, int *avail_out, int flush, char *error) {
	priv_bufout_t *p = priv;

	if (flush) assert(*avail_in == 0);

	while (*avail_in > 0) { 
		//printf("%p, %d\n", p, *p->a->no_objects);
		try la_array_insert(p->a, next_in++, p->ptr);
		catch {
			LA_ERROR("in bufout: %s", p->a->strerr);
			return LA_CHAR_IO_ERROR;
		}
		endtry;
		p->ptr++;
		(*avail_in)--;
	}

	if (flush) return LA_CHAR_IO_FLUSHED;

	return LA_CHAR_IO_OK;
}

static int control(void *priv, char *command, void *param) {
	//priv_bufout_t *p = priv;
#if 0
	if (!strcmp(command, "passive")) {
		p->active = 0;
		return LA_CHAR_IO_OK;
	}
#endif
	LA_ERROR("unkown command %s", command);
	return LA_CHAR_IO_ERROR;
}

static void uninit(void *priv) {
	free(priv);
}

la_char_filter_info_t la_char_filter_bufout = { &init, &filter, &control, 
	&uninit, "bufout" };

