/* crlf2z.c - char_io plugin for translating \r\n to \0
 *
 * Copyright (C) 2004 Rik Snel <snel@phys.uu.nl>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; only version 2 of the License, not
 * any other version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. 
 *
 * $Id: crlf2z.c 365 2004-03-14 17:19:16Z rsnel $
 */
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <errno.h>
#include <assert.h>
#include "../verbose.h"
#include "../char_io.h"

#define LA_VERBOSE_TYPE Char_IO

typedef struct {
	int state;
	int line_number, *lnp;
	int line_start;
} priv_crlf2z_t;

static int init(void **priv, void *argument) {
	priv_crlf2z_t *p;

	if (!(p = malloc(sizeof(priv_crlf2z_t)))) {
		LA_ERROR("%s", strerror(errno));
		return LA_CHAR_IO_ERROR;
	}

	p->state = 0;
	p->line_number = 0;
	
	/* ability to use 'external' line number */
	if (argument) 
		p->lnp = argument;
	else
		p->lnp = &p->line_number;

	p->line_start = 1;
	*priv = p;

	return LA_CHAR_IO_OK;
}

static int filter(void *priv, char *next_in, int *avail_in, char *next_out, 
		int *avail_out, int flush, char *error) {
	priv_crlf2z_t *p = priv;

	if (flush) assert(*avail_in == 0);
		
	while (*avail_in > 0 && *avail_out > 0) { 
		if (p->line_start == 1) {
			(*p->lnp)++;
			p->line_start = 0;
		}
		switch (p->state) {
			case 0:
				if (*next_in == '\n') {
					LA_ERROR( 
				"LF without preceeding CR at line %d of input", 
							*p->lnp);
					return LA_CHAR_IO_ERROR;
				}
				if (*next_in == '\r') {
					p->state = 1;
				} else {
					*next_out++ = *next_in++;
					(*avail_out)--;
				}
				break;
			case 1:
				if (*next_in != '\n') { 
					LA_ERROR(
					"CR without LF at line %d of input", 
							*p->lnp);
					return LA_CHAR_IO_ERROR;
				}
				p->state = 0;
				p->line_start = 1;
				*next_out++ = '\0';
				(*avail_out)--;
		}
		(*avail_in)--;
	}

	if (flush) return LA_CHAR_IO_FLUSHED;
	return LA_CHAR_IO_OK;
}

static int control(void *priv, char *command, void *param) {
	priv_crlf2z_t *p = priv;
	if (!strcmp(command, "lineno")) {
		*((int*)param) = *p->lnp;
		return LA_CHAR_IO_OK;
	}
	LA_ERROR("unknown control \"%s\" for crlf2z filter", command);
	return LA_CHAR_IO_ERROR;
}

static void uninit(void *priv) {
	free(priv);
}

la_char_filter_info_t la_char_filter_crlf2z = { &init, &filter, &control, 
	&uninit, "crlf2z" };

