/* deflate.c - stream compressor using zlib
 *
 * Copyright (C) 2004 Rik Snel <snel@phys.uu.nl>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; only version 2 of the License, not
 * any other version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. 
 *
 * $Id: deflate.c 365 2004-03-14 17:19:16Z rsnel $
 */
#include <stdio.h>
#include <stdlib.h>
#include <zlib.h>
#include <string.h>
#include <errno.h>
#include "../verbose.h"
#include "../char_io.h"

#define LA_VERBOSE_TYPE Char_IO

static int init(void **priv, void *argument) {
	int err;
	int level = Z_DEFAULT_COMPRESSION;
	z_stream *c_stream;

	if (argument) {
		level = *(int*)argument;
		if (level < 0 || level > 9) {
			LA_WARNING("illegal compression level specified, taking default");
			level = Z_DEFAULT_COMPRESSION;
		}
	}

	c_stream = malloc(sizeof(*c_stream));
	if (!c_stream) {
		LA_ERROR("%s", strerror(errno));
		return LA_CHAR_IO_ERROR;
	}

	c_stream->zalloc = (alloc_func)0;
	c_stream->zfree = (free_func)0;
	
	if ((err = deflateInit(c_stream, level)) != Z_OK) {
		LA_ERROR("error %d initing zlib", err);
		return LA_CHAR_IO_ERROR;
	}

	*priv = c_stream;
	return LA_CHAR_IO_OK;
}

static int filter(void *priv, char *next_in, int *avail_in, char *next_out, 
		int *avail_out, int flush, char *error) {
	z_stream *c_stream = priv;
	int err;

	c_stream->next_in = next_in;
	c_stream->avail_in = *avail_in;
	c_stream->next_out = next_out;
	c_stream->avail_out = *avail_out;
	if (flush) {
		err = deflate(c_stream, Z_FINISH);
	} else {
		err = deflate(c_stream, Z_NO_FLUSH);
	}
	*avail_in = c_stream->avail_in;
	*avail_out = c_stream->avail_out;

	switch (err) {
		case Z_OK:
			return LA_CHAR_IO_OK;
		case Z_STREAM_END:
			snprintf(error, LA_CHAR_IO_STRERR_LEN,
					"zlib: end of stream");
			if (flush) return LA_CHAR_IO_FLUSHED;
			else return LA_CHAR_IO_EOF;
	}

	LA_ERROR("zlib: error %d compressing data", err);

	return LA_CHAR_IO_ERROR;
}

static int control(void *priv, char *command, void *param) {
	LA_ERROR("deflate filter does not have commands");
	return LA_CHAR_IO_ERROR;
}

static void uninit(void *priv) {
	z_stream *c_stream = priv;

	deflateEnd(c_stream);

	free(c_stream);
	return;
}

la_char_filter_info_t la_char_filter_deflate = { &init, &filter, 
	&control, &uninit, "deflate" };

