/* fout.c - output to function
 *
 * Copyright (C) 2004 Rik Snel <snel@phys.uu.nl>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; only version 2 of the License, not
 * any other version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. 
 *
 * $Id: fpout.c 365 2004-03-14 17:19:16Z rsnel $
 */
#include <stdlib.h>
#include <errno.h>
#include <string.h>
#include <stdio.h>
#include <assert.h>
#include "../verbose.h"
#include "../char_io.h"

#define LA_VERBOSE_TYPE Char_IO

typedef struct {
	int (*putchar)(void*, int);
	int (*flush)(void*);
	void *ptr;
} fout_priv_t;

static int init(void **priv, void *argument) {
	fout_priv_t *f = (fout_priv_t*)argument;
	fout_priv_t *p;
	assert(f);

	p = malloc(sizeof(fout_priv_t));
	if (!p) {
		LA_ERROR("%s", strerror(errno));
		return LA_CHAR_IO_ERROR;
	}

	p->putchar = f->putchar;
	p->flush = f->flush;
	p->ptr = f->ptr;
	*priv = p;

	return LA_CHAR_IO_OK;
}

static int filter(void *priv, char *next_in, int *avail_in, char *next_out, 
		int *avail_out, int flush, char *error) {
	fout_priv_t *f = priv;
	assert(f);

	if (flush) {
		assert(*avail_in == 0);
		if ((*f->flush)(f->ptr) == EOF) {
			LA_ERROR("error flushing buffer");
			return LA_CHAR_IO_ERROR;
		}
		return LA_CHAR_IO_FLUSHED;
	}

	while (*avail_in > 0) { 
		if ((*f->putchar)(f->ptr, *next_in) == EOF) {
			LA_ERROR("error outputting character");
			return LA_CHAR_IO_ERROR;
		}
		next_in++;
		(*avail_in)--;
	}

	return LA_CHAR_IO_OK;
}

static int control(void *priv, char *command, void *param) {
	LA_ERROR("fout knows_no_commands");
	return LA_CHAR_IO_ERROR;
}

static void uninit(void *priv) {
	free(priv);
}

la_char_filter_info_t la_char_filter_fout = { &init, &filter, &control, 
	&uninit, "fout" };

