/* fpin.c - char_io input plugin for input from FILE* pointer
 *
 * Copyright (C) 2004 Rik Snel <snel@phys.uu.nl>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; only version 2 of the License, not
 * any other version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. 
 *
 * $Id: fpin.c 365 2004-03-14 17:19:16Z rsnel $
 */
#include <stdlib.h>
#include <stdio.h>
#include <assert.h>
#include "../char_io.h"
#include "../verbose.h"

#define LA_VERBOSE_TYPE Char_IO

static int init(void **priv, void *argument) {
	assert(argument);

	*priv = argument;

	return LA_CHAR_IO_OK;
}

static int filter(void *priv, char *next_in, int *avail_in, char *next_out, 
		int *avail_out, int flush, char *error) {
	FILE *fp = priv;
	int c;
	assert(fp);

	while (*avail_out > 0) { 
		c = getc(fp);
		if (c == EOF) {
			snprintf(error, LA_CHAR_IO_STRERR_LEN,
					"end of file (while reading)");
			return LA_CHAR_IO_ERROR;
		}
		*next_out++ = c;
		(*avail_out)--;
	}

	return LA_CHAR_IO_OK;
}

static int control(void *priv, char *command, void *param) {
	LA_ERROR("fpin knows_no_commands");
	return LA_CHAR_IO_ERROR;
}

static void uninit(void *priv) {
}

la_char_filter_info_t la_char_filter_fpin = { &init, &filter, &control, 
	&uninit, "fpin" };

