/* inflate.c - stream decompressor using zlib
 *
 * Copyright (C) 2004 Rik Snel <snel@phys.uu.nl>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; only version 2 of the License, not
 * any other version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. 
 *
 * $Id: inflate.c 365 2004-03-14 17:19:16Z rsnel $
 */
#include <stdio.h>
#include <stdlib.h>
#include <zlib.h>
#include <string.h>
#include <errno.h>
#include "../verbose.h"
#include "../char_io.h"

#define LA_VERBOSE_TYPE Char_IO

static int init(void **priv, void *argument) {
	int err;
	z_stream *d_stream;

	d_stream = malloc(sizeof(*d_stream));
	if (!d_stream) {
		LA_ERROR("%s", strerror(errno));
		return LA_CHAR_IO_ERROR;
	}

	d_stream->zalloc = (alloc_func)0;
	d_stream->zfree = (free_func)0;
	
	if ((err = inflateInit(d_stream)) != Z_OK) {
		LA_ERROR("error %d initing zlib", err);
		return LA_CHAR_IO_ERROR;
	}

	*priv = d_stream;
	return LA_CHAR_IO_OK;
}

static int filter(void *priv, char *next_in, int *avail_in, char *next_out, 
		int *avail_out, int flush, char *error) {
	z_stream *d_stream = priv;
	int err;

	d_stream->next_in = next_in;
	d_stream->avail_in = *avail_in;
	d_stream->next_out = next_out;
	d_stream->avail_out = *avail_out;
	err = inflate(d_stream, Z_NO_FLUSH);
	*avail_in = d_stream->avail_in;
	*avail_out = d_stream->avail_out;

	switch (err) {
		case Z_OK:
			return LA_CHAR_IO_OK;
		case Z_STREAM_END:
			snprintf(error, LA_CHAR_IO_STRERR_LEN,
					"zlib: end of stream");
			return LA_CHAR_IO_EOF;
	}

	LA_ERROR("zlib: error %d decompressing data", err);

	return LA_CHAR_IO_ERROR;
}

static int control(void *priv, char *command, void *param) {
	LA_ERROR("inflate filter does not have commands");
	return LA_CHAR_IO_ERROR;
}

static void uninit(void *priv) {
	z_stream *d_stream = priv;

	inflateEnd(d_stream);

	free(d_stream);
	return;
}

la_char_filter_info_t la_char_filter_inflate = { &init, &filter, 
	&control, &uninit, "inflate" };

