/* linebreak.c - naive line breaker
 *
 * Copyright (C) 2004 Rik Snel <snel@phys.uu.nl>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; only version 2 of the License, not
 * any other version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. 
 *
 * $Id: fpout.c 365 2004-03-14 17:19:16Z rsnel $
 */
#include <stdlib.h>
#include <assert.h>
#include <string.h>
#include <errno.h>
#include "../adrift.h"
#include "../verbose.h"
#include "../char_io.h"

#define LA_VERBOSE_TYPE Char_IO

typedef struct {
	char *ring;
	int size;
	int rindex;
	int windex;
	int sindex;
	int tobewritten;
	int state; 
} linebreak_priv_t;

#define FILL	(p->windex - p->rindex)
#define EMPTY	(FILL == 0)
#define FULL	(FILL == p->size)
#define WRITE	*(p->ring + (p->windex++)%p->size)
#define READ	*(p->ring + (p->rindex++)%p->size)
static int init(void **priv, void *argument) {
	linebreak_priv_t *p;
	assert(argument);

	p = malloc(sizeof(linebreak_priv_t));
	if (!p) {
		LA_ERROR(strerror(errno));
		return LA_CHAR_IO_ERROR;
	}

	p->state = 0;
	p->size = (int)argument;
	p->rindex = 0;
	p->sindex = -1;
	p->windex = 0;
	p->ring = calloc(sizeof(char)*p->size, 1);
	if (!p->ring) {
		LA_ERROR(strerror(errno));
		free(p);
		return LA_CHAR_IO_ERROR;
	}
	*priv = p;

	return LA_CHAR_IO_OK;
}

static int filter(void *priv, char *next_in, int *avail_in,
		char *next_out, int *avail_out, int flush, char *error) {
	linebreak_priv_t *p = priv;
	if (flush) {
		assert(*avail_in == 0);
		p->state = 2;
	}

#if 0
	while (*avail_in > 0 && *avail_out > 0) {
		*next_out++ = *next_in++;
		(*avail_out)--; (*avail_in)--;
	}
#endif 
	//VERBOSE("p->state=%d", p->state);
	switch (p->state) {
		case 0:
			while (*avail_in > 0 && !FULL) {
				if (*next_in == ' ') p->sindex = p->windex;
				WRITE = *next_in;
				next_in++;
				(*avail_in)--;
				if (*(next_in - 1) == '\n') {
					p->tobewritten = 
						p->windex - p->rindex - 1;
					p->state = 1;
					break;
				}
			}
			if (FULL && p->state == 0) {
				p->state = 1;
				if (p->sindex >= p->rindex) {
					//VERBOSE("BREAK");
					p->tobewritten = p->sindex - p->rindex;
				} else {
					//VERBOSE("PANIC");
					p->tobewritten = p->size;
				}
				//VERBOSE("tobewritten=%d", p->tobewritten);
			}
			//VERBOSE("r=%d w=%d s=%d", p->rindex, p->windex, p->sindex);
			break;
		case 1:
			while (*avail_out > 0) {
				if (p->tobewritten-- > 0) {
					*next_out = READ;
					(*avail_out)--;
				} else {
					*next_out = '\n';
					(*avail_out)--;
					p->rindex++;
					p->state = 0;
				}
			}
			break;
		case 2:
			while (*avail_out > 0 && !EMPTY) {
				*next_out = READ;
				(*avail_out)--;
			}
			break;
		default:
			assert(0);
	}


	
	if (flush && EMPTY) {
		p->state = 0;
#if 0
		p->rindex = 0;
		p->windex = 0;
#endif
		return LA_CHAR_IO_FLUSHED;
	}
	return LA_CHAR_IO_OK;
}

static int control(void *priv, char *command, void *param) {
#if 0
	linebreak_priv_t *p = priv;
	if (!strcmp(command, "sepon")) {
		p->seperator = 1;
		return LA_OK;
	}
	if (!strcmp(command, "sepoff")) {
		p->seperator = 0;
		return LA_OK;
	}
	ERROR("unknown command %s", command);
#endif
	LA_ERROR("linebreak has no commands");
	return LA_CHAR_IO_ERROR;
}

static void uninit(void *priv) {
	linebreak_priv_t *p = priv;
	free(p->ring);
	free(priv);
}

la_char_filter_info_t la_char_filter_linebreak = { &init, &filter, &control, 
	&uninit, "linebreak" };

