/* tagkill.c - char_io plugin for simple acting on html tags
 *
 * Copyright (C) 2004 Rik Snel <snel@phys.uu.nl>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; only version 2 of the License, not
 * any other version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. 
 *
 * $Id: tagkill.c 461 2004-04-03 17:51:28Z rsnel $
 */
#include <stdlib.h>
#include <assert.h>
#include <string.h>
#include <errno.h>
#include "../verbose.h"
#include "../char_io.h"
#include "../array.h"

#define LA_VERBOSE_TYPE Char_IO

typedef struct {
	int intag;
	la_array_t tag;
	int no_tags;
	char *tags;
} tagkill_priv_t;

static int init(void **priv, void *argument) {
	tagkill_priv_t *p;

	p = malloc(sizeof(tagkill_priv_t));
	if (!p) {
		LA_ERROR("%s", strerror(errno));
		return LA_CHAR_IO_ERROR;
	}

	p->intag = 0;
	p->tags = NULL;
	p->no_tags = 0;
	la_array_sinit(&p->tag, (*p), tags, NULL);
	*priv = p;

	return LA_CHAR_IO_OK;
}

static int filter(void *priv, char *next_in, int *avail_in, char *next_out, 
		int *avail_out, int flush, char *error) {
	tagkill_priv_t *p = priv;
	char null = '\0';

	if (flush) {
		assert(*avail_in == 0);
		if (p->intag == 1) LA_WARNING("tag unclosed");
	}

	switch (p->intag) {
		case 0:
			if (*avail_in > 0) {
				if (*next_in == '<') p->intag = 1;
				else {
					if (*avail_out > 0) {
						*next_out++ = *next_in;
						(*avail_out)--;
					}
				}
				next_in++;
				(*avail_in)--;
			}
			break;
		case 1:
			if (*avail_in > 0 && *avail_out > 0) {
				if (*next_in == '>') {
					p->intag = 0;
					try la_array_push(&p->tag, &null); 
					catch {
						LA_ERROR("in tagkill: %s",
								p->tag.strerr);
						return LA_CHAR_IO_ERROR;
					}
					endtry;
					if (!strcasecmp(p->tags, "br")) {
						*next_out++ = '\n';
						(*avail_out)--;
					} else if (!strcasecmp(p->tags, "p")) {
						*next_out++ = '\n';
						(*avail_out)--;
						p->intag = 2;
					}
					la_array_empty(&p->tag);
				} else {
					try la_array_push(&p->tag, next_in);
					catch {
						LA_ERROR("in tagkill: %s",
								p->tag.strerr);
						return LA_CHAR_IO_ERROR;
					}
					endtry;
				}
				next_in++;
				(*avail_in)--;
			}
			break;
		case 2:
			if (*avail_out > 0) {
				*next_out++ = '\n';
				(*avail_out)--;
				p->intag = 0;
			}
			break;
	}

	if (flush) {
		p->intag = 0;
		return LA_CHAR_IO_FLUSHED;
	}
	return LA_CHAR_IO_OK;
}

static int control(void *priv, char *command, void *param) {
	LA_ERROR("tagkill filter has no controls");
	return LA_CHAR_IO_ERROR;
}

static void uninit(void *priv) {
	tagkill_priv_t *p = priv;
	la_array_free(&p->tag);
	free(priv);
}

la_char_filter_info_t la_char_filter_tagkill = { &init, &filter, &control, 
	&uninit, "tagkill" };

