/* tee.c - trivial stream splitter
 *
 * Copyright (C) 2004 Rik Snel <snel@phys.uu.nl>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; only version 2 of the License, not
 * any other version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. 
 *
 * $Id: tee.c 365 2004-03-14 17:19:16Z rsnel $
 */
#include <stdio.h>
#include <errno.h>
#include <string.h>
#include <stdlib.h>
#include <assert.h>
#include "../exception.h"
#include "../verbose.h"
#include "../char_io.h"
#include "../adrift.h"

#define LA_VERBOSE_TYPE Char_IO

static int init(void **priv, void *argument) {
	assert(argument);
	*priv = argument;
	return LA_CHAR_IO_OK;
}

static int filter(void *priv, char *next_in, int *avail_in,
		char *next_out, int *avail_out, int flush, char *error) {
	la_char_io_t *cout = priv;


	if (flush) {
		assert(*avail_in == 0);
		try la_char_io_flush(cout);
		catch {
			LA_ERROR("flushing in tee");
			return LA_CHAR_IO_ERROR;
		}
		endtry;
		return LA_CHAR_IO_FLUSHED;
	}

	while (*avail_in > 0 && *avail_out > 0) { 
		try la_char_io_putc(*next_in, cout);
		catch {
			LA_ERROR("writing too tee'd cout");
			return LA_CHAR_IO_ERROR;
		}
		endtry;
		*next_out++ = *next_in++;
		(*avail_in)--; (*avail_out)--;
	}

	return LA_CHAR_IO_OK;
}

static int control(void *priv, char *command, void *param) {
	LA_ERROR("tee has no commands");
	return LA_CHAR_IO_ERROR;
}

static void uninit(void *priv) {
}

la_char_filter_info_t la_char_filter_tee = { &init, &filter, &control, 
	&uninit, "tee" };

