/* zescape.c - char_io plugin for escaping ASCII NUL bytes
 *
 * Copyright (C) 2004 Rik Snel <snel@phys.uu.nl>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; only version 2 of the License, not
 * any other version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. 
 *
 * $Id: zescape.c 365 2004-03-14 17:19:16Z rsnel $
 */
#include <stdio.h>
#include <stdlib.h>
#include <assert.h>
#include <string.h>
#include <errno.h>
#include "../verbose.h"
#include "../char_io.h"

#define LA_VERBOSE_TYPE Char_IO

typedef struct {
	int escaping;
} zescape_priv_t;

static int init(void **priv, void *argument) {
	zescape_priv_t *p;

	p = malloc(sizeof(zescape_priv_t));
	if (!p) {
		LA_ERROR("%s", strerror(errno));
		return LA_CHAR_IO_ERROR;
	}

	p->escaping = 0;

	*priv = p;

	return LA_CHAR_IO_OK;
}

static int filter(void *priv, char *next_in, int *avail_in,
		char *next_out, int *avail_out, int flush,
		char *error) {
	zescape_priv_t *p = priv;
	while (*avail_in > 0 && *avail_out > 0) { 
		switch (p->escaping) {
			case 0:
				if (*next_in == '\0') {
					LA_WARNING("ASCII NULL (0x00) encountered in input, escaping");
					p->escaping = 1;
					*next_in = '\\';
				} else if (*next_in == '\\') {
					p->escaping = 2;
				}
				*next_out++ = *next_in++; 
				(*avail_in)--; (*avail_out)--;
				break;
			case 1:
				p->escaping = 0;
				*next_out++ = '0';
				(*avail_out)--;
				break;
			case 2:
				p->escaping = 0;
				*next_out++ = '\\';
				(*avail_out)--;
				break;
			default:
				assert(0);
		}
	}

	return LA_CHAR_IO_OK;
}

static int control(void *priv, char *command, void *param) {
	LA_ERROR("zescape filter has no controls");
	return LA_CHAR_IO_ERROR;
}

static void uninit(void *priv) {
	free(priv);
}

la_char_filter_info_t la_char_filter_zescape = { &init, &filter, &control, 
	&uninit, "zescape" };

