/* char_io.h - interface for handling character input/output
 *
 * Copyright (C) 2004 Rik Snel <snel@phys.uu.nl>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; only version 2 of the License, not
 * any other version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. 
 *
 * $Id: char_io.h 456 2004-04-03 13:50:14Z rsnel $
 */
#ifndef INCLUDE_LIBADRIFT_CHAR_IO_H
#define INCLUDE_LIBADRIFT_CHAR_IO_H 1

#include <stdarg.h>
#include <setjmp.h>
#include "verbose.h"
#include "array.h"

#define LA_CHAR_IO_OK		0
#define LA_CHAR_IO_ERROR	1
#define LA_CHAR_IO_EOF		2
#define LA_CHAR_IO_FLUSHED	3

#define LA_CHAR_IO_STRERR_LEN	70

typedef struct {
	int (*init)(void**, void*);
	int (*filter)(void*, char*, int*, char*, int*, int, char*);
	int (*control)(void*, char*, void*);
	void (*uninit)(void*);
	char *name;
} la_char_filter_info_t;

typedef struct {
	la_char_filter_info_t *filter_info;
	void *priv;
	char next_in;
	int avail_in;
	int bytecount;
} la_char_filter_t;

typedef struct {
	int no_filters;
	la_char_filter_t *filters;
	char strerr[LA_CHAR_IO_STRERR_LEN];
	int err;
	char *name;
} la_char_io_t;

void la_char_io_init(la_char_io_t*, char*);

void la_char_io_free(la_char_io_t*);

void la_char_filter_push(la_char_io_t*, char*, void*);

void la_char_filter_pop(la_char_io_t*);

void la_char_filter_bytecount(int*, la_char_io_t*, int);

void la_char_filter_command(la_char_io_t*, int, char*, void*);

int la_char_io_getc_nowhine(la_char_io_t*);

int la_char_io_getc(la_char_io_t*);

void la_char_io_skip(la_char_io_t*, int);

void la_char_io_read(char*, int, la_char_io_t*);

void la_char_io_putc(int, la_char_io_t*);

void la_char_io_write(char*, int, la_char_io_t*);

void la_char_io_puts(char*, la_char_io_t*);

void la_char_io_printf(la_char_io_t*, char*, ...);

void la_char_io_vprintf(la_char_io_t*, char*, va_list);

void la_char_io_flush(la_char_io_t*);

#define LA_CHAR_FILTERS 14

extern la_char_filter_info_t la_char_filter_fpin;
extern la_char_filter_info_t la_char_filter_crypt;
extern la_char_filter_info_t la_char_filter_inflate;
extern la_char_filter_info_t la_char_filter_deflate;
extern la_char_filter_info_t la_char_filter_crlf2z;
extern la_char_filter_info_t la_char_filter_linebreak;
extern la_char_filter_info_t la_char_filter_zescape;
extern la_char_filter_info_t la_char_filter_zunescape;
extern la_char_filter_info_t la_char_filter_fpout;
extern la_char_filter_info_t la_char_filter_fout;
extern la_char_filter_info_t la_char_filter_tagkill;
extern la_char_filter_info_t la_char_filter_bufout;
extern la_char_filter_info_t la_char_filter_null;
extern la_char_filter_info_t la_char_filter_tee;


#endif /* INCLUDE_LIBADRIFT_CHAR_IO_H */

