/* emem.c - functions for handling memory allocation
 *
 * Copyright (C) 2004 Rik Snel <snel@phys.uu.nl>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; only version 2 of the License, not
 * any other version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. 
 *
 * $Id: emem.c 440 2004-03-30 11:32:19Z rsnel $
 */
#include <config.h>
#include <errno.h>
#include <string.h> /* memmove, strcpy */
#include <stdlib.h> /* realloc, free */
#include <setjmp.h>
#include <assert.h>
#include "emem.h"
#include "verbose.h"

#define LA_VERBOSE_TYPE Memory

void *ecalloc(size_t nmemb, size_t size) {
	void *ptr;
	if (!(ptr = calloc(nmemb, size))) LA_THROW("%s", strerror(errno));
	return ptr;
}

void *emalloc(size_t size) {
	void *ptr;
	if (!(ptr = malloc(size))) LA_THROW("%s", strerror(errno));
	return ptr;
}

void *erealloc(void *ptr, size_t size) {
	if (!(ptr = realloc(ptr, size))) LA_THROW("%s", strerror(errno));
	return ptr;
}

char *estrdup(char *str) {
	char *new;
	assert(str);
	new = strdup(str);
	if (!new) LA_THROW("%s", strerror(errno));
	return new;
}
