/* exception.h - C++ like exception handling, based on info from
 * http://ldeniau.home.cern.ch/ldeniau/html/exception/exception.html
 *
 * Copyright (C) 2004 Rik Snel <snel@phys.uu.nl>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; only version 2 of the License, not
 * any other version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. 
 *
 * $Id: exception.h 366 2004-03-14 17:27:24Z rsnel $
 */
#ifndef INCLUDE_LIBADRIFT_EXCEPTION_H
#define INCLUDE_LIBADRIFT_EXCEPTION_H 1
#include <assert.h>
#include <setjmp.h>
#include <stdlib.h>
#include "autoclean.h"

typedef struct la_exception_context_s {
	struct la_exception_context_s *next;
	la_autoclean_t *autoclean;
	jmp_buf env;
} la_exception_context_t;

extern la_exception_context_t *current_exception_context;

#define try { \
	la_exception_context_t local_exception_context; \
	local_exception_context.next = current_exception_context; \
	local_exception_context.autoclean = NULL; \
	current_exception_context = &local_exception_context; \
	if (!setjmp(current_exception_context->env)) {

#define catch \
	} else { \
		current_exception_context = current_exception_context->next;

#define endtry \
	} \
	if (current_exception_context == &local_exception_context) { \
		la_autoclean_all(current_exception_context->autoclean); \
		current_exception_context = current_exception_context->next; \
	} \
}

#define throw() do { \
	if (!current_exception_context) exit(1); \
	la_autoclean_all(current_exception_context->autoclean); \
	current_exception_context->autoclean = NULL; \
	longjmp(current_exception_context->env, 1); \
} while (0)

#define delegate catch; throw(); endtry;

#define la_exception_cleanup_add(a, b) \
la_autoclean_t _cleanup_##a = \
	la_autoclean_add(&current_exception_context->autoclean, &_cleanup_##a, \
			(void*)a, (void(*)(void*))(b));

#define la_exception_cleanup_adda(a, b) \
la_autoclean_t _cleanup_##a = \
	la_autoclean_add(&current_exception_context->autoclean, &_cleanup_##a, \
			(void*)&a, (void(*)(void*))(b));

#endif /* INCLUDE_LIBADRIFT_EXCEPTION_H */
