/* game.c - functions for handling games
 *
 * Copyright (C) 2004 Rik Snel <snel@phys.uu.nl>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; only version 2 of the License, not
 * any other version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. 
 *
 * $Id: game.c 462 2004-04-03 22:31:26Z rsnel $
 */
#include "taf/generic.h"
#include "taf/copy.h"
#include "resolve.h"
#include "emem.h"
#include "game.h"
#include "output.h"
#include "action.h"
#include "object.h"

void la_game_restart(la_tas_t *tas) {
	int i;
	la_taf_t *t = tas->taf;
	assert(tas && t);
	tas->data.exp0 = estrdup(t->data.title);
	tas->data.exp1 = NO_ROOMS;
	tas->data.exp2 = NO_OBJECTS;
	tas->data.exp3 = NO_TASKS;
	tas->data.exp4 = NO_EVENTS;
	tas->data.exp5 = NO_CHARACTERS;
	la_action_player_room(tas, &t->data.player.location.room);
	la_taf_person_gender_copy(&tas->data.player.gender, &t->data.player.gender, "", tas);
	tas->data.player.size_capacity = 1;
	for (i = 0; i < t->data.player.size_carriable.type; i++) tas->data.player.size_capacity *= t->data.size_factor;
	tas->data.player.size_capacity *= t->data.player.size_no_carriable;
	tas->data.player.weight_capacity = 1;
	for (i = 0; i < t->data.player.weight_carriable.type; i++) tas->data.player.weight_capacity *= t->data.weight_factor;
	tas->data.player.weight_capacity *= t->data.player.weight_no_carriable;
	tas->data.rooms.ptr = la_slot_alloc(tas->data.rooms.ptr, tas->data.rooms.slots, 0, NO_ROOMS, sizeof(la_tas_room_t), 1);
	tas->data.objects.ptr = la_slot_alloc(tas->data.objects.ptr, tas->data.objects.slots, 0, NO_OBJECTS, sizeof(la_tas_object_t), 1);
	tas->data.tasks.ptr = la_slot_alloc(tas->data.tasks.ptr, tas->data.tasks.slots, 0, NO_TASKS, sizeof(la_tas_task_t), 1);
	tas->data.events.ptr = la_slot_alloc(tas->data.events.ptr, tas->data.events.slots, 0, NO_EVENTS, sizeof(la_tas_event_t), 1);
	tas->data.characters.ptr = la_slot_alloc(tas->data.characters.ptr, tas->data.characters.slots, 0, NO_CHARACTERS, sizeof(la_tas_character_t), 1);
	for (i = 0; i < NO_OBJECTS; i++) {
		if (OF(i)->is_static) {
			OS(i)->exp0 = -1;
			OS(i)->exp1 = -1;
		} else {
			la_action_dynamic_object_location(tas, i, &OF(i)->location.spec.definite);
		}
		if (OF(i)->open_info.type) {
			la_action_object_openstate(tas, i, OF(i)->open_info.spec.openable.open_state.realtype);
		}
	}
	la_action_player_posture(tas, &t->data.player.location.posture);
	for (i = 0; i < NO_CHARACTERS; i++) {
		la_action_character_room(tas, i, &CF(i)->location.room);
	}
}	

