/* intstack.h - interface for handling (out of order) compound integers
 *
 * Copyright (C) 2004 Rik Snel <snel@phys.uu.nl>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; only version 2 of the License, not
 * any other version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. 
 *
 * $Id: intstack.h 366 2004-03-14 17:27:24Z rsnel $
 */
#ifndef INCLUDE_LIBADRIFT_INTSTACK_H
#define INCLUDE_LIBADRIFT_INTSTACK_H 1
#include "array.h"

typedef struct {
	int type;
	union {
		int integer;
		char *string;
	} cnts;
	char string[256];
} la_intstack_elt_t;

typedef struct {
	la_intstack_elt_t *elts;
	int no_elts;
	la_array_t stack;
	int curr;
} la_intstack_t;

void la_intstack_init(la_intstack_t*);

void la_intstack_free(la_intstack_t*);

int la_intstack_avail(la_intstack_t*);

int la_intstack_avail_string(la_intstack_t*);

void la_intstack_push(la_intstack_t*, int, char[]);

void la_intstack_push_string(la_intstack_t*, char*, char[]);

void la_intstack_pop(la_intstack_t*);

void la_intstack_update(la_intstack_t*, int, char[]);

int la_intstack_peek(la_intstack_t*, char[]);

int la_intstack_read(la_intstack_t*, char[]);

char *la_intstack_read_string(la_intstack_t*, char[]);

#endif /* INCLUDE_LIBADRIFT_INTSTACK_H */
