/* line_out.h - interface for handling (out of order) line output to .TAF files
 *
 * Copyright (C) 2004 Rik Snel <snel@phys.uu.nl>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; only version 2 of the License, not
 * any other version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. 
 *
 * $Id: line_out.h 366 2004-03-14 17:27:24Z rsnel $
 */
#ifndef INCLUDE_LIBADRIFT_LINE_OUT_H
#define INCLUDE_LIBADRIFT_LINE_OUT_H 1

#include <stdarg.h>
#include <setjmp.h>
#include "verbose.h"
#include "array.h"
#include "char_io.h"

typedef enum { integer_e, string_e, cstart_e, spaceint_e, 
	centry_e, scstart_e, root_e } delayed_type_t;

typedef struct delayed_s {
	struct delayed_s *next;
	delayed_type_t type;
	union {
		int integer;
		char *string;
	} content;
} delayed_t;

typedef struct {
	delayed_t *crpos;
	int allowed_writes;
} output_t;

typedef struct {
	delayed_t root;
	output_t *outputs;
	int no_outputs;
	la_array_t outstack;
	int current_out;
	int blackhole;
	la_char_io_t *cout;
	int spaceint, line_no;
} la_line_out_t;

void la_line_out_init(la_line_out_t*, la_char_io_t*, int);

void la_line_out_flush(la_line_out_t*);

void la_line_out_free(la_line_out_t*);

void la_line_out_space(la_line_out_t*);

void la_line_out_put_string(la_line_out_t*, char*);

void la_line_out_skip(la_line_out_t*);

void la_line_out_put_int(la_line_out_t*, int);

void la_line_out_make_compound(la_line_out_t*, int, int, ...);

void la_line_out_push(la_line_out_t*, int);

void la_line_out_pop(la_line_out_t*, int);

void la_line_out_blackhole(la_line_out_t*);

#endif /* INCLUDE_LIBADRIFT_LINE_OUT_H */

