/* objcache.c - functions caching objects
 *
 * Copyright (C) 2004 Rik Snel <snel@phys.uu.nl>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; only version 2 of the License, not
 * any other version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. 
 *
 * $Id: objcache.c 462 2004-04-03 22:31:26Z rsnel $
 */
#include <config.h>
#include "exception.h"
#include "verbose.h"
#include "object.h"
#include "objcache.h"
#include "emem.h"
#include "taf/generic.h"

#define LA_VERBOSE_TYPE ObjCache

//	fprintf(stderr, " is_" #a "[%d]", a);
#define cachefill(a) do { \
	if (is_##a(OBJ(i))) { \
		oc->elts[i].rev##a = a; \
		oc->elts[a].a = i; \
		a++; \
	} else { \
		oc->elts[i].rev##a = -1; \
	} \
} while (0)

void la_objcache_init(la_taf_t *t) {
	la_objcache_t *oc = &t->oc;
	la_tas_t ts, *tas = &ts;
	int i;
	int surface = 0;
	int container = 0;
	int chairlike = 0;
	int bedlike = 0;
	int clothing = 0;
	int stateful = 0;
	int dynamic = 0;
	assert(oc);
	ts.taf = t;
	oc->no_elts = NO_OBJECTS;
	oc->elts = ecalloc(oc->no_elts, sizeof(la_objcache_elt_t));
	for (i = 0; i < oc->no_elts; i++) {
		cachefill(surface);
		cachefill(container);
		cachefill(chairlike);
		cachefill(bedlike);
		cachefill(clothing);
		cachefill(stateful);
		cachefill(dynamic);
	}
	for (; surface < oc->no_elts; surface++) oc->elts[surface].surface = -1;
	for (; container < oc->no_elts; container++) oc->elts[container].container = -1;
	for (; chairlike < oc->no_elts; chairlike++) oc->elts[chairlike].chairlike = -1;
	for (; bedlike < oc->no_elts; bedlike++) oc->elts[bedlike].bedlike = -1;
	for (; clothing < oc->no_elts; clothing++) oc->elts[clothing].clothing = -1;
	for (; stateful < oc->no_elts; stateful++) oc->elts[stateful].stateful = -1;
	for (; dynamic < oc->no_elts; dynamic++) oc->elts[dynamic].dynamic = -1;
}

void la_objcache_free(la_taf_t *t) {
	free(t->oc.elts);
}
