/* output.c - output functions
 *
 * Copyright (C) 2004 Rik Snel <snel@phys.uu.nl>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; only version 2 of the License, not
 * any other version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. 
 *
 * $Id: game.h 454 2004-04-01 19:45:12Z rsnel $
 */
#include <string.h>
#include "taf/generic.h"
#include "resolve.h"
#include "char_io.h"
#include "object.h"

void la_output_red_print(la_tas_t *tas, char *format, ...) {
	va_list ap;
	va_start(ap, format);
	la_char_io_vprintf(tas->red, format, ap);
	la_char_io_flush(tas->red);
	va_end(ap);
}

void la_output_green_print(la_tas_t *tas, char *format, ...) {
	va_list ap;
	va_start(ap, format);
	la_char_io_vprintf(tas->green, format, ap);
	la_char_io_flush(tas->green);
	va_end(ap);
}

void la_output_green_firstcapital(la_tas_t *tas, char *string) {
	if (string == NULL) return;
	if (*string == '\0') return;
	if (*string >= 'a' && *string <= 'z') la_char_io_putc(*string - 'a' + 'A', tas->green);
	else la_char_io_putc(*string, tas->green);
	la_output_green_print(tas, "%s", string + 1);
}

void la_output_output(la_tas_t *tas, la_taf_output_t *output) {
	int i;
	for (i = 0; i < output->lines.no_lines - 1; i++) 
		la_output_green_print(tas, "%s\n", output->lines.lines[i]);
	if (output->lines.no_lines > 0) 
		la_output_green_print(tas, "%s", output->lines.lines[output->lines.no_lines - 1]);
}

void la_output_descriptions(la_tas_t *tas, la_taf_description_t *d) {
	la_output_output(tas, &d->desc_if);
}

void la_output_object_ident(la_tas_t *tas, int i, int capital) {
	if (capital) {
		la_output_green_firstcapital(tas, OF(i)->prefix);
		la_output_green_print(tas, " %s", OF(i)->name);
	} else la_output_green_print(tas, "%s %s", OF(i)->prefix, OF(i)->name);
}

void la_output_object_theident(la_tas_t *tas, int i, int capital) {
	int skip = 0;
	if (!strncmp(OF(i)->prefix, "a ", 2)) skip = 2;
	if (!strncmp(OF(i)->prefix, "some ", 5)) skip = 5;
	if (skip > 0) {
		if (capital) la_output_green_print(tas, "The ");
		else la_output_green_print(tas, "the ");
		if (strlen(OF(i)->prefix) != skip) la_output_green_print(tas, "%s ", OF(i)->prefix + skip);
		la_output_green_print(tas, "%s", OF(i)->name);
	} else {
		if (capital) {
			la_output_green_firstcapital(tas, OF(i)->prefix);
			la_output_green_print(tas, " %s", OF(i)->name);
		} else la_output_green_print(tas, "%s %s", OF(i)->prefix, OF(i)->name);
	}
}

void la_output_object_list(la_tas_t *tas, int *objects, int no_objects, int c) {
	int i;
	for (i = 0; i < no_objects; i++) {
		la_output_object_ident(tas, objects[i], c&&(!i));
		if (i < no_objects - 2) la_output_green_print(tas, ", ");
		else if (i < no_objects - 1) la_output_green_print(tas, " and ");
	}
}

void la_output_room_descriptions(la_tas_t *tas, int room) {
	la_array_t a, b;
	int i;
	int no_strings = 0;
	char **strings = NULL;
	int no_stuff = 0;
	int *stuff = NULL;
	la_output_descriptions(tas, RF(room)->descriptions.ptr);
	la_array_ninit(&a, stuff, NULL);
	la_array_ninit(&b, strings, NULL);
	la_resolve_objects_directly_in_same_room_as_player(tas, &a);
	if (no_stuff > 0) {
		la_output_green_print(tas, " Also here");
		if (OF(stuff[0])->is_plural) la_output_green_print(tas, " are ");
		else la_output_green_print(tas, " is ");
		la_output_object_list(tas, stuff, no_stuff, 0);
		la_output_green_print(tas, ".");
	}
	la_array_empty(&a);
	la_resolve_characters_in_same_room_as_player(tas, &a);
	for (i = 0; i < no_stuff; i++) {
		la_output_green_print(tas, " ");
		la_output_descriptions(tas, CF(stuff[i])->present_desc.ptr);
	}
	la_resolve_allowed_directions(tas, &b);
	if (no_strings > 0) {
		la_output_green_print(tas, " ");
		la_output_green_firstcapital(tas, tas->i);
		la_output_green_print(tas, "can ");
		if (no_strings == 1) la_output_green_print(tas, "only ");
		la_output_green_print(tas, "move ");
		for (i = 0; i < no_strings; i++) {
			la_output_green_print(tas, "%s", strings[i]);
			if (i < no_strings - 2) la_output_green_print(tas, ", ");
			else if (i < no_strings - 1) la_output_green_print(tas, " and ");
		}
		la_output_green_print(tas, ".");
	}
	la_output_green_print(tas, "\n\n");
	la_array_free(&a);
	la_array_free(&b);
}

void la_output_intro(la_tas_t *tas) {
	la_output_red_print(tas, "%s\n", tas->taf->data.title);
	la_output_output(tas, &tas->taf->data.intro);
	la_output_green_print(tas, "\n");
	if (tas->taf->data.display_first_room) {
		la_output_room_descriptions(tas, tas->data.player.location.room.spec.room);
	}
}

/* output caused by this functions attempts to be identical to that of
 * ADRIFT Runner Version 3.90 Release 20 */
void la_output_inventory(la_tas_t *tas) {
	la_array_t a, b, c;
	int i;
	int no_objects = 0;
	int *objects = NULL;
	int no_supported = 0;
	int *supported = NULL;
	int no_contained = 0;
	int *contained = NULL;
	la_output_green_firstcapital(tas, tas->iam);
	la_array_ninit(&a, objects, NULL);
	la_array_ninit(&b, supported, NULL);
	la_array_ninit(&c, contained, NULL);
	la_resolve_all_worn_objects(tas, &a);
	if (no_objects > 0) {
		la_output_green_print(tas, "wearing ");
		la_output_object_list(tas, objects, no_objects, 0);
		la_output_green_print(tas, ", and %s", tas->iam);
	}
	la_output_green_print(tas, "carrying ");
	la_array_empty(&a);
	la_resolve_all_held_objects(tas, &a);
	if (no_objects > 0) {
		la_output_object_list(tas, objects, no_objects, 0);
		la_output_green_print(tas, ".");
		for (i = 0; i < no_objects; i++) {
			if (is_container(OF(objects[i]))) la_resolve_all_objects_in_container(tas, &c, objects[i]);
			if (is_surface(OF(objects[i]))) la_resolve_all_objects_on_surface(tas, &b, objects[i]);
			if (no_supported == 0 && no_contained != 0) {
				la_output_green_print(tas, " Inside ");
				la_output_object_theident(tas, objects[i], 0);
				if (OF(contained[0])->is_plural) la_output_green_print(tas, " are ");
				else la_output_green_print(tas, " is ");
				la_output_object_list(tas, contained, no_contained, 0);
				la_output_green_print(tas, ".");
			}
			if (no_supported != 0 && no_contained == 0) {
				la_output_green_print(tas, " On ");
				la_output_object_theident(tas, objects[i], 0);
				if (OF(supported[0])->is_plural) la_output_green_print(tas, " are ");
				else la_output_green_print(tas, " is ");
				la_output_object_list(tas, supported, no_supported, 0);
				la_output_green_print(tas, ".");
			}
			if (no_supported != 0 && no_contained != 0) {
				la_output_green_print(tas, " ");
				la_output_object_list(tas, supported, no_supported, 1);
				if (OF(supported[i])->is_plural || no_supported > 1) 
					la_output_green_print(tas, " are on ");
				else la_output_green_print(tas, " is on ");
				la_output_object_theident(tas, objects[i], 0);
				la_output_green_print(tas, ", and inside");
				if (OF(contained[0])->is_plural) la_output_green_print(tas, " are ");
				else la_output_green_print(tas, " is ");
				la_output_object_list(tas, contained, no_contained, 0);
				la_output_green_print(tas, ".");
			}
		}
		la_output_green_print(tas, "\n");
	} else la_output_green_print(tas, "nothing.\n");
	la_array_free(&a);
	la_array_free(&b);
	la_array_free(&c);
}

