/* slot.h - functions for flexible array parsing
 *
 * Copyright (C) 2004 Rik Snel <snel@phys.uu.nl>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; only version 2 of the License, not
 * any other version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. 
 *
 * $Id: slot.c 465 2004-07-12 09:16:39Z rsnel $
 */
#include <assert.h>
#include <string.h>
#include "slot.h"
#include "emem.h"
#include "verbose.h"

#define LA_VERBOSE_TYPE Slot 

int la_slot_index(la_slot_info_t slots[], int slot) {
	int total = 0, i;
	assert(slots);
	assert(slot >= 0 && slot <= LA_SLOT_MAX);
	for (i = 0; i < slot; i++) total += slots[i].size;
	return total;
}

int la_slot_total(la_slot_info_t slots[]) {
	return la_slot_index(slots, LA_SLOT_MAX);
}

void *la_slot_alloc(void *array, la_slot_info_t slots[], int slot, int count, int size, int fixed) {
	int old_count, move;
	assert(slots && count >= 0 && size > 0);
	assert(slot >= 0 && slot < LA_SLOT_MAX);
	if (slots[slot].size == count && slots[slot].fixed == fixed) return array; /* slot already allocated (or empty) */
	if (slots[slot].size != 0) LA_THROW("impossible error or slot reused");
	old_count = la_slot_total(slots);
	slots[slot].size = count;
	slots[slot].fixed = fixed;
	array = erealloc(array, (old_count + count)*size);
	move = old_count - la_slot_index(slots, slot);
	if (move) memmove(array + la_slot_index(slots, slot + 1)*size,
			array + la_slot_index(slots, slot)*size,
			move*size);
	/* initialize to zeroes */
	memset(array + la_slot_index(slots, slot)*size, 0, size*count);
	return array;
}
