/* common.h - interface for handling stuff in .taf files
 *
 * Copyright (C) 2004 Rik Snel <snel@phys.uu.nl>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; only version 2 of the License, not
 * any other version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. 
 *
 * $Id: common.h 408 2004-03-25 17:56:00Z rsnel $
 */
#ifndef INCLUDE_LIBADRIFT_TAF_COMMON_H
#define INCLUDE_LIBADRIFT_TAF_COMMON_H 1

#include "../verbose.h"
#include "../char_io.h"
#include "../line_in.h"
#include "../line_out.h"
#include "../slot.h"

#ifdef NDEBUG
#define LA_COMMON_DBG_ARGS
#define LA_COMMON_DBG_ARG
#else
#define LA_COMMON_DBG_ARGS , char *prefix, char *name
#define LA_COMMON_DBG_ARG , char *name
#endif

typedef char* la_taf_line_t;

typedef struct {
	la_taf_line_t *lines;
	int no_lines;
} la_taf_lines_t;

typedef struct {
	void *ptr;
	la_slot_info_t slots[LA_SLOT_MAX];
	int curr;
} la_taf_array_t;

int la_taf_array_total(la_taf_array_t*);

void la_taf_postlines_parse(la_taf_lines_t*, la_line_in_t* LA_COMMON_DBG_ARG, char*);

void la_taf_lines_parse(la_taf_lines_t*, la_line_in_t* LA_COMMON_DBG_ARG, int);

int la_taf_noneg_parse(la_line_in_t* LA_COMMON_DBG_ARGS);

int la_taf_integt_parse(la_line_in_t*, int LA_COMMON_DBG_ARGS);

int la_taf_range_parse(la_line_in_t*, int, int LA_COMMON_DBG_ARGS);

int la_taf_boolean_parse(la_line_in_t* LA_COMMON_DBG_ARGS);

void la_taf_postlines_write(la_taf_lines_t*, la_line_out_t* LA_COMMON_DBG_ARG, char*);

void la_taf_lines_write(la_taf_lines_t*, la_line_out_t* LA_COMMON_DBG_ARG);

void la_taf_lines_free(la_taf_lines_t* LA_COMMON_DBG_ARG);

void la_taf_lines_copy(la_taf_lines_t*, la_taf_lines_t* LA_COMMON_DBG_ARG);

#endif /* INCLUDE_LIBADRIFT_TAF_COMMON_H */
