/* generate.c - program to generate parsers et al.
 *
 * Copyright (C) 2004 Rik Snel <snel@phys.uu.nl>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; only version 2 of the License, not
 * any other version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. 
 *
 * $Id: generate.c 476 2004-07-18 21:39:48Z rsnel $
 */
#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <string.h>
#include <assert.h>
#include "config.h"

#define sfy(a)			#a
#define postlines(a,b) 		postlines_c(#a,#b)
#define line(a)			line_c(#a)
#define buffer_for_later(a,b,c)	buffer_for_later_c(#a,#b,#c)

#define noneg(a) 		noneg_c(#a)
#define intg(a) 		intg_c(#a, "int")
#define integt(a,b) 		integt_c(#a,#b)
#define string(a) 		string_c(#a, "string")
#define boolean(a) 		boolean_c(#a)
#define range(a,c)		range_c(#a,"0",#c,0)
#define legenda(a)		legenda_c(#a)

#define stringexp(a)		stringexp_c(#a)
#define expi(...)		expi_c(__VA_ARGS__)
#define exps(...)		exps_c(__VA_ARGS__)
#define skip(a)			skip_c()

#define fstruct(a,b)		stuff_c(#a,#b,"",'f')
#define fastruct(a,b,c)		stuff_c(#a,#b,sfy(_##c),'f')
#define floop(a,b,c,d)		loop_c(#a,#b,"",#c,d,1,'f')
#define faloop(a,b,c,d,e)	loop_c(#a,#b,sfy(_##c),#d,e,1,'f')
#define autofloop(a,b,c)	autoloop_c(#a,#b,"",c,'f')
#define autofaloop(a,b,c,d)	autoloop_c(#a,#b,sfy(_##c),d,'f')

#define oldfstruct(a,b)		oldstuff_c(#a,#b,"",'f')
#define oldfastruct(a,b,c)	oldstuff_c(#a,#b,sfy(_##c),'f')
#define oldfloop(a,b,c,d)	oldloop_c(#a,#b,"",#c,d,1,'f')
#define oldfaloop(a,b,c,d,e)	oldloop_c(#a,#b,sfy(_##c),#d,e,1,'f')
#define oldautofloop(a,b,c)	oldautoloop_c(#a,#b,"",c,'f')
#define oldautofaloop(a,b,c,d)	oldautoloop_c(#a,#b,sfy(_##c),d,'f')

#define endfstruct()		endstuff_c()
#define endfastruct()		endstuff_c()
#define endfloop()		endstuff_c()
#define endfaloop()		endstuff_c()
#define endautofloop()		endstuff_c()
#define endautofaloop()		endstuff_c()

#define sstruct(a,b)		stuff_c(#a,#b,"",'s')
#define sastruct(a,b,c)		stuff_c(#a,#b,sfy(_##c),'s')
#define sloop(a,b,c,d)		loop_c(#a,#b,"",#c,d,1,'s')
#define saloop(a,b,c,d,e)	loop_c(#a,#b,sfy(_##c),#d,e,1,'s')
#define autosloop(a,b,c)	autoloop_c(#a,#b,"",c,'s')
#define autosaloop(a,b,c,d)	autoloop_c(#a,#b,sfy(_##c),d,'s')

#define oldsstruct(a,b)		oldstuff_c(#a,#b,"",'s')
#define oldsastruct(a,b,c)	oldstuff_c(#a,#b,sfy(_##c),'s')
#define oldsloop(a,b,c,d)	oldloop_c(#a,#b,"",#c,d,1,'s')
#define oldsaloop(a,b,c,d,e)	oldloop_c(#a,#b,sfy(_##c),#d,e,1,'s')
#define oldautosloop(a,b,c)	oldautoloop_c(#a,#b,"",c,'s')
#define oldautosaloop(a,b,c,d)	oldautoloop_c(#a,#b,sfy(_##c),d,'s')

#define endsstruct()		endstuff_c()
#define endsastruct()		endstuff_c()
#define endsloop()		endstuff_c()
#define endsaloop()		endstuff_c()
#define endautosloop()		endstuff_c()
#define endautosaloop()		endstuff_c()

#define iff(a)			iff_c(#a, "")
#define	parse_stop		condition_c("0")
#define condition(a)		condition_c(#a)
#define eliff(a)		iff_c(#a, "else ")
#define aswitch()		aswitch_c()
#define kswitch()		kswitch_c()

#define endiff()		endiff_c()
#define endeliff()		endiff_c()
#define endaswitch(a)		endaswitch_c()
#define endkswitch(a)		endaswitch_c()

#define compound(a,...)		compound_c(a,__VA_ARGS__)
#define space()			space_c()
#define push(a)			push_c(#a)
#define pop(a)			pop_c(#a)
#define fakeint(a)		fake_c(#a, 0)
#define fakestring(a)		fake_c(a, 1)

#define ENTER fprintc(stdout, ".\nk%c\n'a\na\n", 'b' + struct_depth);
#define EXIT  fprintc(stdout, "\n.\nka\n'%c\na\n", 'b' + struct_depth);

#ifndef NDEBUG
#define MPREFIX(a) prt(stdout, "snprintf(new_prefix, 128, " a ");", getuprfx(), name);
#define SPREFIX MPREFIX("\"%%s%s%s\", prefix")
#define NWPR ", new_prefix"
#else
#define MPREFIX(a)
#define SPREFIX
#define NWPR
#endif

static int arbeit;
static char *actions[8] = { "header", "parse", "free", "write", 
	"xtra_parse", "xtra_write", "copy", "widget" };
#if 0
static char *ucactions[7] = { "HEADER", "PARSE", "FREE", "WRITE", 
	"XTRA_PARSE", "XTRA_WRITE", "COPY" };
#endif
#ifndef NDEBUG
static char *actionings[8] = { "header", "parsing", "freeing", "writing", 
	"xtra_parsing", "xtra_writing", "copying", "widgeting" };
#endif
/* arbitrary limit 25 inherited from ed (26 labels) */
#define MAX_STRUCT_DEPTH 25
#define MAX_STRUCT_INFOS 512
#define MAX_UNION_INFOS 1
#define MAX_CASE_INFOS 32
#define MAX_NAME_INFOS 256
#define CURST struct_infos[struct_stack[struct_depth]]
#define ADVST do { \
	if (struct_depth + 1 >= MAX_STRUCT_DEPTH) { \
		fprintf(stderr, "max struct depth reached\n"); \
		exit(1); \
	} \
	struct_depth++; \
	if (free_struct >= MAX_STRUCT_INFOS) { \
		fprintf(stderr, "max # of structs reached\n"); \
		exit(1); \
	} \
	struct_stack[struct_depth] = free_struct++; \
} while (0)
#define CURU CURST.union_infos[CURST.free_union]
#define ADVU do { \
	if (CURST.free_union + 1 >= MAX_UNION_INFOS) { \
		fprintf(stderr, "max # of unions reached\n"); \
		exit(1); \
	} \
	CURST.free_union++; \
} while (0)
#define CURNM CURST.name_infos[CURST.free_name]
#define ADVNM do { \
	if (CURST.free_name + 1 >= MAX_NAME_INFOS) { \
		fprintf(stderr, "max # of names reached\n"); \
		exit(1); \
	} \
	CURST.free_name++; \
} while (0)
#define CURCS CURU.case_infos[CURU.free_case]
#define ADVCS do { \
	if (CURU.free_case + 1 >= MAX_CASE_INFOS) { \
		fprintf(stderr, "max # of cases reached\n"); \
		exit(1); \
	} \
	CURU.free_case++; \
} while (0)
struct name_info {
	char name[256];
	int type; /* 0 none, 1 int, 2 string, 3 struct, 4 array,
		     5 lines, 6 buffer_for_later, 7 bool */
	char *sname; /* if any */
	char *mod;
	char fs;
};
struct case_info {
	char *name;
	struct name_info nfo;
	int input;
	int first;
};
struct union_info {
	int free_case;
	int last_case;
	int free_legal_case;
	int last_legal_case;
	struct case_info case_infos[MAX_CASE_INFOS];
	int type;
	int useless;
	int legal_useless;
	int named;
	char *condition;
	int case_open;
	int free;
};
static struct struct_info {
	char *name;
	char *mod;
	struct union_info union_infos[MAX_UNION_INFOS];
	struct case_info case_infos[MAX_CASE_INFOS];
	int no_case_infos;
	int indentlevel;
	int in_union;
	int has_union;
	int utype;
	int free_union;
	struct name_info name_infos[MAX_NAME_INFOS];
	int free_name;
	int last_name;
	struct name_info decl_infos[MAX_NAME_INFOS];
	int free_decl;
	int expcount;
	char fs;
	/* deps */
	struct struct_info *deps[128];
	int no_deps;
	struct struct_info *gdeps[128];
	int no_gdeps;
	int resolved;
	int resolving;
	int gresolved;
	int gresolving;
	int b4ldep;
	int free;
	int cfree;
} struct_infos[MAX_STRUCT_INFOS];
static int struct_stack[MAX_STRUCT_DEPTH];
static int struct_depth = -1;
static int free_struct;
static int no_structs;
static int in_expect = 0;
static int disable_output = 0;
static int simple_indent = 0;
void struct_verbose(struct struct_info *info) {
	int i;
	assert(struct_depth >= 0);
	for (i = 0; i < struct_depth; i++) fprintf(stderr, "*"); 
	fprintf(stderr, "name=%s, mod=%s\n",
			info->name, info->mod);
}
struct struct_info *get_struct_info(char *name, char *mod, char fs) {
	int i = 0;
	int max = no_structs;
	if (max == 0) max = free_struct;
	do {
		if (!strcmp(name, struct_infos[i].name) && struct_infos[i].fs == fs) {
			if (mod) {
				if (!strcmp(mod, struct_infos[i].mod))
					return &struct_infos[i];
			} else return &struct_infos[i];
		}
	} while (++i < max);
	return NULL;
}

/* fullname */
static char fname[256];
static char fsname[256];
static char wname[256];
char *getuprfx() {
	if (!CURST.in_union) return "";
	assert(CURST.free_union != -1);
	return "spec."; 
}

char *getfn(char *name) {
	snprintf(fname, 256, "d->%s%s", (struct_depth < 0 && simple_indent > 2)?"spec.":getuprfx(), name);
	return fname;
}

char *getfns(char *name) {
	snprintf(fsname, 256, "s->%s%s", (struct_depth < 0 && simple_indent > 2)?"spec.":getuprfx(), name);
	return fsname;
}

char *getwn(char *name) {
	snprintf(wname, 256, "w->%s%s", (struct_depth < 0 && simple_indent > 2)?"spec.":getuprfx(), name);
	return wname;
}

void print_header(FILE *fp, char *name) {
	fprintf(fp, "/* ");
	fprintf(fp, name);
	fprintf(fp, "\n");
	fprintf(fp,
" *\n"
" * This file is autogenerated from ta[fs].spec by generate.c. Do not edit.\n"
" * Copyright (C) 2004 Rik Snel <snel@phys.uu.nl>\n"
" *\n"
" * This program is free software; you can redistribute it and/or modify\n"
" * it under the terms of the GNU General Public License as published by\n"
" * the Free Software Foundation; only version 2 of the License, not\n"
" * any other version.\n"
" *\n"
" * This program is distributed in the hope that it will be useful,\n"
" * but WITHOUT ANY WARRANTY; without even the implied warranty of\n"
" * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the\n"
" * GNU General Public License for more details.\n"
" *\n"
" * You should have received a copy of the GNU General Public License\n"
" * along with this program; if not, write to the Free Software\n"
" * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. \n"
" */\n");
}

static void vfprintc(FILE *fp, char *format, va_list ap) {
	assert(disable_output >= 0);
	if (!disable_output) vfprintf(fp, format, ap);
}

static void fprintc(FILE *fp, char *format, ...) {
	va_list ap;
	va_start(ap, format);
	vfprintc(fp, format, ap);
	va_end(ap);
}

static void indent(FILE *fp) {
	int i, j = (struct_depth < 0)?simple_indent:CURST.indentlevel;
	for (i = 0; i < j; i++) fprintc(fp, "\t");
}

static void vprt(FILE *fp, char *format, va_list ap) {
	indent(fp);
	vfprintc(fp, format, ap);
	fprintc(fp, "\n");
}

static void prt(FILE *fp, char *format, ...) {
	va_list ap;
	va_start(ap, format);
	vprt(fp, format, ap);
	va_end(ap);
}

static void fprt(FILE *fp, char *format, ...) {
	va_list ap;
	va_start(ap, format);
	vprt(fp, format, ap);
	va_end(ap);
	if (struct_depth < 0) simple_indent++;
	else CURST.indentlevel++;
}

static void bprt(FILE *fp, char *format, ...) {
	va_list ap;
	if (struct_depth < 0) simple_indent--;
	else CURST.indentlevel--;
	va_start(ap, format);
	vprt(fp, format, ap);
	va_end(ap);
}

static void dprt(FILE *fp, char *format, ...) {
	va_list ap;
	va_start(ap, format);
	vprt(fp, format, ap);
	va_end(ap);
	if (struct_depth < 0) simple_indent--;
	else CURST.indentlevel--;
}

#define ORIG (!strcmp(CURST.mod, ""))
void close_case() {
	assert(CURU.free_case == -1 || CURU.named == 1);
	if (!CURU.case_open) return;
	CURU.case_open = 0;
	if (CURU.free_case > -1) {
		if (arbeit > 0 && arbeit < 10) dprt(stdout, "break;");
		if (arbeit == 20 && CURCS.nfo.type == 0) CURU.useless++;
		if (arbeit == 20 && CURCS.nfo.type == 0 && !CURU.condition) CURU.legal_useless++;
	}
}

struct case_info *get_case_info(char *name) {
	int i, max;
	if (CURU.last_case == 0) max = CURU.free_case;
	assert(CURU.free_case >= 0);
	for (i = 0; i < CURU.free_case; i++) {
		if (!strcmp(name, CURU.case_infos[i].name)) 
			return &CURU.case_infos[i];
	}
	return NULL;
}

void autoskip_c(void) {
	if (arbeit == 1) prt(stdout, "la_line_in_skip(lin);");
	else if (arbeit == 3) prt(stdout, "la_line_out_skip(lout);");
}

void fixindex(char *name) {
	struct struct_info *sinfo;
	int i;
	sinfo = get_struct_info(CURST.name, NULL, CURST.fs);
	assert(sinfo);
	for (i = 0; i < sinfo->no_case_infos; i++) 
		if (!strcmp(name, sinfo->case_infos[i].name))
			break;
	if (i >= sinfo->no_case_infos) {
		fprintf(stderr, "correct index not found, req: %s\n", name);
		exit(1);
	}
	prt(stdout, "d->realtype = %d;", i);	
}
	
int findex(char *name) {
	int i;
	assert(MAX_UNION_INFOS < 10);
	for (i = 0; i <= MAX_UNION_INFOS; i++) 
		if (strlen(name) > 5) 
			if (!strncmp("spec.", name, 5)) 
				return i;
	return i;
}

void add_global_case_name(char *name) {
	struct struct_info *sinfo;
	int i;
	/* find right struct */
	sinfo = get_struct_info(CURST.name, NULL, CURST.fs);
	assert(sinfo);
	for (i = 0; i < sinfo->no_case_infos; i++) {
		if (!strcmp(name, sinfo->case_infos[i].name)) {
			/* name already declared */
			return;
		}
	}
	sinfo->case_infos[sinfo->no_case_infos].name = name;
	sinfo->no_case_infos++;
	//fprintf(stderr, "set has_union == 1: %s\n", CURST.name);
	sinfo->has_union = 1;
}

void unioncheck(char *name, int type, char *sname, char *mod, char fs) {
	int unionc;
	struct struct_info *sinfo;
	unionc = findex(name);
	if (unionc >= MAX_UNION_INFOS) return;
	if (arbeit == 20) {
		sinfo = get_struct_info(CURST.name, NULL, CURST.fs);
		assert(sinfo);
		strncpy(sinfo->case_infos[sinfo->no_case_infos].nfo.name, name + 5, 255);
		sinfo->case_infos[sinfo->no_case_infos].nfo.sname = sname;
		sinfo->case_infos[sinfo->no_case_infos].nfo.type = type;
		sinfo->case_infos[sinfo->no_case_infos].nfo.fs = fs;
		//fprintf(stderr, "possibly adding %s to %s\n", name, CURST.name);
		add_global_case_name(name + 5);
		return;
	}
	if (ORIG) assert(0);
	fixindex(name + 5);
}

void namecheck(char *name, int type, char *sname, char *mod, char fs) {
	int i;
	struct struct_info *sinfo;
	ADVNM;
	if (arbeit == 20) {
		strncpy(CURNM.name, name, 255);
		CURNM.sname = sname;
		CURNM.mod = mod;
		CURNM.type = type;
		CURNM.fs = fs;
		if (!type) return;
		if (findex(name) < MAX_UNION_INFOS) return;
		/* find first instance of current struct */
		sinfo = get_struct_info(CURST.name, NULL, CURST.fs); 
		assert(sinfo);
		for (i = 0; i <= sinfo->free_decl; i++) {
			if (!strcmp(name, sinfo->decl_infos[i].name)) {
				if (strcmp(sname, sinfo->decl_infos[i].sname) ||
						sinfo->decl_infos[i].type != type) {
					fprintf(stderr, "duplicate of wrong type\n");
					exit(1);
				}
				/* name already declared */
				return;
			}
		}
		sinfo->free_decl++;
		strncpy(sinfo->decl_infos[sinfo->free_decl].name, name, 255);
		sinfo->decl_infos[sinfo->free_decl].sname = sname;
		sinfo->decl_infos[sinfo->free_decl].mod = mod;
		sinfo->decl_infos[sinfo->free_decl].type = type;
		sinfo->decl_infos[sinfo->free_decl].fs = fs;
	} 
}

void add_possible_case(char *name, int data, int input, int type, char *sname, char *mod, char fs) {
	struct case_info *info;
	struct struct_info *sinfo;
	if (!CURST.in_union) {
		unioncheck(name, type, sname, mod, fs);
		namecheck(name, type, sname, mod, fs);
		return;
	}
	if (CURU.free_case == -1 || CURU.named == 1) {
		CURST.case_infos[CURST.no_case_infos].nfo.type = 0;
		close_case();
		sinfo = get_struct_info(CURST.name, NULL, CURST.fs);
		assert(sinfo);
		sinfo->case_infos[sinfo->no_case_infos].nfo.type = 0;

		if (arbeit == 20 && CURU.condition == NULL) {
			CURU.free_legal_case++;
		}
		CURU.case_open = 1;
		ADVCS;
		if (arbeit == 20) {
			CURCS.input = 0;
			CURCS.first = 0;
			CURCS.name = NULL;
		}
		if (arbeit > 0 && arbeit < 10) {
			prt(stdout, "case %d: /* %s */", CURU.free_case, CURCS.name);
			CURST.indentlevel++;
			if (CURU.type == 1 && !CURU.legal_useless && CURCS.input == 0) {
				autoskip_c();
			}
			if (CURU.type == 1 && /*!ORIG &&*/ arbeit == 1) 
				fixindex(CURCS.name);
		}
	}
	CURU.named = 0;
	if (arbeit == 20) {
		CURCS.input += input;
	}
	if (type != 0) {
		strncpy(CURCS.nfo.name, name, 255);
		CURCS.nfo.sname = sname;
		CURCS.nfo.type = type;
		sinfo = get_struct_info(CURST.name, NULL, CURST.fs);
		assert(sinfo);
		strncpy(sinfo->case_infos[sinfo->no_case_infos].nfo.name, name, 255);
		sinfo->case_infos[sinfo->no_case_infos].nfo.sname = sname;
		sinfo->case_infos[sinfo->no_case_infos].nfo.type = type;
		sinfo->case_infos[sinfo->no_case_infos].nfo.fs = fs;
	}
	if (in_expect) return;
	if (strlen(name) != 0) {
		if (arbeit == 20) {
			CURCS.name = name;
			info = get_case_info(name);
			if (!info) CURCS.first = 1;
			add_global_case_name(name);
		}
		CURU.named = 1;
	}
}

void funcdecl(FILE *fp, char *name, char *mod, int arbt, char fs) {
	fprintc(fp, "void la_ta%c_%s%s_%s(la_ta%c_%s_t*, ", fs, name, mod,
			actions[arbt], fs, name);

	switch (arbt) {
		case 1:
			fprintc(fp, "la_line_in_t*, ");
			break;
		case 3:
			fprintc(fp, "la_line_out_t*, ");
			break;
		case 4:
		case 5:
			fprintc(fp, "la_char_io_t*, ");
			break;
		case 6:
			fprintc(fp, "la_ta%c_%s_t*, ", fs, name);
			break;
		case 7:
			fprintc(fp, "la_ta%c_%s_widget_t*, GtkBox*, ", fs, name);
			break;
	}

#ifndef NDEBUG
	fprintc(fp, "char*, ");
#endif
	fprintc(fp, "la_tas_t*);\n");
}

void funcstart(FILE *fp, char *name, char *mod, int arbt, char fs) {
	fprintc(fp, "void la_ta%c_%s%s_%s(la_ta%c_%s_t *d, ", fs,
				name, mod, actions[arbt], fs, name);
	switch (arbt) {
		case 1:
			fprintc(fp, "la_line_in_t *lin, ");
			break;
		case 3:
			fprintc(fp, "la_line_out_t *lout, ");
			break;
		case 4:
		case 5:
			fprintc(fp, "la_char_io_t *cio, ");
			break;
		case 6:
			fprintc(fp, "la_ta%c_%s_t *s, ", fs, name);
			break;
		case 7:
			fprintc(fp, "la_ta%c_%s_widget_t *w, GtkBox *box, ", fs, name);
			break;
	}
#ifndef NDEBUG
	fprintc(fp, "char *prefix, ");
#endif
	fprintc(fp, "la_tas_t *tas) {\n");
	if (struct_depth < 0) simple_indent++;
	else CURST.indentlevel++;
#ifndef NDEBUG
	prt(fp, "char new_prefix[128] __attribute__ ((unused));"); 
	prt(fp, "la_taf_t *t __attribute__ ((unused)) = tas->taf;");
	prt(fp, "LA_DEBUG(\"%s %%.*s %%p\", strlen(prefix) - 1, prefix, d);", actionings[arbt]);
#endif
}

void arraddr(FILE *fp, char *fn, char *sname, char *index, char fs) {
	int tmp;
	tmp = strlen(index);
	fprintc(fp, "%s%s", tmp?"":"&", fn);
	if (tmp) {
		fprintc(fp, ".ptr + %s*sizeof(la_ta%c_%s_t)", index, fs, sname);
	}
	fprintc(fp, ", ");
}

void funccall(FILE *fp, char *sname, char *name, char *index, char *mod, int arbt, char fs) {
	indent(fp);
	fprintc(fp, "la_ta%c_%s%s_%s(", fs, name, mod, actions[arbt]);
	arraddr(fp, getfn(sname), name, index, fs);
	switch (arbt) {
		case 1:
			fprintc(fp, "lin, ");
			break;
		case 3:
			fprintc(fp, "lout, ");
			break;
		case 4:
		case 5:
			fprintc(fp, "cio, ");
			break;
		case 6:
			arraddr(fp, getfns(sname), name, index, fs);
			break;
		case 7:
			if (CURST.in_union) {
				//if (CURU.type == 0) fprintf(fp, "&%s, la_taf_leftwidget_widget(w->type, box), ", getwn(sname));
				//else fprintf(fp, "&%s, la_taf_leftwidget_widget(w->pwtype, box), ", getwn(sname));
				fprintf(fp, "&%s, w->specbox, ", getwn(sname));
			} else fprintf(fp, "&%s, la_taf_enframe_widget(\"%s\", box), ", getwn(sname), sname);
			break;
	}
#ifndef NDEBUG
	fprintc(fp, "new_prefix, ");
#endif
	fprintc(fp, "tas);\n");
}

void space_c(void) {
	add_possible_case("", 0, 0, 0, "", "", 0);
	if (arbeit == 1) prt(stdout, "la_line_in_space(lin);");
	else if (arbeit == 3) prt(stdout, "la_line_out_space(lout);");
}

void push_c(char *arg) {
	add_possible_case("", 0, 0, 0, "", "", 0);
	if (arbeit == 1) prt(stdout, "la_line_in_push(lin, %s);", arg);
	else if (arbeit == 3) prt(stdout, "la_line_out_push(lout, %s);", arg);
}

void pop_c(char *arg) {
	add_possible_case("", 0, 0, 0, "", "", 0);
	if (arbeit == 1) prt(stdout, "la_line_in_pop(lin, %s);", arg);
	else if (arbeit == 3) prt(stdout, "la_line_out_pop(lout, %s);", arg);
}

void fake_c(char *value, int type) {
	char *name[] = { "int", "string" };
	char *quote[] = { "", "\"" };
	add_possible_case("", 0, 0, 0, "", "", 0);
	switch (arbeit) {
		case 1:
			prt(stdout, "la_line_in_fake%s(lin, %s%s%s);", name[type], 
					quote[type], value, quote[type]);
			break;
		case 3:
			prt(stdout, "la_line_out_blackhole(lout);");
			break;
	}
}

void skip_c(void) {
	add_possible_case("", 0, 1, 0, "", "", 0);
	if (arbeit == 1) {
		prt(stdout, "la_line_in_skip(lin);");
		prt(stdout, "LA_DBG2(\"%%s:skipping\", lin->line_no);");
	} else if (arbeit == 3) prt(stdout, "la_line_out_skip(lout);");
}

int scanford(char *string) {
	char *ptr = string;
	if (*ptr == '\0') return 0;
	while (*(ptr + 1) != '\0') {
		if (!strncmp(ptr, "%d", 2)) return 1;
		ptr++;
	}
	return 0;
}

void compound_c(int no, ...) {
	va_list ap;
	int i;
	int tmp;
	int extra = 0;
	char buffer[256];
	char *ptr = buffer;
	int buffer_size = 255;
	char *string;
	add_possible_case("", 0, 0, 0, "", "", 0);
	if (arbeit == 1) {
		for (i = 0; i < CURST.indentlevel; i++) fprintc(stdout, "\t");
		fprintc(stdout, "la_line_in_break_compound(lin, %d", no);
		va_start(ap, no);
		for (i = 0; i < no; i++) {
			string = va_arg(ap, char*);
			extra += scanford(string);
		 	tmp = snprintf(ptr, buffer_size, ", \"%s\"", string);
			ptr += tmp;
			buffer_size -= tmp;
			va_arg(ap, char*);
		}
		if (extra) fprintc(stdout, ", %s%s);\n", va_arg(ap, char*), buffer);
		else fprintc(stdout, ", 0%s);\n", buffer);
		va_end(ap);
	}
	if (arbeit == 3) {
		for (i = 0; i < CURST.indentlevel; i++) fprintc(stdout, "\t");
		fprintc(stdout, "la_line_out_make_compound(lout, %d", no);
		va_start(ap, no);
		for (i = 0; i < no; i++) {
			va_arg(ap, char*);
			string = va_arg(ap, char*);
			extra += scanford(string);
			tmp = snprintf(ptr, buffer_size, ", \"%s\"", string);
			ptr += tmp;
			buffer_size -= tmp;
		}
		if (extra) fprintc(stdout, ", %s%s);\n", va_arg(ap, char*), buffer);
		else fprintc(stdout, ", 0%s);\n", buffer);
		va_end(ap);
	}
}

void line_c(char *name) {
	add_possible_case(name, 1, 1, 5, "lines", "", 'f');
	switch (arbeit) {
		case 1:
		case 3:
			SPREFIX;
			if (arbeit == 1) prt(stdout, "la_taf_lines_%s(&%s, lin" NWPR ", 1);", actions[arbeit], getfn(name));
			else prt(stdout, "la_taf_lines_%s(&%s, lout" NWPR ");", actions[arbeit], getfn(name));
			break;
		case 7:
			prt(stdout, "%s = la_taf_textview_widget(&%s, box);", getwn(name), getfn(name));
			break;
	}
}

void postlines_c(char *name, char *end) {
	char *post = "post";
	add_possible_case(name, 1, 1, 5, "lines", "", 'f');
	if (!strlen(end)) post = "";
	switch (arbeit) {
		case 1:
		case 3:
			SPREFIX;
			if (arbeit == 1) prt(stdout, "la_taf_postlines_%s(&%s, lin" NWPR ", %s);", actions[arbeit], getfn(name), end);
			else prt(stdout, "la_taf_postlines_%s(&%s, lout" NWPR ", %s);", actions[arbeit], getfn(name), end);
			break;
	}
}

void endaswitch_c() {
	assert(CURST.in_union == 1);
	CURST.in_union = 0;
	close_case();
	if (arbeit < 10) bprt(stdout, "}");
	else if (arbeit == 20) {
		if (CURU.useless == CURU.free_case + 1) {
			CURU.useless = 1;
		} else CURU.useless = 0;
		if (CURU.legal_useless == CURU.free_legal_case + 1) {
			CURU.legal_useless = 1;
		} else CURU.legal_useless = 0;
		CURU.last_legal_case = CURU.free_legal_case;
		CURU.last_case = CURU.free_case;
	}
#ifdef NDEBUG
	if (arbeit == 1 && CURU.useless) disable_output--;
#endif
	if (arbeit == 3 && CURU.useless) disable_output--;
	if (arbeit == 4 || arbeit == 5) disable_output--;
}

void oldloop_c(char *name, char *sname, char *mod, char *count, int slot, int fixed, char fs) {
	struct struct_info *sinfo;
	char c[128];
	add_possible_case(name, 1, 1, 4, sname, mod, fs);
	sinfo = get_struct_info(sname, mod, fs);
	if ((arbeit == 4 || arbeit == 5) && !sinfo->b4ldep) return;
	snprintf(c, 127, "%s.curr", getfn(name));
	switch (arbeit) {
		case 1:
			prt(stdout, "%s.ptr = la_slot_alloc(%s.ptr, %s.slots, %d, %s, sizeof(la_ta%c_%s_t), %d);", getfn(name), 
					getfn(name), getfn(name), slot, count, fs, sname, fixed);
		case 3:
		case 4:
		case 5:
			fprt(stdout, "for (%s.curr = la_slot_index(%s.slots, %d); %s.curr < la_slot_index(%s.slots, %d); %s.curr++) {", 
					getfn(name), getfn(name), slot, getfn(name), getfn(name), slot + 1, getfn(name));
#ifndef NDEBUG
			prt(stdout, "snprintf(new_prefix, 128, \"%%s%s%s[%d,%%d].\", prefix, %s.curr - la_slot_index(%s.slots, %d));", getuprfx(), name, slot, getfn(name), getfn(name), slot); 
#endif
			funccall(stdout, name, sname, c, mod, arbeit, fs);
			bprt(stdout, "}");
			break;
	}
}

void oldautoloop_c(char *name, char *sname, char *mod, int slot, char fs) {
#ifndef NDEBUG
	char tmp[256];
#endif
	if (arbeit == 3) {
		prt(stdout, "la_line_out_put_int(lout, %s.slots[%d].size);", getfn(name), slot);
	}
#ifndef NDEBUG
	snprintf(tmp, 255, "la_taf_noneg_parse(lin, prefix, \"no_%s_in_slot_%d\")", name, slot);
	oldloop_c(name, sname, mod, tmp, slot, 0, 'f');
#else
	oldloop_c(name, sname, mod, "la_taf_noneg_parse(lin)", slot, 0, fs);
#endif
}

void oldstuff_c(char *name, char *sname, char *mod, char fs) {
	struct struct_info *sinfo;
	if (strcmp(sname, "data")) {
		add_possible_case(name, 1, 1, 3, sname, mod, fs);
		sinfo = get_struct_info(sname, mod, fs);
		if ((arbeit == 4 || arbeit == 5) && !sinfo->b4ldep) return;
		if (arbeit < 10) {
			MPREFIX("\"%%s%s%s.\", prefix");
			funccall(stdout, name, sname, "", mod, arbeit, fs);
		}
	}
}

void buffer_for_later_c(char *name, char *size, char *filename) {
	add_possible_case(name, 1, 1, 6, size, "", 0);
	switch (arbeit) {
		case 4:
			fprt(stdout, "if (%s > 0) {", size);
#ifndef NDEBUG
			prt(stdout, "LA_DEBUG(\"parsing %%d bytes of extra data from %%s\", %s, %s);", size, filename);
#endif
			prt(stdout, "%s = emalloc(%s);", getfn(name), size);
			prt(stdout, "la_char_io_read(%s, %s, cio);", getfn(name), size);
			prt(stdout, "if (la_char_io_getc(cio) != 0) LA_WARNING(\"ASCII NUL byte expected, but other byte found\");");
			bprt(stdout, "}");
			break;
		case 5:
			fprt(stdout, "if (%s > 0) {", size);
#ifndef NDEBUG
			prt(stdout, "LA_DEBUG(\"writing %%d bytes of extra data\", %s);", size);
#endif
			prt(stdout, "la_char_io_write(%s, %s, cio);", getfn(name), size);
			prt(stdout, "la_char_io_putc(0, cio);");
			bprt(stdout, "}");
			break;
		case 20:
			break;
	}
}

void legenda_c(char *name) {
	add_possible_case(name, 0, 0, 0, "", "", 0);
#ifndef NDEBUG
	switch (arbeit) {
		case 1:
			prt(stdout, "LA_DBG2(\"%%s:%%s%s%s (legenda)\", lin->line_no, prefix);", getuprfx(), name);
			break;
	}
#endif
}

void intg_c(char *name, char *sname, ...) {
#ifndef NDEBUG
	va_list ap;
	char tmp[128];
#endif
	add_possible_case(name, 1, 1, 1, "", "", 0);
	switch (arbeit) {
		case 1:
			prt(stdout, "%s = la_line_in_get_int(lin);", getfn(name));
#ifndef NDEBUG
			va_start(ap, sname);
			vsnprintf(tmp, 128, sname, ap);
			va_end(ap);
			prt(stdout, "LA_DBG2(\"%%s:%%s%s%s=%%d (%s)\", lin->line_no, prefix, %s);", getuprfx(), name, tmp, getfn(name));
#endif
			break;
		case 3:
			prt(stdout, "la_line_out_put_int(lout, %s);", getfn(name));
			break;
		case 20:
			break;
	}
}

void string_c(char *name, char *sname) {
	add_possible_case(name, 1, 1, 2, "", "", 0);
	switch (arbeit) {
		case 3:
			prt(stdout, "la_line_out_put_string(lout, %s);", getfn(name));
			break;
		case 1:
			prt(stdout, "%s = la_line_in_get_string(lin);", getfn(name));
#ifndef NDEBUG
			prt(stdout, "LA_DBG2(\"%%s:%%s%s%s=\\\"%%s\\\" (%s)\", lin->line_no, prefix, %s);", getuprfx(), name, sname, getfn(name));
#endif
			break;
		case 7:
			prt(stdout, "%s = la_taf_string_widget(&%s, box, \"%s\");", getwn(name), getfn(name), name);
			break;
		case 20:
			break;
	}
}

void stringexp_c(char *s) {
	char name[256];
	in_expect = 1;
	snprintf(name, 255, "exp%d", CURST.expcount++);
	string_c(name, "stringexp");
	if (arbeit == 1) {
		prt(stdout, "if (strcmp(%s, %s)) LA_WARNING(\"%s expected on line %%s of %%s, but found \\\"%%s\\\" instead\", lin->line_no, lin->cin->name, %s);", s, getfn(name), s, getfn(name));
	}
	in_expect = 0;
}

void exps_c(int args, ...) {
	va_list ap;
	char buf1[256];
	char *ptr1 = buf1;
	int size1 = 255, i, tmp = 0;
	char buf2[1024];
	char *ptr2 = buf2;
	int size2 = 1024;
	char *add2 = " && ";
	char *add1 = " or ";
	char *string;
	char *whine = "WARNING";
	char name[256];
	in_expect = 1;
	snprintf(name, 255, "exp%d", CURST.expcount++);
	args++;
	assert(args >= 1);
	string_c(name, "exps");
	if (arbeit == 1) {
		va_start(ap, args);
		for (i = 0; i < args; i++) {
			if (i == args - 1) {
				add1 = "";
				add2 = "";
			}
			string = va_arg(ap, char*);
			if (string) tmp = snprintf(ptr1, size1, "\\\"%s\\\"%s", string, add1);
			else {
				whine = "VERBOSE";
				tmp = 0;
				ptr1 -= 4;
				*ptr1 = '\0';
			}
			size1 -= tmp;
			ptr1 += tmp;
			if (string) tmp = snprintf(ptr2, size2, "strcmp(%s, \"%s\")%s", getfn(name), string, add2);
			else tmp = snprintf(ptr2, size2, "1");
			size2 -= tmp;
			ptr2 += tmp;
		}
		va_end(ap);
		prt(stdout, "if (%s) LA_%s(\"%s expected on line %%s of %%s, but found \\\"%%s\\\" instead\", lin->line_no, lin->cin->name, %s);", buf2, whine, buf1, getfn(name));
	}
	in_expect = 0;
}

void expi_c(int args, ...) {
	va_list ap;
	char buf1[256];
	char *ptr1 = buf1;
	int size1 = 255, i, tmp = 0;
	char buf2[1024];
	char *ptr2 = buf2;
	int size2 = 1024;
	char *add2 = " && ";
	char *add1 = " or ";
	char *string;
	char *whine = "WARNING";
	char name[256];
	in_expect = 1;
	args++;
	assert(args >= 1);
	snprintf(name, 255, "exp%d", CURST.expcount++);
	intg_c(name, "expi");
	if (arbeit == 1) {
		va_start(ap, args);
		for (i = 0; i < args; i++) {
			if (i == args - 1) {
				add1 = "";
				add2 = "";
			}
			string = va_arg(ap, char*);
			if (string) 
				tmp = snprintf(ptr1, size1, "%s%s", 
						string, add1);
			else {
				whine = "VERBOSE";
				tmp = 0;
				ptr1 -= 4;
				*ptr1 = '\0';
			}
			size1 -= tmp;
			ptr1 += tmp;
			if (string) tmp = snprintf(ptr2, size2, "%s != %s%s", getfn(name), string, add2);
			else tmp = snprintf(ptr2, size2, "1");
			size2 -= tmp;
			ptr2 += tmp;
		}
		va_end(ap);
		prt(stdout, "if (%s) LA_%s(\"%s expected on line %%s of %%s, found %%d\", lin->line_no, lin->cin->name, %s);", buf2, whine, buf1, getfn(name));
	}
	in_expect = 0;
}

void integt_c(char *name, char *min) {
	if (arbeit == 1) {
		add_possible_case(name, 1, 1, 1, "", "", 0);
#ifndef NDEBUG
		prt(stdout, "%s = la_taf_integt_parse(lin, %s, prefix, \"%s%s\");", getfn(name), min, getuprfx(), name);
#else
		prt(stdout, "%s = la_taf_integt_parse(lin, %s);", getfn(name), min);
#endif
	} else {
		intg_c(name, "uint");
	}
}

void noneg_c(char *name) {
	if (arbeit == 1) {
		add_possible_case(name, 1, 1, 1, "", "", 0);
#ifndef NDEBUG
		prt(stdout, "%s = la_taf_noneg_parse(lin, prefix, \"%s%s\");", getfn(name), getuprfx(), name);
#else
		prt(stdout, "%s = la_taf_noneg_parse(lin);", getfn(name));
#endif
	} else {
		intg_c(name, "uint");
	}
}

void boolean_c(char *name) {
	if (arbeit == 1) {
		add_possible_case(name, 1, 1, 7, "", "", 0);
#ifndef NDEBUG
		prt(stdout, "%s = la_taf_boolean_parse(lin, prefix, \"%s%s\");", getfn(name), getuprfx(), name);
#else
		prt(stdout, "%s = la_taf_boolean_parse(lin);", getfn(name));
#endif
	} else if (arbeit == 7) {
		prt(stdout, "%s = la_taf_boolean_widget(&%s, box, \"%s\");", getwn(name), getfn(name), name);
	} else {
		intg_c(name, "boolean");
	}
}

void range_c(char *name, char *min, char *max, int internal) {
	if (arbeit == 1) {
		add_possible_case(name, 1, 1, 1, "", "", 0);
#ifndef NDEBUG
		prt(stdout, "%s = la_taf_range_parse(lin, %s, %s, prefix, \"%s%s\");", getfn(name), min, max, getuprfx(), name);
#else
		prt(stdout, "%s = la_taf_range_parse(lin, %s, %s);", getfn(name), min, max);
#endif
	} else if (arbeit == 7 && !internal) {
		prt(stdout, "%s = la_taf_range_widget(&%s, box, %s, \"%s\");", getwn(name), getfn(name), max, name);
	} else {
		intg_c(name, "range from %s to %s", min, max);
	}
}

void aswitch_c() {
	char tmp[256];
	struct struct_info *sinfo;
	ADVU;
	assert(CURST.in_union == 0);
	CURU.free_case = -1;
	CURU.free_legal_case = -1;
	CURU.type = 0;
	sinfo = get_struct_info(CURST.name, NULL, CURST.fs);
	sinfo->utype = 0;
	if (arbeit == 4 || arbeit == 5) disable_output++;
	CURU.case_open = 0;
	if (arbeit != 20) {
		if (CURU.condition == NULL) 
			snprintf(tmp, 255, "%d", CURU.last_legal_case);
		else snprintf(tmp, 255, "%s?%d:%d", CURU.condition, 
				CURU.last_case, CURU.last_legal_case);
	} else {
		CURU.free = 0;
		CURU.useless = 0;
		CURU.legal_useless = 0;
	}
	range_c("type", "0", tmp, 1);
	CURST.in_union = 1;
	if (arbeit == 3 && CURU.useless) disable_output++;
#ifdef NDEBUG
	if (arbeit == 1 && CURU.useless) disable_output++;
#endif
	if (arbeit < 10) {
		if (arbeit == 7) {
			prt(stdout, "w->type = la_taf_comboboxonly_widget(&d->type, %s, la_taf_%s%s_types);",
					tmp, CURST.name, CURST.mod);
			prt(stdout, "w->specbox = la_taf_leftwidget_widget(w->type, box);");
		}
		fprt(stdout, "switch (d->type) {");
	} else if (arbeit == 20) {
		CURU.condition = NULL;
		CURU.last_case = 0;
		CURU.last_legal_case = 0;
	}	
}

#define LAST_POSSIBLE ((CURU.condition && !strcmp(CURU.condition, "0"))?CURU.last_legal_case:CURU.last_case)
void kswitch_c() {
	struct struct_info *sinfo;
	char tmp[256];
	int i, c = 0;
	ADVU;
	assert(CURST.in_union == 0);
	CURU.free_case = -1;
	CURU.free_legal_case = -1;
	CURU.type = 1;
	sinfo = get_struct_info(CURST.name, NULL, CURST.fs);
	sinfo->utype = 1;
	if (arbeit == 4 || arbeit == 5) disable_output++;
	if (arbeit != 20) {
		if (CURU.condition == NULL || !strcmp(CURU.condition, "0")) 
			snprintf(tmp, 255, "%d", CURU.last_legal_case);
		else snprintf(tmp, 255, "%s?%d:%d", CURU.condition, 
				CURU.last_case, CURU.last_legal_case);
	} else {
		CURU.useless = 0;
		CURU.legal_useless = 0;
	}
	for (i = 0; i < LAST_POSSIBLE; i++) c += CURU.case_infos[i].nfo.type != 0;
	if (c == 0 && i > 0 && CURU.case_infos[LAST_POSSIBLE].nfo.type != 0) {
		switch(LAST_POSSIBLE) {
			case 1:
				compound_c(2, "i!=0", "i", "i!=0?i-1:0", "i");
				break;
			case 2:
				compound_c(2, "i<0||i>1?2:i", "i", "i<0||i>1?i-2:0", "i");
				break;
			case 3:
				compound_c(2, "i<0||i>2?3:i", "i", "i<0||i>2?i-3:0", "i");
				break;
			default:
				fprintf(stderr, "unhandled case\n");
				exit(1);
				break;
		}
	}
	if (LAST_POSSIBLE == 0) fake_c("0", 0);
	range_c("pwtype", "0", tmp, 1);
	if (arbeit != 1 && arbeit != 3) noneg_c("realtype");
	CURST.in_union = 1;
	if (arbeit == 3 && CURU.useless) disable_output++;
#ifdef NDEBUG
	if (arbeit == 1 && CURU.useless) disable_output++;
#endif
	if (arbeit < 10) {
		if (arbeit == 7) {
			prt(stdout, "w->pwtype = la_taf_comboboxonly_widget(&d->pwtype, %s, la_taf_%s%s_types);",
					tmp, CURST.name, CURST.mod);
			prt(stdout, "w->specbox = la_taf_leftwidget_widget(w->pwtype, box);");
		}
		//if (arbeit == 7) prt(stdout, "w->pwtype = la_taf_comboboxonly_widget(&d->pwtype, %s, la_taf_%s%s_types);",
		//			tmp, CURST.name, CURST.mod);
		fprt(stdout, "switch (d->pwtype) {");
	} else if (arbeit == 20) {
		CURU.condition = NULL;
		CURU.last_case = 0;
		CURU.last_legal_case = 0;
	}	
}

void common_c(char *sname, char *name, char *mod, char fs) {
	struct struct_info *sinfo;
	int i;
	ADVST;
	CURST.expcount = 0;
	CURST.free_union = -1;
	CURST.free_name = -1;
	CURST.free_decl = -1;
	if (arbeit < 10) {
		if (!CURST.b4ldep && (arbeit == 4 || arbeit == 5)) disable_output++;
		if (struct_depth > 0) ENTER;
		sinfo = get_struct_info(name, NULL, fs); /* info is stored in first occurance of struct */
		if (arbeit == 7 && sinfo->has_union) {
			fprt(stdout, "char *la_taf_%s%s_types[%d] = {", 
					name, mod, CURST.union_infos[0].last_case + 1);
			for (i = 0; i < CURST.union_infos[0].last_case + 1; i++) {
				prt(stdout, "\"%s\"%s", CURST.union_infos[0].case_infos[i].name,
						i < CURST.union_infos[0].last_case?",":"");
			}
			bprt(stdout, "};");
			prt(stdout, "");
		}
		funcstart(stdout, name, mod, arbeit, fs);
	}
	if (arbeit == 20) {
		CURST.name = name;
		CURST.has_union = 0;
		CURST.free = 0;
		CURST.cfree = 0;
		CURST.mod = mod;
		CURST.b4ldep = 0;
		CURST.in_union = 0;
		CURST.fs = fs;
		CURST.no_case_infos = 0;
		CURST.indentlevel = 0;
		CURST.no_deps = 0;
		CURST.resolved = 0;
		CURST.resolving = 0;
		CURST.gresolved = 0;
		CURST.gresolving = 0;
		CURST.no_gdeps = 0;
		//struct_verbose(&CURST);
	}
}

void stuff_c(char *sname, char *name, char *mod, char fs) {
	oldstuff_c(sname, name, mod, fs);
	common_c(sname, name, mod, fs);
}

void autoloop_c(char *sname, char *name, char *mod, int slot, char fs) {
	oldautoloop_c(sname, name, mod, slot, fs);
	common_c(sname, name, mod, fs);
}

void loop_c(char *sname, char *name, char *mod, char* count, int slot, int fixed, char fs) {
	oldloop_c(sname, name, mod, count, slot, fixed, fs);
	common_c(sname, name, mod, fs);
}

void endstuff_c() {
	if (arbeit < 10) {
		bprt(stdout, "}");
		if (struct_depth > 0) EXIT;
		if (!CURST.b4ldep && (arbeit == 4 || arbeit == 5)) disable_output--;
	}
	if (arbeit == 20) CURST.last_name = CURST.free_name;
	struct_depth--;
}

void iff_c(char *condition, char *els) {
	add_possible_case("", 0, 0, 0, "", "", 0);
	switch (arbeit) {
		case 1:
		case 3:
		case 4:
		case 5:
		case 6:
		case 7:
			if (CURST.in_union == 0 || (CURU.free_case != -1 && CURU.named == 0)) {
				fprt(stdout, "%sif (%s) {", els, condition);
			}
			break;
	}
}

void condition_c(char *condition) {
	switch (arbeit) {
		case 20:
			if (CURST.in_union == 1 && (CURU.free_case == -1 || CURU.named == 1)) {
				assert(CURU.condition == NULL || !strcmp(CURU.condition, "0"));
				close_case();
				CURU.condition = condition;
			}
			break;
	}
}

void endiff_c(void) {
	switch (arbeit) {
		case 1:
		case 3:
		case 4:
		case 5:
		case 6:
		case 7:
			if (CURST.in_union == 0 || (CURU.free_case != -1 && CURU.named == 0)) {
				bprt(stdout, "}");
			}
			break;
	}
}

void specfunc(int i) {
	arbeit = i;
	free_struct = 0;
	fstruct(data, data);
#include "taf.spec.i"
	endfstruct();
	sstruct(data, data);
#include "tas.spec.i"
	endsstruct();
}

FILE *gtkh, *genrc, *freec, *freeh, *copyh, *copyc;

void print_decl(struct name_info *nfo, int *tbf) {
	struct struct_info *sinfo;
	switch (nfo->type) {
		case 1:
			prt(genrc, "int %s;", nfo->name);
			break;
		case 2:
		case 6:
			prt(genrc, "char *%s;", nfo->name);
			*tbf = 1;
			break;
		case 4:
			prt(genrc, "la_taf_array_t %s;", nfo->name);
			*tbf = 1;
			break;
		case 5:
		case 3:
			sinfo = get_struct_info(nfo->sname, NULL, nfo->fs);
			if (!sinfo) {
				*tbf = 1;
			} else {
				*tbf |= sinfo->free;
			}
			prt(genrc, "la_ta%c_%s_t %s;", nfo->fs, nfo->sname, nfo->name);
			break;
	}
}

void print_widgetdecl(struct name_info *nfo) {
	switch (nfo->type) {
		case 5:
		case 2:
		case 1:
			prt(gtkh, "GtkWidget *%s;", nfo->name);
			break;
		case 3:
			prt(gtkh, "la_ta%c_%s_widget_t %s;", nfo->fs, nfo->sname, nfo->name);
			break;
	}
}

void print_free(struct name_info *nfo, int k) {
	struct struct_info *sinfo;
	char c[128];
	switch (nfo->type) {
		case 6:
		case 2:
			if (k >= 0) fprt(freec, "case %d:", k);
			prt(freec, "free(%s);", getfn(nfo->name));
			if (k >= 0) dprt(freec, "break;");
			break;
		case 3:
			sinfo = get_struct_info(nfo->sname, NULL, nfo->fs);
			if (!sinfo || sinfo->free) {
				if (k >= 0) fprt(freec, "case %d:", k);
#ifndef NDEBUG
				prt(freec, "snprintf(new_prefix, 128, \"%%s%s%s\", prefix);", getuprfx(), nfo->name);
#endif
				funccall(freec, nfo->name, nfo->sname, "", "", 2, nfo->fs);
				if (k >= 0) dprt(freec, "break;");
			}
			break;
		case 5:
			if (k >= 0) fprt(freec, "case %d:", k);
#ifndef NDEBUG
			prt(freec, "snprintf(new_prefix, 128, \"%%s%s%s\", prefix);", getuprfx(), nfo->name);
			prt(freec, "la_taf_lines_free(&%s, new_prefix);", getfn(nfo->name));
#else
			prt(freec, "la_taf_lines_free(&%s);", getfn(nfo->name));
#endif
			if (k >= 0) dprt(freec, "break;");
			break;
		case 4:
			if (k >= 0) fprt(freec, "case %d:", k);
			sinfo = get_struct_info(nfo->sname, NULL, nfo->fs);
			if (!sinfo || sinfo->free) {
				snprintf(c, 127, "%s.curr", getfn(nfo->name));
				fprt(freec, "if (%s.ptr) for (%s.curr = 0; %s.curr < la_slot_total(%s.slots); %s.curr++) {",
						getfn(nfo->name), getfn(nfo->name), getfn(nfo->name), getfn(nfo->name),
						getfn(nfo->name));
#ifndef NDEBUG
				prt(freec, "snprintf(new_prefix, 128, \"%%s%s%s[%%d].\", prefix, %s.curr);", getuprfx(), nfo->name, getfn(nfo->name)); 
#endif
				funccall(freec, nfo->name, nfo->sname, c, "", 2, nfo->fs);
				dprt(freec, "}");
			}
			prt(freec, "free(%s.ptr);", getfn(nfo->name));
			if (k >= 0) dprt(freec, "break;");
			break;
	}
}

#if 0
void print_widget(struct name_info *nfo) {
	switch (nfo->type) {
		case 2:
			prt(gtkc, "w->%s = gtk_entry_new();", nfo->name);
			prt(gtkc, "gtk_entry_set_text(GTK_ENTRY(w->%s), %s);", nfo->name, getfn(nfo->name));
			prt(gtkc, "gtk_box_pack_start_defaults(box, w->%s);", nfo->name);
			break;
		case 7:
			prt(gtkc, "w->%s = gtk_check_button_new_with_label(\"%s\");", nfo->name, nfo->name);
			prt(gtkc, "gtk_box_pack_start_defaults(box, w->%s);", nfo->name);
			break;
	}
}
#endif

void print_copy(struct name_info *nfo, int k) {
	char c[128];
	if (k >= 0) fprt(copyc, "case %d:", k);
	switch (nfo->type) {
		case 1:
			prt(copyc, "%s = %s;", getfn(nfo->name), getfns(nfo->name));
			break;
		case 2:
			prt(copyc, "%s = %s?estrdup(%s):NULL;", getfn(nfo->name), getfns(nfo->name), getfns(nfo->name));
			break;
		case 6:
			fprt(copyc, "if (%s > 0) {", nfo->sname);
			prt(copyc, "%s = emalloc(%s);", getfn(nfo->name), nfo->sname);
			prt(copyc, "memcpy(%s, %s, %s);", getfn(nfo->name), getfns(nfo->name), nfo->sname);
			bprt(copyc, "}");
			break;
		case 3:
#ifndef NDEBUG
			prt(copyc, "snprintf(new_prefix, 128, \"%%s%s%s\", prefix);", getuprfx(), nfo->name);
#endif
			funccall(copyc, nfo->name, nfo->sname, "", "", 6, nfo->fs);
			break;
		case 5:
#ifndef NDEBUG
			prt(copyc, "snprintf(new_prefix, 128, \"%%s%s%s\", prefix);", getuprfx(), nfo->name);
			prt(copyc, "la_taf_lines_copy(&%s, &%s, new_prefix);", getfn(nfo->name), getfns(nfo->name));
#else
			prt(copyc, "la_taf_lines_copy(&%s, &%s);", getfn(nfo->name), getfns(nfo->name));
#endif
			break;
		case 4:
			snprintf(c, 127, "%s.curr", getfn(nfo->name));
			prt(copyc, "memcpy(&%s, &%s, sizeof(la_slot_info_t)*LA_SLOT_MAX);", getfn(nfo->name), getfns(nfo->name));
			prt(copyc, "%s.ptr = ecalloc(la_slot_total(%s.slots), sizeof(la_ta%c_%s_t));", getfn(nfo->name),
					getfn(nfo->name), nfo->fs, nfo->sname);
			fprt(copyc, "for (%s.curr = 0; %s.curr < la_slot_total(%s.slots); %s.curr++) {",
					getfn(nfo->name), getfn(nfo->name), getfn(nfo->name), getfn(nfo->name),
					getfn(nfo->name));
#ifndef NDEBUG
			prt(copyc, "snprintf(new_prefix, 128, \"%%s%s%s[%%d].\", prefix, %s.curr);", getuprfx(), nfo->name, getfn(nfo->name)); 
#endif
			funccall(copyc, nfo->name, nfo->sname, c, "", 6, nfo->fs);
			bprt(copyc, "}");
			break;
	}
	if (k >= 0) dprt(copyc, "break;");
}

void print_struct(char *name, char fs) {
	struct struct_info *sinfo;
	int i, j;
	sinfo = get_struct_info(name, NULL, fs); /* info is stored in first occurance of struct */
	assert(sinfo);
	if (sinfo->has_union) {
		fprt(genrc, "typedef enum {");
		for (i = 0; i < sinfo->no_case_infos; i++) {
			prt(genrc, "la_ta%c_%s_%s_e%s", sinfo->fs, name, sinfo->case_infos[i].name, 
					(i + 1 < sinfo->no_case_infos)?",":"");
		}
		bprt(genrc, "} la_ta%c_%s_types_t;", sinfo->fs, name);
		prt(genrc, "");
	}
	fprt(genrc, "typedef struct {");
	for (i = 0; i <= sinfo->free_decl; i++) {
		print_decl(&sinfo->decl_infos[i], &sinfo->free);
	}
	if (sinfo->has_union) {
		fprt(genrc, "union {");
		for (j = 0; j < sinfo->no_case_infos; j++) {
			print_decl(&sinfo->case_infos[j].nfo, &sinfo->cfree);
		}
		sinfo->free |= sinfo->cfree;
		bprt(genrc, "} spec;");
	}
	bprt(genrc, "} la_ta%c_%s_t;", sinfo->fs, name);
	fprt(gtkh, "typedef struct {");
	for (i = 0; i <= sinfo->free_decl; i++) {
		print_widgetdecl(&sinfo->decl_infos[i]);
	}
	if (sinfo->has_union) {
		prt(gtkh, "GtkBox *specbox;");
		fprt(gtkh, "union {");
		for (j = 0; j < sinfo->no_case_infos; j++) {
			print_widgetdecl(&sinfo->case_infos[j].nfo);
		}
		bprt(gtkh, "} spec;");
	}
	bprt(gtkh, "} la_ta%c_%s_widget_t;", sinfo->fs, name);
	prt(gtkh, "");
	funcdecl(gtkh, sinfo->name, "", 7, sinfo->fs);
	prt(gtkh, "");
	prt(copyc, "");
	funcstart(copyc, name, "", 6, sinfo->fs);
	for (i = 0; i <= sinfo->free_decl; i++) {
		print_copy(&sinfo->decl_infos[i], -1);
	}
	if (sinfo->has_union) {
		fprt(copyc, "switch (d->%stype) {", sinfo->utype?"real":"");
		for (j = 0; j < sinfo->no_case_infos; j++) {
			print_copy(&sinfo->case_infos[j].nfo, j);
		}
		bprt(copyc, "}");
	}
	bprt(copyc, "}");
	prt(copyh, "");
	funcdecl(copyh, sinfo->name, "", 6, sinfo->fs);
	if (sinfo->free) {
		fprintc(freec, "\n");
		funcstart(freec, name, "", 2, sinfo->fs);
		for (i = 0; i <= sinfo->free_decl; i++) {
			print_free(&sinfo->decl_infos[i], -1);
		}
		if (sinfo->has_union) {
			if (sinfo->cfree) {
				fprt(freec, "switch (d->%stype) {", sinfo->utype?"real":"");
				for (j = 0; j < sinfo->no_case_infos; j++) {
					print_free(&sinfo->case_infos[j].nfo, j);
				}
				bprt(freec, "}");
			}
		}
		bprt(freec, "}");
		prt(freeh, "");
		funcdecl(freeh, sinfo->name, "", 2, sinfo->fs);
	}
	prt(genrc, "");
}

void calcgdep(struct struct_info *sinfo) {
	int i;
	for (i = 0; i < sinfo->no_gdeps; i++) 
		sinfo->b4ldep |= sinfo->gdeps[i]->b4ldep;
	for (i = 0; i <= sinfo->free_name; i++)
		if (sinfo->name_infos[i].type == 6)
			sinfo->b4ldep = 1;
}

void gdeptree(struct struct_info *sinfo) {
	int i;
	if (sinfo->gresolved == 1) return;
	if (sinfo->gresolving) assert(0);
	sinfo->gresolving = 1;
	for (i = 0; i < sinfo->no_gdeps; i++) {
		gdeptree(sinfo->gdeps[i]);
	}
	sinfo->gresolved = 1;
	calcgdep(sinfo);
}

void deptree(struct struct_info *sinfo) {
	int i;
	if (sinfo->resolved == 1) return;
	if (sinfo->resolving) assert(0);
	sinfo->resolving = 1;
	for (i = 0; i < sinfo->no_deps; i++) {
		deptree(sinfo->deps[i]);
	}
	sinfo->resolved = 1;
	print_struct(sinfo->name, sinfo->fs);
}

void openh(FILE *fp, char *name) {
	prt(fp, "#ifndef INCLUDE_LIBADRIFT_TAF_%s_H", name);
	prt(fp, "#define INCLUDE_LIBADRIFT_TAF_%s_H 1", name);
	prt(fp, "");
}

void closeh(FILE *fp, char *name) {
	prt(fp, "");
	prt(fp, "#endif /* INCLUDE_LIBADRIFT_TAF_%s_H */", name);
}

int main(int argc, char **argv) {
	struct struct_info *sinfo;
	int i, j, k;

	/* pre pass */
	specfunc(20);
	no_structs = free_struct;

	/* ordinary dependencies */
	for (i = 0; i < no_structs; i++) {
		sinfo = get_struct_info(struct_infos[i].name, NULL, struct_infos[i].fs);
		if (sinfo != struct_infos + i) continue; /* already processed */
		/* store dependencies */
		for (j = 0; j <= sinfo->free_decl; j++)
			if (sinfo->decl_infos[j].type == 3 || sinfo->decl_infos[j].type == 4)
				sinfo->deps[sinfo->no_deps++] = get_struct_info(sinfo->decl_infos[j].sname, NULL,
						sinfo->decl_infos[j].fs);
		if (sinfo->has_union) {
		//for (j = 0; j <= sinfo->free_union; j++) {
			for (k = 0; k <= sinfo->no_case_infos; k++) {
				if (sinfo->case_infos[k].nfo.type == 3 || sinfo->case_infos[k].nfo.type == 4)
					sinfo->deps[sinfo->no_deps++] = get_struct_info(sinfo->case_infos[k].nfo.sname, NULL,
							sinfo->case_infos[k].nfo.fs);
			}
		}
	}
	for (i = 0; i < no_structs; i++) {
		sinfo = &struct_infos[i];
		for (j = 0; j <= sinfo->free_name; j++) {
			if (sinfo->name_infos[j].type == 3 || sinfo->name_infos[j].type == 4)
				sinfo->gdeps[sinfo->no_gdeps++] = get_struct_info(sinfo->name_infos[j].sname,
						sinfo->name_infos[j].mod, sinfo->name_infos[j].fs);
		}
	}
	gdeptree(&struct_infos[0]);

	genrc = fopen("generic.h", "w");
	print_header(genrc, "generic.h - generic interface for taf functions");
	openh(genrc, "GENERIC");

	gtkh = fopen("gtkwidgets.h", "w");
	print_header(gtkh, "gtkwidgets.h - storage for widgets");
	openh(gtkh, "GTKWIDGETS");

	freeh = fopen("free.h", "w");
	print_header(freeh, "free.h - interface for freeing taf data");
	openh(freeh, "FREE");

	copyh = fopen("copy.h", "w");
	print_header(copyh, "copy.h - interface for copying taf data");
	openh(copyh, "COPY");

	copyc = fopen("copy.c", "w");
	print_header(copyc, "copy.c - functions for copying taf data");
	freec = fopen("free.c", "w");
	print_header(freec, "free.c - functions for freeing taf data");

	prt(genrc, "#include \"../char_io.h\"");
	prt(genrc, "#include \"../line_in.h\"");
	prt(genrc, "#include \"../line_out.h\"");
	prt(freec, "#include <string.h>");
	prt(freec, "#include \"../slot.h\"");
	prt(freec, "#include \"free.h\"");
	prt(freeh, "#include \"common.h\"");
	prt(freeh, "#include \"generic.h\"");
	prt(copyc, "#include <string.h>");
	prt(copyc, "#include \"emem.h\"");
	prt(copyc, "#include \"../slot.h\"");
	prt(copyc, "#include \"copy.h\"");
	prt(copyh, "#include \"common.h\"");
	prt(copyh, "#include \"generic.h\"");
	prt(genrc, "#include \"common.h\"");
	prt(gtkh, "#include \"generic.h\"");
	prt(gtkh, "#include <gtk/gtk.h>");
	prt(gtkh, "#include <string.h>");
	prt(gtkh, "");
	prt(genrc, "");
	prt(freec, "");
	prt(copyc, "");
	prt(freec, "#define LA_VERBOSE_TYPE Taf_Freer");
	prt(copyc, "#define LA_VERBOSE_TYPE Taf_Copier");
	prt(genrc, "#define NO_ROOMS la_taf_array_total(&tas->taf->data.rooms)");
	prt(genrc, "#define NO_OBJECTS la_taf_array_total(&tas->taf->data.objects)");
	prt(genrc, "#define NO_TASKS la_taf_array_total(&tas->taf->data.tasks)");
	prt(genrc, "#define NO_EVENTS la_taf_array_total(&tas->taf->data.events)");
	prt(genrc, "#define NO_CHARACTERS la_taf_array_total(&tas->taf->data.characters)");
	prt(genrc, "#define NO_VARIABLES la_taf_array_total(&tas->taf->data.variables)");
	prt(genrc, "#define OBJ(a) (((la_taf_object_t*)(tas->taf->data.objects.ptr)) + (a))");
	prt(genrc, "#define COBJ (((la_taf_object_t*)tas->taf->data.objects.ptr) + tas->taf->data.objects.curr)");
	prt(genrc, "#define FOBJ (((la_taf_object_t*)tas->taf->data.objects.ptr) + tas->data.objects.curr)");
	prt(genrc, "#define SOBJ (((la_tas_object_t*)tas->data.objects.ptr) + tas->data.objects.curr)");
	prt(genrc, "");
	prt(genrc, "typedef struct la_taf_s la_taf_t;");
	prt(genrc, "typedef struct la_tas_s la_tas_t;\n");
	sinfo = get_struct_info("data", NULL, 'f');
	deptree(sinfo);
	sinfo = get_struct_info("data", NULL, 's');
	deptree(sinfo);
	prt(genrc, "#include \"taf.h\"");
	prt(genrc, "#include \"tas.h\"");
	prt(genrc, "");
	funcdecl(genrc, "data", "", 1, 'f');
	prt(genrc, "");
	funcdecl(genrc, "data", "", 4, 'f');
	prt(genrc, "");
	funcdecl(genrc, "data", "", 3, 'f');
	prt(genrc, "");
	funcdecl(genrc, "data", "", 5, 'f');
	prt(genrc, "");
	funcdecl(genrc, "data", "", 1, 's');
	prt(genrc, "");
	funcdecl(genrc, "data", "", 3, 's');
	closeh(genrc, "GENERIC");
	closeh(gtkh, "GTKWIDGETS");
	closeh(freeh, "FREE");
	closeh(copyh, "COPY");
	fclose(genrc);
	fclose(gtkh);
	fclose(freeh);
	fclose(freec);
	fclose(copyh);
	fclose(copyc);
	printf("a\n");
	printf("\n.\ni\n");
	print_header(stdout, "generic.c - generic taf functions");
	printf("#include <assert.h>\n");
	printf("#include <errno.h>\n");
	printf("#include <stdlib.h>\n");
	printf("#include <string.h>\n");
	printf("#include \"../verbose.h\"\n");
	printf("#include \"../char_io.h\"\n");
	printf("#include \"../line_in.h\"\n");
	printf("#include \"../line_out.h\"\n");
	printf("#include \"../emem.h\"\n");
	printf("#include \"../slot.h\"\n");
	printf("#include \"common.h\"\n");
	printf("#include \"generic.h\"\n");
	printf("#include \"gtkwidgets.h\"\n");
	printf("#include \"gtkutils.h\"\n");

	printf("\n#define LA_VERBOSE_TYPE Taf_Parser\n");
	printf("\n.\nka\na\n\n");
	specfunc(1);
	printf("\n.\nka\na\n\n");
	specfunc(4);

	printf("\n#undef LA_VERBOSE_TYPE\n");
	printf("#define LA_VERBOSE_TYPE Taf_Writer\n");
	printf("\n.\nka\na\n\n");
	specfunc(3);
	printf("\n.\nka\na\n\n");
	specfunc(5);

	printf("\n#undef LA_VERBOSE_TYPE\n");
	printf("#define LA_VERBOSE_TYPE Taf_GTK\n");
	printf("\n.\nka\na\n\n");
	specfunc(7);

	printf(".\nw generic.c\nq\n");
	exit(0);
}
