/* gtkutils.c - interface for making some widgets
 *
 * Copyright (C) 2004 Rik Snel <snel@phys.uu.nl>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; only version 2 of the License, not
 * any other version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. 
 *
 * $Id: taf.h 462 2004-04-03 22:31:26Z rsnel $
 */
#include <gtk/gtk.h>
#include "gtkutils.h"

GtkWidget *la_taf_boolean_widget(int *state, GtkBox *box, char *name) {
	GtkWidget *widget;
	widget = gtk_check_button_new_with_label(name);
	gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(widget), *state);
	gtk_box_pack_start_defaults(box, widget);
	return widget;
}

static GtkWidget *prepend(GtkBox *box, GtkWidget *widget, char *name) {
	GtkWidget *label, *hbox;
	label = gtk_label_new(name);
	hbox = gtk_hbox_new(FALSE, 1);
	gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, TRUE, 0);
	gtk_box_pack_start(GTK_BOX(hbox), widget, TRUE, TRUE, 0);
	gtk_box_pack_start_defaults(box, hbox);
	return widget;
}

GtkWidget *la_taf_string_widget(char **string, GtkBox *box, char *name) {
	GtkWidget *widget = gtk_entry_new();
	gtk_entry_set_text(GTK_ENTRY(widget), *string);
	return prepend(box, widget, name);
}

GtkWidget *la_taf_range_widget(int *value, GtkBox *box, int limit, char *name) {
	GtkObject *adjustment = gtk_adjustment_new((gdouble)(*value), 0, (gdouble)limit, 1, 1, 0);
	GtkWidget *widget = gtk_spin_button_new(GTK_ADJUSTMENT(adjustment), .5, 0);
	return prepend(box, widget, name);
}

GtkWidget *la_taf_comboboxonly_widget(int *type, int last, char **entries) {
	int i;
	GList *list = NULL;
	GtkWidget *widget;
	for (i = 0; i <= last; i++) 
		list = g_list_append(list, entries[i]);
	widget = gtk_combo_new();
	gtk_editable_set_editable(GTK_EDITABLE(GTK_COMBO(widget)->entry), 
			FALSE);
	gtk_combo_set_popdown_strings(GTK_COMBO(widget), list);
	g_list_free(list);
	gtk_entry_set_text(GTK_ENTRY(GTK_COMBO(widget)->entry), entries[*type]);
	return widget;
}

GtkWidget *la_taf_combobox_widget(int *type, GtkBox *box, int last, char **entries) {
	GtkWidget *widget = la_taf_comboboxonly_widget(type, last, entries);
	gtk_box_pack_start_defaults(box, widget);
	return widget;
}

GtkWidget *la_taf_textview_widget(la_taf_lines_t *lines, GtkBox *box) {
	GtkWidget *view = gtk_text_view_new();
	GtkTextBuffer *buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(view));

	switch (lines->no_lines) {
		case 0:
			gtk_text_buffer_set_text(buffer, "", -1); /* is this needed? */
			break;
		case 1:
			gtk_text_buffer_set_text(buffer, lines->lines[0], -1);
			break;
		default:
			break;
	}
	gtk_box_pack_start_defaults(box, view);
	return view;
}

GtkBox *la_taf_leftwidget_widget(GtkWidget *widget, GtkBox *box) {
	GtkWidget *hbox = gtk_hbox_new(FALSE, 1);
	GtkWidget *vbox = gtk_vbox_new(FALSE, 1);
	gtk_box_pack_start_defaults(GTK_BOX(box), hbox);
	gtk_box_pack_start(GTK_BOX(hbox), widget, FALSE, TRUE, 0);
	gtk_box_pack_start(GTK_BOX(hbox), vbox, TRUE, TRUE, 0);
	return GTK_BOX(vbox);
}

GtkBox *la_taf_leftlabel_widget(char *text, GtkBox *box) {
	GtkWidget *label = gtk_label_new(text);
	return la_taf_leftwidget_widget(label, box);
}

GtkBox *la_taf_enframe_widget(char *title, GtkBox *box) {
	GtkWidget *vbox = gtk_vbox_new(FALSE, 1);
	GtkWidget *widget = gtk_frame_new(title);
	gtk_box_pack_start_defaults(GTK_BOX(box), widget);
	gtk_container_add(GTK_CONTAINER(widget), vbox);
	return GTK_BOX(vbox);
}
