/* taf.h - interface for handling generic .TAF files
 *
 * Copyright (C) 2004 Rik Snel <snel@phys.uu.nl>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; only version 2 of the License, not
 * any other version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. 
 *
 * $Id: taf.h 475 2004-07-18 19:52:33Z rsnel $
 */
#ifndef INCLUDE_LIBADRIFT_TAF_H
#define INCLUDE_LIBADRIFT_TAF_H 1

#include "../verbose.h"
#include "../char_io.h"
#include "../objcache.h"

#define LA_TAF_WUNOBFUSC 0x00000001
#define LA_TAF_WZLIBHIGH 0x00000002
#define LA_TAF_WNOPASSWD 0x00000004

#include "generic.h"
#include "free.h"
#include "copy.h"

struct la_taf_s {
	int version;
	la_taf_data_t data;
	char password[8];
	la_objcache_t oc;
};

void la_taf_init(la_taf_t*);

void la_taf_free(la_taf_t*);

void la_taf_copy(la_taf_t*, la_taf_t*);

void la_taf_parse_version(la_taf_t*, la_char_io_t*, int*);

void la_taf_parse_password(la_taf_t*, la_char_io_t*, int*);

void la_taf_parse_char_io(la_taf_t*, la_char_io_t*);

void la_taf_parse_fp(la_taf_t*, FILE*, char*);

void la_taf_parse_file(la_taf_t*, char*);

int la_taf_has_password(la_taf_t*);

int la_taf_validate_password(la_taf_t*, char*);

void la_taf_write_version(la_taf_t*, la_char_io_t*);

void la_taf_write_password(la_taf_t*, la_char_io_t*);

void la_taf_write_char_io(la_taf_t*, la_char_io_t*, int);

void la_taf_write_fp(la_taf_t*, FILE*, char*, int);

void la_taf_write_file(la_taf_t*, char*, int);

#endif /* INCLUDE_LIBADRIFT_TAF_H */
