/* vim: set filetype=c cindent: spec - Public Domain 2004, Rik Snel
 *
 * --- PRELIMINARY VERSION --- PRELIMINARY VERSION --- PRELIMINARY VERSION ---
 *
 * This file documents in human and machine readable form the structure
 * of (currently only) .TAF files from ADRIFT 3.80, 3.90 and 4.00. It can
 * be used to generate include files, a parser, a writer, memory management,
 * reporting and GUI functions related to the data in aforementioned .taf 
 * files.
 *
 * Goal:
 * - based on the info in this file it must be easy to store the contents 
 *   of .TAF files in a 4.00'ish structure
 * - if there are no fatal parsing errors (expecting an integer and
 *   getting a non integer, or a filesize mismatch, ...) but only warnings,
 *   it must be possible to recreate the exact same (broken) .TAF file 
 *   from the info parsed by this file
 * - there should be no redundant information in this file
 * - this file should specify how a 3.[89]0 must be translated to 4.00
 *
 * So if a .TAF file contains a 0, it must be clear from this file
 * what that zero means, it could mean, for example:
 * - FALSE
 * - 'referenced object'
 * - 'hidden'
 * - 'the first task'
 * - etc...
 *
 * Two important problems are solved:
 * 1. sometimes the order of the information in .TAF files is illogical; i.e.
 *    the fact that '0' means 'the first task' can only be inferred after
 *    it is read, not before
 * 2. sometimes to logically separate things are encoded in a single integer
 *
 * This specification is released to the Public Domain to facilitate
 * the creation of software to deal with .taf files on platforms like
 * unix, PDA's and DOS.
 *
 * $Id: taf.spec 474 2004-07-18 17:05:22Z rsnel $ 
 */
fstruct(intro, output, intro) {
	postlines(lines, (t->version < 400) ? "**" : "\275\320");
}
push(1);
fstruct(win, output, intro);
string(title);
string(author);
iff(t->version < 390) {
	push(1);
}
fstruct(dont_understand, output);
fstruct(perspective, perspective) {
	aswitch() {
		legenda(first_person);
		legenda(second_person);
		condition(t->version > 390);
		legenda(third_person);
	}
}
boolean(show_exists);
range(wait_turns, 99);
iff(t->version >= 390) {
	boolean(display_first_room);
	boolean(battle_system);
}
iff(t->version < 390) {
	pop(1);
}
noneg(max_score);
fstruct(player, player) {
	iff(t->version < 390) {
		fake("", "");
	}
	string(name);
	iff(t->version >= 390) {
		space();
		boolean(prompt_name);
	}
	floop(descriptions, description, prim, 1, 0) {
		fstruct(desc_if, output) {
			line(lines);
		}
	}
	iff(t->version < 390) {
		fake(0);
	}
	floop(descriptions, description, alt_pdesc_task, 1, 1) {
		compound(i!=0, 0, i, i);
		boolean(valid);
		floop(restrictions, restriction, description, 1, 0) {
			fstruct(spec.task, state_of_task_restriction, pdesc) {
				fstruct(completed, task_ref, pt) {
					kswitch() {
						legenda(not_selected);
						noneg(task);
					}
				}
			}
		}
	}
	floop(descriptions, description, alt_pdesc_ifval, 1, 1) {
		iff(d->valid) {
			fstruct(desc_if, output);
		}
	}
	fstruct(location, person_location) {
		pop(1);
		fstruct(room, room_spec) {
			kswitch() {
				noneg(room);
			}
		}
		iff(t->version > 380) {
			fstruct(posture, person_posture) {
				kswitch() {
					fstruct(standing_on, object_spec) {
						kswitch() {
							legenda(the_floor);
							noneg(chairlike);
						}
					}
					fstruct(sitting_on, object_spec);
					fstruct(lying_on, object_spec, floor_or_bed) {
						kswitch() {
							legenda(the_floor);
							noneg(bedlike);
						}
					}
				}
			}
		}
	}
	iff(t->version > 380) {
		fstruct(gender, person_gender) {
			kswitch() {
				legenda(male);
				legenda(female);
				legenda(prompt);
			}
		}
	}
	iff(t->version < 390) {
		fake(50, "2", 50, 2);
	}
	iff(t->version >= 390) {
		compound(i/10, 10*i, i%10, i);
	}
	range(size_no_carriable, 99);
	fstruct(size_carriable, size) {
		aswitch() {
			legenda(tiny);
			legenda(small);
			legenda(normal);
			legenda(large);
			legenda(huge);
		}
	}
	iff(t->version >= 390) {
		compound(i/10, 10*i, i%10, i);
	}
	range(weight_no_carriable, 99);
	fstruct(weight_carriable, weight) {
		aswitch() {
			legenda(very_light);
			legenda(light);
			legenda(normal);
			legenda(heavy);
			legenda(very_heavy);
		}
	}
	iff(t->data.battle_system) {
		fake(0);
		fstruct(battle_parameters, person_battle_parameters) {
			boolean(is_character);
			iff(d->is_character) {
				fstruct(to, character_attitude) {
					aswitch() {
						legenda(ally);
						legenda(neutral);
						legenda(enemy);
					}
				}
			}
			iff(t->version >= 400) {
				range(min_stamina, 100);
				range(max_stamina, 100);
				range(min_strength, 100);
				range(max_strength, 100);
				range(min_accuracy, 100);
				range(max_accuracy, 100);
				range(min_defense, 100);
				range(max_defense, 100);
				range(min_agility, 100);
				range(max_agility, 100);
			}
			iff(t->version < 400) {
				intg(min_stamina);
				fake(d->min_stamina);
				intg(max_stamina);
				intg(min_strength);
				fake(d->min_strength);
				intg(max_strength);
				intg(min_defense);
				fake(d->min_defense);
				intg(max_defense);
			}
			iff(d->is_character) {
				fstruct(attack_speed, attack_speed) {
					aswitch() {
						legenda(every_turn);
						legenda(most_turns);
						legenda(every_second_turn);
						legenda(every_third_turn);
						legenda(every_fourth_turn);
					}
				}
				noneg(task_run_if_killed);
				iff(t->version >= 400) {
					noneg(task_if_stamina_low);
				}
			}
			iff(t->version >= 400) {
				noneg(automatic_stamina_recovery);
			}
		}
	}
}
iff(t->version >= 390) {
	boolean(eightp_compass);
	boolean(disable_debugger);
	boolean(disable_display_score);
	boolean(disable_map);
	boolean(disable_autocomplete);
	boolean(disable_control_panel);
	boolean(disable_receive_mouseclicks);
	boolean(enable_sound);
	boolean(enable_graphics);
}
iff(t->data.enable_sound || t->data.enable_graphics) {
	fstruct(intro, output, media) {
		fstruct(media, media) {
			iff(t->data.enable_sound) {
				string(sound_filename);
				iff(t->version >= 400) {
					space();
					noneg(sound_size);
				}
				buffer_for_later(sound_data, d->sound_size, d->sound_filename);
			}
			iff(t->data.enable_graphics) {
				string(graphics_filename);
				iff(t->version >= 400) {
					space();
					noneg(graphics_size);
				}
				buffer_for_later(graphics_data, d->graphics_size, d->graphics_filename);
			}
		}
	}
}
iff(t->data.enable_sound || t->data.enable_graphics) {
	fstruct(win, output, media);
}
iff(t->version >= 400) {
	fstruct(user_status_box, user_status_box) {
		aswitch() {
			exp("", NULL);
			legenda(none);
			string(contents);
		}
	}
}
iff(t->version >= 390) {
	/* these two may be in the wrong order */
	noneg(size_factor);
	noneg(weight_factor);
}
iff(t->version >= 400) {
	boolean(datafiles_embedded);
}
autofloop(rooms, room, 0) {
	floop(descriptions, description, prim_rdesc, 1, 0) {
		string(sdesc);
		fstruct(desc_if, output);
	}
	iff(t->version < 400) {
		floop(descriptions, description, prim, 1, 2);
	}
	fstruct(north, direction) {
		compound(i!=0, i, i!=0?i-1:0, i);
		aswitch() {
			skip();
			legenda(invalid);
			fstruct(dest, destspec) {
				noneg(room);
				exp(0, NULL);
				iff(t->version >= 400) {
					exp(0, 1);
				}
				exp(0, 1);
			}
		}
	}
	fstruct(east, direction);
	fstruct(south, direction);
	fstruct(west, direction);
	fstruct(up, direction);
	fstruct(down, direction);
	fstruct(in, direction);
	fstruct(out, direction);
	iff(t->data.eightp_compass) {
		fstruct(northeast, direction);
		fstruct(southeast, direction);
		fstruct(southwest, direction);
		fstruct(northwest, direction);
	}	
	iff(t->version < 400) {
		floop(descriptions, description, prim, 1, 3);
		compound("i", "i", "i", "0");
		floop(descriptions, description, alt_pdesc_task, 1, 2);
		floop(descriptions, description, not, 1, 2) {
			floop(restrictions, restriction, not, 1, 0) {
				fake(1);
				boolean(negate);
			}
		}
		floop(descriptions, description, alt_pdesc_task, 1, 3);
		floop(descriptions, description, prim, 1, 4);
		floop(descriptions, description, alt_pdesc_task, 1, 4);
		floop(descriptions, description, alt3, 1, 5) {
			push(1);
			fstruct(desc_if, output);
			compound(i%10, i, i/10, i*10);
			boolean(hide_objects_in_room);
			floop(restrictions, restriction, room3_description, 1, 0) {
				compound(1-i%2, 1-i, i/2, i*2);
				boolean(negate);
				exp(0, 1, 2);
				pop(1);
				iff(d->exp0 == 0) {
					fstruct(spec.object, object_restriction, rd2) {
						floop(lhss, object_spec, not_selected_or_dynamic, 1, 0) {
							kswitch() {
								legenda(not_selected);
								noneg(dynamic);
							}
						}
						fstruct(spec.posture, dynamic_posture, rd2) {
							fstruct(spec.held_by, person_spec, player) {
								legenda(spec.player);
							}
						}
					}
				}
				eliff(d->exp0 == 1) {
					fstruct(spec.object, object_restriction, rd3) {
						floop(lhss, object_spec, not_selected_or_clothing, 1, 0) {
							kswitch() {
								legenda(not_selected);
								noneg(clothing);
							}
						}
						fstruct(spec.posture, dynamic_posture, rd3) {
							fstruct(spec.worn_by, person_spec, player);
						}
					}
				}
				eliff(d->exp0 == 2) {
					fstruct(spec.object, object_restriction, rd4) {
						floop(lhss, object_spec, not_selected_or_dynamic, 1, 0); 
						fstruct(spec.room, room_spec, rd4) {
							fstruct(spec.same_as, person_spec, player);
						}
					}
				}
			}
		}
	}
	floop(descriptions, description, media, 1, 0) {
		fstruct(desc_if, output, media);
	}
	iff(t->version < 400) {
		floop(descriptions, description, media, 1, 2);
		floop(descriptions, description, media, 1, 3);
		floop(descriptions, description, media, 1, 4);
		floop(descriptions, description, media, 1, 5);
	}
	iff(t->version >= 400) {
		autofloop(descriptions, description, 1) {
			fstruct(desc_if, output);
			push(1);
			fstruct(desc_if, output, media);
			fstruct(desc_if_not, output);
			push(1);
			fstruct(desc_if_not, output, media);
			boolean(hide_objects_in_room);
			string(sdesc);
			pop(2);
			floop(restrictions, restriction, room_description, 1, 0) {
				range(pwtype, 2);
				iff(d->pwtype == 0) {
					fstruct(spec.task, state_of_task_restriction, nt) {
						fstruct(completed, task_ref, nt) {
							kswitch() {
								legenda(none);
								noneg(task);
							}
						}
					}
				}
				eliff(d->pwtype == 1) {
					fstruct(spec.object, object_restriction, rd) {
						floop(lhss, object_spec, none_or_stateobject, 1, 0) {
							kswitch() {
								legenda(none);
								noneg(stateful);
							}
						}
						compound(i, i, 0, 0);
						noneg(spec.state);
					}
				}
				eliff(d->pwtype == 2) {
					compound(1-i%2, 1-i, i/2, i*2);
					push(1);
					exp(0, 1, 2);
					iff(d->exp0 == 0) {
						fstruct(spec.object, object_restriction, rd2);
					}
					eliff(d->exp0 == 1) {
						fstruct(spec.object, object_restriction, rd3);
					}
					eliff(d->exp0 == 2) {
						fstruct(spec.object, object_restriction, rd4);
					}
					pop(1);
				}
				boolean(negate);
			}
			aswitch() {
				legenda(start_with_this);
				legenda(start_directly_after_main);
				legenda(append_to_other);
			}
			fake(1);
			boolean(valid);
		}
	}
	iff(!t->data.disable_map && t->version >= 390) boolean(hide_on_map);
}
autofloop(objects, object, 0) {
	string(prefix);
	string(name);
	iff(t->version < 400) {
		fake(1);
	}
	noneg(no_aliases);
	floop(aliases, vlines, d->no_aliases, 0) {
		string(line); /* FIXME add modifyer to demand nonempty */
		fake(strlen(d->line)?1:0);
		boolean(valid);
	}
	boolean(is_static);
	floop(descriptions, description, prim, 1, 0); 
	push(1);
	floop(descriptions, description, alt_pdesc_task, 1, 1);
	floop(descriptions, description, not2, 1, 1) {
		floop(restrictions, restriction, not2, 1, 0) {
			boolean(negate);
		}
	}
	floop(descriptions, description, prim, 1, 1);
	iff(d->is_static) {
		pop(1);
		noneg(redundant); 
		fstruct(slocation, static_location) {
			aswitch() {
				legenda(nowhere);
				fstruct(one_place, one_place) {
					compound(i!=0, 0, i-1, i+1);
					aswitch() {
						exp(-1);
						legenda(on_character);
						noneg(room);
					}
				}
				fstruct(some_places, some_places) {
					boolean(on_character);
					floop(some_rooms, boollist, NO_ROOMS, 0) {
						boolean(in_room);
					}
				}
				legenda(all_rooms_and_on_character);
				condition(t->version >= 400);
				legenda(on_character);
			}
		}
	}
	iff(t->version < 390) {
		compound(i&1, i, i>>1, i<<1);
	}
	boolean(is_container);
	boolean(is_surface);
	iff(t->version > 380) {
		compound(i/10, 10*i, i%10, i);
	}
	range(no_containable, 99);
	iff(t->version < 390) {
		fake(2);
	}
	fstruct(size_containable, size);
	iff(!d->is_static) {
		boolean(is_clothing);
		iff(t->version >= 390) {
			compound(i/10, 10*i, i%10, i); 
		}
		iff(t->version < 390) {
			compound(i<3?2:i, 2*i+(i==3), i<3?i+2:2, i-6);
		}
		fstruct(size, size);
		fstruct(weight, weight);
		pop(1);
		iff(t->version >= 390) {
			compound((i>0&&i<4)||i==4+%d?-1:i==0?0:i-3, i==-1?1:i==0?-3:i, i>0&&i<4?i-1:i==4+%d?4:3, i==4?3+%d:i, NO_ROOMS);
		}
		iff(t->version < 390) {
			compound((i>0&&i<3)||i==3+%d?-1:i==0?0:i-2, i==-1?1:i==0?-3:i-1, i>0&&i<3?i-1:i==3+%d?4:3, i==4?2+%d:i, NO_ROOMS);
		}
		fstruct(location, dynamic_location) {
			kswitch() {
				fstruct(definite, definite_dynamic_location) {
					compound("i+1", "i-1");
					fstruct(room, room_spec, unspec) {
						kswitch() {
							legenda(posture);
							legenda(none);
							noneg(room);
						}
					}
					fstruct(posture, dynamic_posture) {
						kswitch() {
							fstruct(held_by, person_spec) {
								kswitch() {
									legenda(player);
									noneg(character);
								}
								fake(0);
								exp(0);
							}
							fstruct(in, object_spec, container) {
								noneg(spec.container);
							}
							fstruct(on, object_spec, surface) {
								noneg(spec.surface);
							}
							exp(-1,0);
							legenda(around);
							condition(COBJ->is_clothing);
							fstruct(worn_by, person_spec);
						}
					}
				}
			}
		}
	}
	iff(d->is_static && d->slocation.type == 4) {
		fstruct(slocation, static_location, charpart) {
			noneg(part_of_which_character);
		}
	}
	iff(t->version >= 400) {
		compound(i!=0, i==0?-4:i, i==0?0:i-5, i+4);
	}
	eliff(t->version >= 390) {
		compound(i!=0, i==0?-5:i, i==0?0:6-i, 5-i);
	}
	eliff(t->version >= 380) {
		/* one 3.80 file has a 1 in this place, interpret as not_openable, 
	 	 * and emit warning */
		compound(i!=0&&i!=1, i==0?-5:i, i==0||i==1?-i:6-i, 5-i);
	}
	fstruct(open_info, open_info) {
		aswitch() {
			exp(0);
			legenda(not_openable);
			fstruct(openable, openable) {
				iff(t->version >= 400) {
					push(1);
					compound(i>=0, 0, i, i);
				}
				iff(t->version < 400) {
					fake(0,-1);
				}
				aswitch() {
					exp(-1);
					legenda(not_lockable);
					noneg(lockable_with_key_);
				}
				iff(t->version >= 400) {
					pop(1);
				}
				fstruct(open_state, open_state) {
					kswitch() {
						legenda(open);
						legenda(closed);
						condition(COBJ->open_info.spec.openable.type != 0);
						legenda(locked);
					}
				}
			}
		}
	}
	compound(i&1, i, i>>1, i<<1);
	boolean(is_chairlike);
	boolean(is_bedlike);
	iff(!d->is_static) boolean(is_food);
	boolean(is_readable);
	iff(d->is_readable) string(text);
	iff(!d->is_static) boolean(is_weapon);
	iff(0) boolean(is_transparent);
	iff(0) boolean(is_plural);
	iff(t->version >= 400) {
		compound(i!=0, 0, i, i);
		boolean(is_stateful);
		noneg(start_state);
		iff(d->start_state) {
			string(possible_states);
			boolean(show_in_description);
		}
		boolean(add_in_roomdesc);
	}
	floop(descriptions, description, media, 1, 0);
	fstruct(alt, media);
	iff(t->data.battle_system) {
		integt(protective_value, -1);
		noneg(hit_value);
		aswitch() {
			legenda(chop);
			legenda(cut);
			legenda(hit);
			legenda(shoot);
			legenda(stab);
			legenda(throw);
		}
		iff(t->version >= 400) {
			noneg(accuracy);
		}
	}
	iff(t->version >= 400) {
		string(inroomdesc_desc);
		// FIXME, THIS IS WRONG! (but it parses...)
		boolean(onlyshow);
		iff(d->onlyshow && ((!d->is_static && d->add_in_roomdesc))) {
			boolean(also_if_returned);
		}
	}
}
autofloop(tasks, task, 0) {
	iff(t->version < 400) {
		compound(i>=0?i+1:i, i>0?i-1:i);
	}
	iff(t->version >= 400) {
		compound(i>=1?i:i-1, i<0?i+1:i);
	}
	autofloop(commands, command, 0) {
		string(line);
	}
	fstruct(description, output);
	fstruct(reversed, output);
	fstruct(again, output);
	fstruct(additional, output);
	fstruct(show_description_of, room_spec, none_or_room) {
		kswitch() {
			legenda(none);
			noneg(room);
		}
	}
	boolean(repeatable);
	iff(t->version < 390) {
		floop(actions, action, score380, 1, 1) {
			fstruct(spec.change_score, score_action, score380) {
				intg(by);
				boolean(score_only_counts_once);
			}
		}
		floop(actions, action, movement380, 6, 2) {
			compound(i==1, i, i>0?i-1:i, i>0?i+1:i);
			kswitch() {
				fstruct(object, object_action, movement) {
					floop(lhss, object_spec, nrho, 1, 0) {
						kswitch() {
							legenda(none);
							legenda(referenced);
							legenda(all_held);
							noneg(dynamic);
						}
					}
					push(1);
					kswitch() {
						pop(1);
						fstruct(change_location_to, definite_dynamic_location, hidden_same_room) {
							fstruct(room, room_spec, none_or_sameas_or_room) {
								compound(i<0||i>1?2:i, i, i<0||i>1?i-2:0, i);
								kswitch() {
									skip();
									legenda(none);
									skip();
									fstruct(same_as, person_spec, player);
									noneg(room);
								}
							}
						}
						pop(1);
						fstruct(change_posture_to, dynamic_posture, not_selected_or_container) {
							fstruct(spec.in, object_spec, not_selected_or_container) {
								kswitch() {
									legenda(not_selected);
									noneg(container);
								}
							}
						}
						pop(1);
						fstruct(change_posture_to, dynamic_posture, not_selected_or_surface) {
							fstruct(spec.in, object_spec, not_selected_or_surface) {
								kswitch() {
									legenda(not_selected);
									noneg(surface);
								}
							}
						}
						pop(1);
						fstruct(change_posture_to, dynamic_posture, held_by_pc) {
							fstruct(spec.held_by, person_spec, pc) {
								kswitch() {
									legenda(player);
									noneg(character);
								}
							}
						}
						pop(1);
						fstruct(change_posture_to, dynamic_posture, worn_by_pc) {
							fstruct(spec.worn_by, person_spec, pc);
						}
					}
				}
				fstruct(person, person_action, movement) {
					skip();
					floop(lhss, person_spec, player, 1, 0);
					kswitch() {
						compound("i-2", "i+2");
						fstruct(change_location_to, person_location, room) {
							fstruct(room, room_spec);
						}
					}
					exp(0);
				}
			}
		}
	}
	boolean(reversible);
	iff(t->version < 400) {
		compound(i+1, i-1);
	}
	autofloop(rcommands, rcommand, 0) {
		string(line);
	}
	iff(t->version < 390) {
		floop(restrictions, restriction, wear380lhs, 1, 1) {
			fstruct(spec.object, object_restriction, wear380lhs) {
				floop(lhss, object_spec, wear380, 2, 0) {
					kswitch() {
						legenda(not_selected);
						legenda(any);
						legenda(none);
						noneg(clothing);
					}
				}
				fstruct(spec.posture, dynamic_posture, rd3); 
			}
		}
		floop(restrictions, restriction, holdsr, 1, 3) {
			fstruct(spec.object, object_restriction, holdsr3) {
				floop(lhss, object_spec, nro, 3, 0) {
					kswitch() {
						legenda(not_selected);
						legenda(referenced);
						noneg(dynamic);
					}
				}
			}
		}
		floop(restrictions, restriction, rmloc, 1, 6) {
			fstruct(spec.object, object_restriction, holdsr) {
				floop(lhss, object_spec, nro, 1, 0);
			}
		}
		floop(restrictions, restriction, task380, 1, 7) {
			fstruct(spec.task, state_of_task_restriction, task380) {
				fstruct(completed, task_ref, nt);
			}
			boolean(negate);
			fstruct(text, output);
		}
		floop(restrictions, restriction, text380, 1, 3);
		floop(restrictions, restriction, text380, 1, 1) {
			fstruct(text, output);
		}
		floop(restrictions, restriction, company380, 1, 8) {
			fstruct(text, output);
			boolean(negate);
			fstruct(spec.person, person_restriction, sr380) {
				fstruct(spec.room, room_spec, sr380) {
					fstruct(spec.same_as, person_spec, sr380) {
						kswitch() {
							legenda(not_selected);
							legenda(nobody);
							noneg(character);
						}
					}
				}
			}
		}
		floop(restrictions, restriction, obj1, 1, 6) {
			fstruct(text, output);
			fstruct(spec.object, object_restriction, none_or_room) {
				fstruct(spec.room, room_spec, none_or_room);
			}
		}
	}
	fstruct(rooms, roomlist) {
		aswitch() {
			legenda(no_rooms);
			noneg(one_room);
			floop(some_rooms, boollist, NO_ROOMS, 0);
			legenda(all_rooms);
		}
	}
	iff(t->version < 390) {
		floop(actions, action, kill_player, 1, 3) {
			boolean(valid);
			fstruct(spec.end_game, end_game_action, kill) {
				kswitch() {
					legenda(die);
				}
			}
		}
		floop(restrictions, restriction, holdsr_rhs, 1, 3) {
			fstruct(spec.object, object_restriction, holdsr_rhs) {
				kswitch() {
					fstruct(posture, dynamic_posture, rd2);
					fstruct(room, room_spec, rd4);
				}
			}
		}
	}
	string(hint_question);
	iff(strlen(d->hint_question)) {
		string(subtle_hint);
		string(sledgehammer_hint);
	}
	iff(t->version >= 390) {
		autofloop(restrictions, restriction, 0) {
			fake(1);
			boolean(valid);
			kswitch() {
				fstruct(object, object_restriction) {
					floop(lhss, object_spec, naro, 1, 0) {
						kswitch() {
							legenda(none);
							legenda(any);
							legenda(referenced);
							noneg(dynamic);
						}
					}
					compound(i>=6?1:0, i==1?6:0, i>=6?i-6:i, i);
					push(1);
					kswitch() {
						fstruct(room, room_spec, none_or_room);
						fstruct(posture, dynamic_posture, restriction_held_by) {
							fstruct(spec.held_by, person_spec, refd) {
								kswitch() {
									legenda(player);
									legenda(refd_char);
									noneg(character);
								}
							}
						}
						fstruct(posture, dynamic_posture, restriction_worn_by) {
							fstruct(spec.worn_by, person_spec, refd);
						}
						fstruct(general_location, dynamic_location, restriction) {
							fstruct(spec.visible_to, person_spec, refd);
						}
						fstruct(posture, dynamic_posture, restriction_in) {
							fstruct(spec.in, object_spec, container_or_not) {
								kswitch() {
									legenda(none);
									noneg(container);
								}
							}
						}
						fstruct(posture, dynamic_posture, restriction_on) {
							fstruct(spec.on, object_spec, surface_or_not) {
								kswitch() {
									legenda(none);
									noneg(clothing);
								}
							}
						}
					}
					pop(1);
				}
				fstruct(object, object_restriction, state) {
					floop(lhss, object_spec, refd_or_stateobject, 1, 0) {
						kswitch() {
							legenda(referenced);
							noneg(stateful);
						}
					}
					compound(i, i, 0, 0);
					noneg(spec.state);
				}
				fstruct(task, state_of_task_restriction) {
					fstruct(completed, task_ref) {
						kswitch() {
							noneg(task);
						}
					}
				}
				fstruct(person, person_restriction) {
					floop(lhss, person_spec, refd, 1, 0);
					/* 'alone'<->'not alone' */
					compound(i==2?3:i==3?2:i, i==2?3:i==3?2:i); 
					compound(i<4?i%2:0, i, i<4?i/2:i-2, i<2?2*i:i+2);
					push(1);
					kswitch() {
						fstruct(room, room_spec, same_as) {
							fstruct(spec.same_as, person_spec, refd); 
						}
						fstruct(room, room_spec, modpr2) {
							fstruct(spec.same_as, person_spec, any) {
								legenda(spec.any_person);
								exp(-1);
							}
						}
						condition(((la_taf_person_spec_t*)d->lhss.ptr)->realtype == 0);
						fstruct(posture, person_posture, standing) {
							fstruct(spec.standing_on, object_spec);
						}
						fstruct(posture, person_posture, sitting) {
							fstruct(spec.sitting_on, object_spec);
						}
						fstruct(posture, person_posture, lying) {
							fstruct(spec.lying_on, object_spec, floor_or_bed);
						}
						fstruct(of_gender, person_gender, restriction) {
							kswitch() {
								legenda(male);
								legenda(female);
							}
						}
					}
					pop(1);
				}
				fstruct(variable, variable_restriction) {
					noneg(lhs);
					compound(i%10>=3, 3*i, (i%10)%3, i, i>=10, 10*i);
					push(1);
					aswitch() {
						fstruct(less_than, cov) {
							aswitch() {
								intg(constant);
								noneg(variable);
							}
						}
						fstruct(less_than_or_equal_to, cov);
						fstruct(equal_to, cov);
					}
					iff(t->version < 400) {
						fake("");
					}
					exp("");
					pop(1);
				}
			}
			boolean(negate);
			fstruct(text, output);
			fake(0);
			exp(0);
		}
		autofloop(actions, action, 0) {
			fake(1);
			boolean(valid);
			iff(t->version < 400) {
				compound(i>4?i+1:i, i>4?i-1:i);
			}
			kswitch() {
				fstruct(object, object_action) {
					floop(lhss, object_spec, hwro, 1, 0) {
						kswitch() {
							legenda(all_held);
							legenda(referenced);
							noneg(dynamic);
						}
					}
					kswitch() {
						fstruct(change_location_to, definite_dynamic_location, room) {
							fstruct(room, room_spec, none_or_room);
						}
						fstruct(change_location_to, definite_dynamic_location, group) {
							fstruct(room, room_spec, group) {
								noneg(spec.group);
							}
						}
						fstruct(change_posture_to, dynamic_posture, container) {
							fstruct(spec.in, object_spec, container);
						}
						fstruct(change_posture_to, dynamic_posture, surface) {
							fstruct(spec.on, object_spec, surface);
						}
						fstruct(change_posture_to, dynamic_posture, held_by) {
							fstruct(spec.held_by, person_spec, refd);
						}
						fstruct(change_posture_to, dynamic_posture, worn_by) {
							fstruct(spec.worn_by, person_spec, refd);
						}
						fstruct(move_to_same_room_as, person_spec, refd);
					}
				}
				fstruct(person, person_action) {
					iff(0) {
						exp(0);
					}
					floop(lhss, person_spec, refd, 1, 0);
					kswitch() {
						iff(((la_taf_person_spec_t*)d->lhss.ptr)->realtype == 0) {
							compound("i+1", "i-1");
						}
						fstruct(change_location_to, person_location, none_or_room) {
							fstruct(room, room_spec, none_or_room);
						}
						fstruct(change_location_to, person_location, group) {
							fstruct(room, room_spec, group);
						}
						fstruct(change_location_to, person_location, same_room_as) {
							fstruct(room, room_spec, same_as);
						}
						condition(((la_taf_person_spec_t*)d->lhss.ptr)->realtype == 0); 
						fstruct(change_posture_to, person_posture, standing);
						fstruct(change_posture_to, person_posture, sitting);
						fstruct(change_posture_to, person_posture, lying);
					}
				}
#if 0 /* should indeed be dealt with... */
						/* ADRIFT 3.90 generator seems to
						   contain a bug that allows a non-
						   sensical value of d->type,
						   deal with it gracefully */
#endif
				fstruct(state_of_object_action, state_of_object) {
					floop(lhss, object_spec, none_or_stateobject, 1, 0);
					noneg(state);
				}
				fstruct(variable_action, variable_action) {
					noneg(variable);
					aswitch() {
						fstruct(to_value, exact_value) {
							intg(value);
							iff(t->version >= 400) {
								exp("");
							}
							exp(0);
						}
						fstruct(by_value, exact_value); 
						fstruct(to_between, between_values) {
							intg(value1);
							iff(t->version >= 400) {
								exp("");
							}
							integt(dvalue2, d->value1);
						}
						fstruct(by_between, between_values);
						fstruct(to_referenced, referenced_value) {
							exp(0);
							iff(t->version >= 400) {
								exp("");
							}
							exp(0);
						}
						fstruct(to_expression, expression_value) {
							exp(0);
							string(expr);
							iff(t->version >= 400) {
								exp(0);
							}
						}
					}
				}
				fstruct(change_score, score_action) {
					intg(by);
					fake(1);
					boolean(score_only_counts_once);
				}
				fstruct(task, state_of_task_action) {
					boolean(execute);
					fstruct(task, task_ref, task) {
						noneg(spec.task);
					}
				}
				fstruct(end_game, end_game_action) {
					kswitch() {
						legenda(win);
						legenda(dont_win);
						legenda(die);
						condition(t->version >= 400);
						legenda(die_silently);
					}
					iff(t->version >= 400) {
						exp(0, NULL);
						exp(0, NULL);
					}
				}
				condition(t->data.battle_system);
				fstruct(change_battle_parameters, change_battle_parameters) {
					iff(t->version < 400) {
						compound(i<=4?i:i<=6?i+2:i+4, i<=4?i:i<=8?i-2:i-4);
					}
					aswitch() {
						fstruct(set_attitude, set_attitude) {
							fstruct(of, person_spec, rchar) {
								kswitch() {
									legenda(refd_char);
									noneg(character);
								}
							}
							fstruct(to, character_attitude);
						}
						fstruct(set_stamina, set_pb) {
							fstruct(of, person_spec, refd);
							intg(to);
						}
						fstruct(set_max_stamina, set_pb);
						fstruct(set_strength, set_pb);
						fstruct(set_max_strength, set_pb);
						fstruct(set_accuracy, set_pb);
						fstruct(set_max_accuracy, set_pb);
						fstruct(set_defense, set_pb);
						fstruct(set_max_defense, set_pb);
						fstruct(set_agility, set_pb);
						fstruct(set_max_agility, set_pb);
						fstruct(set_attack_speed, set_attack_speed) {
							fstruct(character, person_spec, rchar);
							fstruct(to, attack_speed);
						}
					}
				}
			}
		}
	}
	iff(t->version >= 400) string(restriction_mask);
	fstruct(description, output, media);
	iff(t->version < 390) {
		floop(restrictions, restriction, obj2, 1, 9) {
			fstruct(spec.object, object_restriction, bla) {
				floop(lhss, object_spec, dynamic, 1, 0) {
					space();
					kswitch() {
						legenda(not_selected);
						noneg(dynamic);
					}
				}
				iff(((la_taf_object_spec_t*)d->lhss.ptr)->pwtype) {
					space();
					kswitch() {
						space();
						fstruct(empty, empty_object_restriction) {
							exp(0);
						}
						space();
						fstruct(posture, dynamic_posture, not_selected_or_container);
						space();
						fstruct(posture, dynamic_posture, not_selected_or_surface);
						space();
						noneg(state);
						space();
						compound(i+1, i-1);
						noneg(state);
						space();
						fstruct(posture, dynamic_posture, held_by_c) {
							fstruct(spec.held_by, person_spec, nc) {
								kswitch() {
									legenda(not_selected);
									noneg(character);
								}
							}
						}
						space();
						fstruct(posture, dynamic_posture, worn_by_c) {
							fstruct(spec.worn_by, person_spec, nc);
						}

					}
				}
			}
			iff(((la_taf_object_spec_t*)d->spec.object.lhss.ptr)->pwtype) {
				fstruct(text, output);
			}
		}
		floop(actions, action, win, 1, 4) {
			boolean(valid);
			fstruct(spec.end_game, end_game_action, win) {
				kswitch() {
					legenda(win);
				}
			}
		}
	}
}
autofloop(events, event, 0) {
	string(name);
	compound(i-1, i+1);
	aswitch() {
		legenda(immediately);
		fstruct(between_turns, between_turns) {
			noneg(turn1);
			integt(turn2, d->turn1);
		}
		noneg(start_when_this_task_is_complete);
	}
	range(restart, 2);
	boolean(complete);
	noneg(duration1);
	integt(duration2, d->duration1);
	string(startdesc);
	string(lookdesc);
	string(finishdesc);
	fstruct(visibe_in, roomlist);
	noneg(pauseevent);
	boolean(pauseevent_notcompleted);
	noneg(tfromfinish1);
	string(fromfinish1desc);
	noneg(resumeevent);
	boolean(resumeevent_notcompleted);
	noneg(tfromfinish2);
	string(fromfinish2desc);
	noneg(object1);
	noneg(fmove_object1_to);
	noneg(object2);
	noneg(fmove_object2_to);
	noneg(object);
	noneg(smove_object_to);
	noneg(task_at_finish);
	fstruct(start, media);
	fstruct(look, media);
	fstruct(end, media);
	fstruct(time1, media);
	fstruct(time2, media);
}
autofloop(characters, character, 0) {
	string(name);
	string(prefix);
	iff(t->version < 400) {
		fake(1);
	}
	noneg(no_descriptors);
	floop(descriptors, vlines, d->no_descriptors, 0);
	floop(descriptions, description, prim, 1, 0); 
	fstruct(location, person_location, modc) {
		fstruct(room, room_spec, none_or_room);
	}
	floop(descriptions, description, alt_task, 1, 1) {
		fstruct(desc_if, output);
		compound(i!=0, 0, i, i);
		boolean(valid);
		floop(restrictions, restriction, description, 1, 0);
	}
	autofloop(topics, topic, 0) {
		string(subject);
		floop(responses, description, prim, 1, 0); 
		floop(descriptions, description, alt_rev_task, 1, 1) {
			space();
			compound(i!=0, 0, i, i);
			boolean(valid);
			floop(restrictions, restriction, description, 1, 0);
			fstruct(desc_if, output);
		}
	}
	autofloop(walks, walk, 0) {
		push(1);
		boolean(loop);
		noneg(start);
		iff(t->version >= 400) {
			noneg(character_spec);
		}
		noneg(task_across_character);
		noneg(object);
		noneg(across_object);
		iff(t->version >= 390) {
			noneg(across_end);
		}
		iff(t->version < 390) {
			fake("");
		}
		string(altdesc);
		pop(1);
		noneg(no_whereabouts);
		floop(whereabouts, whereabout, d->no_whereabouts, 0) {
			noneg(room);
			noneg(time);
		}
	}
	boolean(enterleave);
	iff(d->enterleave) {
		floop(enter_desc, description, prim, 1, 0); 
		floop(leave_desc, description, prim, 1, 0); 
	}
	floop(present_desc, description, prim, 1, 0); 
	iff(t->version > 380) {
		fstruct(gender, person_gender, character) {
			kswitch() {
				legenda(male);
				legenda(female);
				legenda(undefined);
			}
		}
	}
	floop(descriptions, description, media, 1, 0);
	floop(descriptions, description, media, 1, 1);
	floop(enter_desc, description, media, 1, 0);
	floop(leave_desc, description, media, 1, 0);
	iff(t->data.battle_system) {
		fake(1);
		fstruct(battle_parameters, person_battle_parameters);
	}
}
autofloop(groups, group, 0) {
	string(name);
	floop(roomref, boollist, NO_ROOMS, 0);
}
autofloop(synonyms, synonym, 0) {
	string(word);
	string(also);
}
iff(t->version >= 390) {
	autofloop(variables, variable, 0) {
		string(name);
		iff(t->version == 390) {
			fake(0);
		}
		fstruct(value, variable_value) {
			kswitch() {
				intg(integer);
				string(string);
			}
		}
	}
	autofloop(alrs, alr, 0) {
		string(message);
		string(altmessage);
	}
	fstruct(alternate_font, alternate_font) {
		aswitch() {
			legenda(none);
			string(name);
		}
	}
}
string(date);
