/* verbose.h - interface for handling verboseness
 *
 * Copyright (C) 2004 Rik Snel <snel@phys.uu.nl>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; only version 2 of the License, not
 * any other version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. 
 *
 * $Id: verbose.h 467 2004-07-16 22:08:50Z rsnel $
 */
#ifndef INCLUDE_LIBADRIFT_VERBOSE_H
#define INCLUDE_LIBADRIFT_VERBOSE_H 1

#include <stdarg.h>
#include <stdio.h>
#include "config.h"
#include "exception.h"

#define LA_VERBOSE_NO_TYPES 15
#define LA_VERBOSE_NO_LEVELS 6
#define LA_VERBOSE_NO_CHANNEL_TYPES 1

#define LA_FATAL(...) do { \
	la_verbose_output(&la_verbose_fatal_channel, "fatal:"); \
	la_verbose_output(&la_verbose_fatal_channel, __VA_ARGS__); \
	la_verbose_output(&la_verbose_fatal_channel, "\n"); \
	exit(1); \
} while (0)
#define LA_ERROR(...) la_verbose(LA_VERBOSE_TYPE, Error, __VA_ARGS__)
#define LA_THROW(...) do { \
	if (!current_exception_context) LA_FATAL(__VA_ARGS__); \
	LA_ERROR(__VA_ARGS__); \
	throw(); \
} while (0)
#define LA_WARNING(...) la_verbose(LA_VERBOSE_TYPE, Warning, __VA_ARGS__)
#define LA_VERBOSE(...) la_verbose(LA_VERBOSE_TYPE, Verbose, __VA_ARGS__)
#ifndef NDEBUG
#define LA_DEBUG(...) la_verbose(LA_VERBOSE_TYPE, Debug, __VA_ARGS__)
#define LA_DBG2(...) la_verbose(LA_VERBOSE_TYPE, Dbg2, __VA_ARGS__)
#define LA_DBG3(...) la_verbose(LA_VERBOSE_TYPE, Dbg3, __VA_ARGS__)
#else
#define LA_DEBUG(...)
#define LA_DBG2(...)
#define LA_DBG3(...)
#endif

typedef enum { Char_IO, Taf_Parser, Taf_Freer, Taf_Writer, 
	Memory, Line_In, Cex, Line_Out, Taf_Copier,
	IntStack, Slot, ObjCache, Action, Taf_GTK,
	Resolve } la_verbose_type_t;

typedef enum { Error, Warning, Verbose, Debug, Dbg2, Dbg3 } la_verbose_level_t;

typedef enum { FP } la_verbose_channel_type_t;

typedef struct {
	la_verbose_channel_type_t type;
	void *priv;
} la_verbose_channel_t;

typedef struct {
	int no_channels;
	la_verbose_channel_t *channels;
} la_verbose_t;

typedef la_verbose_t la_verbosity_t[LA_VERBOSE_NO_TYPES][LA_VERBOSE_NO_LEVELS];

extern la_verbosity_t *v;

extern la_verbose_channel_t la_verbose_fatal_channel;

void la_verbose_init_fp(FILE*);

void la_verbose_init_fp_quiet(FILE*);

void la_verbose_free(void);

void la_verbose_fp_channel_init(la_verbose_channel_t*, FILE*);

void la_verbose_channel_add(la_verbose_type_t, 
		la_verbose_level_t, la_verbose_channel_t*);

void la_verbose_channel_add_alltypes(la_verbose_level_t, la_verbose_channel_t*);

void la_verbose_channel_remove(la_verbose_type_t, la_verbose_level_t, 
		la_verbose_channel_t*);

void la_verbose_voutput(la_verbose_channel_t*, char*, va_list ap);

void la_verbose_output(la_verbose_channel_t*, char*, ...);

void la_verbose(la_verbose_type_t, la_verbose_level_t, char*, ...);

#endif /* INCLUDE_LIBADRIFT_VERBOSE_H */
